@@ -27,7 +27,7 @@
 ;;; 4) FIXMEs
 
 (defpackage :sb-introspect
-  (:use "CL")
+  (:use "CL" "SB-KERNEL")
   (:export "ALLOCATION-INFORMATION"
            "FUNCTION-ARGLIST"
            "FUNCTION-LAMBDA-LIST"
@@ -80,9 +80,9 @@ include the pathname of the file and the position of the definition."
 
 (declaim (ftype (sb-int:sfunction (function) debug-info) function-debug-info))
 (defun function-debug-info (function)
-  (let* ((function-object (sb-kernel::%fun-fun function))
-         (function-header (sb-kernel:fun-code-header function-object)))
-    (sb-kernel:%code-debug-info function-header)))
+  (let* ((function-object (%fun-fun function))
+         (function-header (fun-code-header function-object)))
+    (%code-debug-info function-header)))
 
 (declaim (ftype (sb-int:sfunction (function) debug-source) function-debug-source))
 (defun function-debug-source (function)
@@ -107,16 +107,16 @@ FBOUNDP."
 (declaim (inline map-code-constants))
 (defun map-code-constants (code fn)
   "Call FN for each constant in CODE's constant pool."
-  (check-type code sb-kernel:code-component)
-  (loop for i from sb-vm:code-constants-offset below
-       (sb-kernel:code-header-words code)
-     do (funcall fn (sb-kernel:code-header-ref code i))))
+  (check-type code code-component)
+  (loop for i from sb-vm:code-constants-offset below (code-header-words code)
+     do (funcall fn (code-header-ref code i))))
 
 (declaim (inline map-allocated-code-components))
 (defun map-allocated-code-components (spaces fn)
   "Call FN for each allocated code component in one of SPACES.  FN
 receives the object and its size as arguments.  SPACES should be a
-list of the symbols :dynamic, :static, or :read-only."
+list of the symbols :dynamic, :static, :read-only, or :immobile on
+#+immobile-space"
   (apply #'sb-vm::map-allocated-objects
      (lambda (obj header size)
        (when (= sb-vm:code-header-widetag header)
@@ -135,9 +135,8 @@ constant pool."
        (map-code-constants
         obj
         (lambda (constant)
-          (when (and (sb-kernel:fdefn-p constant)
-                     (eq (sb-kernel:fdefn-fun constant)
-                         function))
+          (when (and (fdefn-p constant)
+                     (eq (fdefn-fun constant) function))
             (funcall fn obj))))))))
 
 ;;;; Finding definitions
@@ -170,33 +169,27 @@ constant pool."
   ;; is.
   (description nil :type list))
 
-(defun vop-sources-from-fun-templates (name)
+(defun vops-translating-fun (name)
   (let ((fun-info (sb-int:info :function :info name)))
     (when fun-info
-      (loop for vop in (sb-c::fun-info-templates fun-info)
-            for source = (find-definition-source
-                          (sb-c::vop-info-generator-function vop))
-            do (setf (definition-source-description source)
-                     (if (sb-c::template-note vop)
-                         (list (sb-c::template-name vop)
-                               (sb-c::template-note vop))
-                         (list (sb-c::template-name vop))))
-            collect source))))
+      (sb-c::fun-info-templates fun-info))))
 
 (defun find-vop-source (name)
-  (let* ((templates (vop-sources-from-fun-templates name))
-         (vop (gethash name sb-c::*backend-template-names*))
-         (generator (when vop
-                      (sb-c::vop-info-generator-function vop)))
-         (source (when generator
-                   (find-definition-source generator))))
-    (cond
-      (source
-       (setf (definition-source-description source)
-             (list name))
-       (cons source templates))
-      (t
-       templates))))
+  (let* ((vop (gethash name sb-c::*backend-template-names*))
+         (translating (vops-translating-fun name))
+         (vops (if vop
+                   (cons vop (remove vop translating))
+                   translating)))
+    (loop for vop in vops
+          for name = (sb-c::vop-info-name vop)
+          for loc = (sb-int:info :source-location :vop name)
+          when loc
+          collect (let ((source (translate-source-location loc)))
+                    (setf (definition-source-description source)
+                          (if (sb-c::vop-info-note vop)
+                              (list name (sb-c::vop-info-note vop))
+                              (list name)))
+                    source))))
 
 (defun find-definition-sources-by-name (name type)
   "Returns a list of DEFINITION-SOURCEs for the objects of type TYPE
@@ -342,7 +335,7 @@ If an unsupported TYPE is requested, the function will return NIL.
               (loop for xform in (sb-c::fun-info-transforms fun-info)
                     for source = (find-definition-source
                                   (sb-c::transform-function xform))
-                    for typespec = (sb-kernel:type-specifier
+                    for typespec = (type-specifier
                                     (sb-c::transform-type xform))
                     for note = (sb-c::transform-note xform)
                     do (setf (definition-source-description source)
@@ -374,12 +367,7 @@ If an unsupported TYPE is requested, the function will return NIL.
                             (list name))
                       source))))))
        (:vop
-        (let ((loc (sb-int:info :source-location type name))
-              (translated (find-vop-source name)))
-          (if loc
-              (cons (translate-source-location loc)
-                    translated)
-              translated)))
+        (find-vop-source name))
        (:alien-type
         (let ((loc (sb-int:info :source-location type name)))
           (and loc
@@ -519,7 +507,7 @@ function designator."
         ((typep function 'generic-function)
          (sb-pcl::generic-function-pretty-arglist function))
         (t
-         (sb-kernel:%fun-lambda-list function))))
+         (%fun-lambda-list function))))
 
 (defun deftype-lambda-list (typespec-operator)
   "Returns the lambda list of TYPESPEC-OPERATOR as first return
@@ -531,13 +519,13 @@ value."
                  (sb-int:info :type :expander typespec-operator)))
          (f (if (listp f) (car f) f)))
     (if (functionp f)
-        (values (sb-kernel:%fun-lambda-list f) t)
+        (values (%fun-lambda-list f) t)
         (values nil nil))))
 
 (defun function-type (function-designator)
   "Returns the ftype of FUNCTION-DESIGNATOR, or NIL."
   (flet ((ftype-of (function-designator)
-           (sb-kernel:type-specifier
+           (type-specifier
             (sb-int:proclaimed-ftype function-designator))))
     (etypecase function-designator
       (symbol
@@ -555,8 +543,7 @@ value."
        ;; Give declared type in globaldb priority over derived type
        ;; because it contains more accurate information e.g. for
        ;; struct-accessors.
-       (let ((type (function-type (sb-kernel:%fun-name
-                                   (sb-impl::%fun-fun function-designator)))))
+       (let ((type (function-type (%fun-name (%fun-fun function-designator)))))
          (if type
              type
              (sb-impl::%fun-type function-designator)))))))
@@ -577,36 +564,36 @@ value."
 
 (defun find-function-callees (function)
   "Return functions called by FUNCTION."
-  (declare (sb-kernel:simple-fun function))
+  (declare (simple-fun function))
   (let ((callees '()))
     (map-code-constants
-     (sb-kernel:fun-code-header function)
+     (fun-code-header function)
      (lambda (obj)
-       (when (sb-kernel:fdefn-p obj)
-         (push (sb-kernel:fdefn-fun obj)
-               callees))))
+       (when (fdefn-p obj)
+         (push (fdefn-fun obj) callees))))
     callees))
 
 (defun find-function-callers (function &optional (spaces '(:read-only :static
-                                                           :dynamic)))
+                                                           :dynamic
+                                                           #+immobile-code :immobile)))
   "Return functions which call FUNCTION, by searching SPACES for code objects"
   (let ((referrers '()))
     (map-caller-code-components
      function
      spaces
      (lambda (code)
-       (dotimes (i (sb-kernel:code-n-entries code))
-         (pushnew (sb-kernel:%code-entry-point code i) referrers))))
+       (dotimes (i (code-n-entries code))
+         (pushnew (%code-entry-point code i) referrers))))
     referrers))
 
 ;;; XREF facility
 
 (defun collect-xref (wanted-kind wanted-name)
   (let ((result '()))
-    (sb-c::map-simple-funs
+    (sb-c:map-simple-funs
      (lambda (name fun)
-       (sb-int:binding* ((xrefs (sb-kernel:%simple-fun-xrefs fun) :exit-if-null))
-         (sb-c::map-packed-xref-data
+       (sb-int:binding* ((xrefs (%simple-fun-xrefs fun) :exit-if-null))
+         (sb-c:map-packed-xref-data
           (lambda (xref-kind xref-name xref-form-number)
             (when (and (eq xref-kind wanted-kind)
                        (equal xref-name wanted-name))
@@ -795,55 +782,69 @@ Experimental: interface subject to change."
   ;; scanning threads for negative answers? Similarly, STACK-ALLOCATED-P for
   ;; checking if an object has been stack-allocated by a given thread for
   ;; testing purposes might not come amiss.
-  (if (typep object '(or fixnum character
-                      #.(if (= sb-vm:n-word-bits 64) 'single-float (values))))
+  (if (not (sb-vm:is-lisp-pointer (get-lisp-obj-address object)))
       (values :immediate nil)
       (let ((plist
-             (sb-sys:without-gcing
-               ;; Disable GC so the object cannot move to another page while
-               ;; we have the address.
-               (let* ((addr (sb-kernel:get-lisp-obj-address object))
+             (sb-sys:with-pinned-objects (object)
+               (let* ((addr (get-lisp-obj-address object))
                       (space
-                       (cond ((< sb-vm:read-only-space-start addr
-                                 (ash sb-vm:*read-only-space-free-pointer*
-                                      sb-vm:n-fixnum-tag-bits))
+                       (cond ((< (current-dynamic-space-start) addr
+                                 (sb-sys:sap-int (dynamic-space-free-pointer)))
+                              :dynamic)
+                             #+immobile-space
+                             ((immobile-space-addr-p addr)
+                              :immobile)
+                             ((< sb-vm:read-only-space-start addr
+                                 (sb-sys:sap-int sb-vm:*read-only-space-free-pointer*))
                               :read-only)
                              ((< sb-vm:static-space-start addr
-                                 (ash sb-vm:*static-space-free-pointer*
-                                      sb-vm:n-fixnum-tag-bits))
-                              :static)
-                             ((< (sb-kernel:current-dynamic-space-start) addr
-                                 (sb-sys:sap-int (sb-kernel:dynamic-space-free-pointer)))
-                              :dynamic))))
+                                 (sb-sys:sap-int sb-vm:*static-space-free-pointer*))
+                              :static))))
                  (when space
                    #+gencgc
                    (if (eq :dynamic space)
-                       (let ((index (sb-vm::find-page-index addr)))
-                         (symbol-macrolet ((page (sb-alien:deref sb-vm::page-table index)))
-                           (let* ((flags (sb-alien:slot page 'sb-vm::flags))
-                                  (allocated (ldb (byte 3 0) flags)))
-                             (list :space space
-                                   :generation (sb-alien:slot page 'sb-vm::gen)
-                                   :write-protected (logbitp 3 flags)
-                                   :boxed (logbitp 0 allocated)
-                                   :pinned (logbitp 5 flags)
-                                   :large (logbitp 7 flags)
-                                   :page index))))
+                       (symbol-macrolet ((page (sb-alien:deref sb-vm::page-table index)))
+                         ;; No wonder #+big-endian failed introspection tests-
+                         ;; bits are packed in the opposite order. And thankfully,
+                         ;; this fix seems not to depend on whether the numbering
+                         ;; scheme is MSB 0 or LSB 0, afaict.
+                         (let* ((index (sb-vm::find-page-index addr))
+                                (flags (sb-alien:slot page 'sb-vm::flags))
+                                .
+                                ;; The unused WP-CLR is for ease of counting
+                                #+big-endian
+                                ((type      (ldb (byte 5 3) flags))
+                                 (wp        (logbitp 2 flags))
+                                 (wp-clr    (logbitp 1 flags))
+                                 (dontmove  (logbitp 0 flags)))
+                                #+little-endian
+                                ((type      (ldb (byte 5 0) flags))
+                                 (wp        (logbitp 5 flags))
+                                 (wp-clr    (logbitp 6 flags))
+                                 (dontmove  (logbitp 7 flags))))
+                           (declare (ignore wp-clr))
+                           (list :space space
+                                 :generation (sb-alien:slot page 'sb-vm::gen)
+                                 :write-protected wp
+                                 :boxed (logbitp 0 type)
+                                 :pinned dontmove
+                                 :large (logbitp 4 type)
+                                 :page index)))
                        (list :space space))
                    #-gencgc
                    (list :space space))))))
         (cond (plist
                (values :heap plist))
               (t
-               (let ((sap (sb-sys:int-sap (sb-kernel:get-lisp-obj-address object))))
+               (let ((sap (sb-sys:int-sap (get-lisp-obj-address object))))
                  ;; FIXME: Check other stacks as well.
                  #+sb-thread
                  (dolist (thread (sb-thread:list-all-threads))
-                   (let ((c-start (sb-di::descriptor-sap
+                   (let ((c-start (sb-int:descriptor-sap
                                    (sb-thread::%symbol-value-in-thread
                                     'sb-vm:*control-stack-start*
                                     thread)))
-                         (c-end (sb-di::descriptor-sap
+                         (c-end (sb-int:descriptor-sap
                                  (sb-thread::%symbol-value-in-thread
                                   'sb-vm:*control-stack-end*
                                   thread))))
@@ -879,14 +880,10 @@ Experimental: interface subject to change."
   (let ((fun (coerce function 'function))
         (seen (sb-int:alloc-xset)))
     (flet ((call (part)
-             (when (and (member (sb-kernel:lowtag-of part)
-                                `(,sb-vm:instance-pointer-lowtag
-                                  ,sb-vm:list-pointer-lowtag
-                                  ,sb-vm:fun-pointer-lowtag
-                                  ,sb-vm:other-pointer-lowtag))
+             (when (and (sb-vm:is-lisp-pointer (get-lisp-obj-address part))
                         (not (sb-int:xset-member-p part seen)))
-               (sb-int:add-to-xset part seen)
-               (funcall fun part))))
+                 (sb-int:add-to-xset part seen)
+                 (funcall fun part))))
       (when ext
         (let ((table sb-pcl::*eql-specializer-table*))
           (call (sb-int:with-locked-system-table (table)
@@ -907,9 +904,9 @@ Experimental: interface subject to change."
          (call (realpart object))
          (call (realpart object)))
         (sb-vm::instance
-         (call (sb-kernel:%instance-layout object))
-         (sb-kernel:do-instance-tagged-slot (i object)
-           (call (sb-kernel:%instance-ref object i)))
+         (call (%instance-layout object))
+         (do-instance-tagged-slot (i object)
+           (call (%instance-ref object i)))
          #+sb-thread
          (when (typep object 'sb-thread:thread)
            (cond ((eq object sb-thread:*current-thread*)
@@ -932,42 +929,48 @@ Experimental: interface subject to change."
                              (sb-thread:signal-semaphore sem)))
                           (sb-thread:wait-on-semaphore sem))
                       (sb-thread:interrupt-thread-error ()))
+                    ;; This is whacky - the other thread signals our condition var,
+                    ;; *then* we call the funarg on objects that may no longer
+                    ;; satisfy VALID-LISP-POINTER-P.
+                    ;; And incidentally, we miss any references from TLS indices
+                    ;; that map onto the 'struct thread', which is just as well
+                    ;; since they're either fixnums or dynamic-extent objects.
                     (mapc #'call refs))))))
         (array
          (if (simple-vector-p object)
              (dotimes (i (length object))
                (call (aref object i)))
-             (when (sb-kernel:array-header-p object)
-               (call (sb-kernel:%array-data object))
-               (call (sb-kernel::%array-displaced-p object))
+             (when (array-header-p object)
+               (call (%array-data object))
+               (call (%array-displaced-p object))
                (unless simple
-                 (call (sb-kernel::%array-displaced-from object))))))
-        (sb-kernel:code-component
-         (call (sb-kernel:%code-debug-info object))
-         (loop for i from sb-vm:code-constants-offset
-               below (sb-kernel:code-header-words object)
-               do (call (sb-kernel:code-header-ref object i)))
-         (loop for i below (sb-kernel:code-n-entries object)
-               do (call (sb-kernel:%code-entry-point object i))))
-        (sb-kernel:fdefn
-         (call (sb-kernel:fdefn-name object))
-         (call (sb-kernel:fdefn-fun object)))
-        (sb-kernel:simple-fun
-         (call (sb-kernel:fun-code-header object))
-         (call (sb-kernel:%simple-fun-name object))
-         (call (sb-kernel:%simple-fun-arglist object))
-         (call (sb-kernel:%simple-fun-type object))
-         (call (sb-kernel:%simple-fun-info object)))
-        (sb-kernel:closure
-         (call (sb-kernel:%closure-fun object))
-         (sb-kernel:do-closure-values (x object)
+                 (call (%array-displaced-from object))))))
+        (code-component
+         (call (%code-debug-info object))
+         (loop for i from sb-vm:code-constants-offset below (code-header-words object)
+               do (call (code-header-ref object i)))
+         (loop for i below (code-n-entries object)
+               do (call (%code-entry-point object i))))
+        (fdefn
+         (call (fdefn-name object))
+         (call (fdefn-fun object)))
+        (simple-fun
+         (call (fun-code-header object))
+         (call (%simple-fun-name object))
+         (call (%simple-fun-arglist object))
+         (call (%simple-fun-type object))
+         (call (%simple-fun-info object)))
+        (closure
+         (call (%closure-fun object))
+         (do-closure-values (x object)
            (call x)))
-        (sb-kernel:funcallable-instance
-         (call (sb-kernel:%funcallable-instance-function object))
+        (funcallable-instance
+         ;; FIXME: layout ?
+         (call (%funcallable-instance-function object))
          (loop for i from sb-vm:instance-data-start
-               below (- (1+ (sb-kernel:get-closure-length object))
+               below (- (1+ (get-closure-length object))
                         sb-vm:funcallable-instance-info-offset)
-               do (call (sb-kernel:%funcallable-instance-info object i))))
+               do (call (%funcallable-instance-info object i))))
         (symbol
          (when ext
            (dolist (thread (sb-thread:list-all-threads))
@@ -991,10 +994,21 @@ Experimental: interface subject to change."
          (unless simple
            (call (symbol-package object))))
         (sb-kernel::random-class
-         (case (sb-kernel:widetag-of object)
+         (case (widetag-of object)
            (#.sb-vm:value-cell-widetag
-            (call (sb-kernel:value-cell-ref object)))
+            (call (value-cell-ref object)))
            (t
             (warn "~&MAP-ROOT: Unknown widetag ~S: ~S~%"
-                  (sb-kernel:widetag-of object) object)))))))
+                  (widetag-of object) object)))))))
   object)
+
+(defun object-size (object)
+  (+ (sb-vm::primitive-object-size object)
+     (typecase object
+       (sb-mop:funcallable-standard-object
+        (sb-vm::primitive-object-size
+         (sb-pcl::standard-funcallable-instance-clos-slots object)))
+       (standard-object
+        (sb-vm::primitive-object-size
+         (sb-pcl::standard-instance-slots object)))
+       (t 0))))
