@@ -11,10 +11,10 @@
 ;;; random bootstrap stuff on per-package basis.
 (defun !unintern-init-only-stuff (&aux result)
   (dolist (package (list-all-packages))
-    (sb-int:binding* ((s (find-symbol "!UNINTERN-SYMBOLS" package)
-                         :exit-if-null)
-                      (list (funcall s))
-                      (package (car list)))
+    (sb-int:awhen (find-symbol "!REMOVE-BOOTSTRAP-SYMBOLS" package)
+      (funcall sb-int:it)))
+  (dolist (list sb-impl::*!removable-symbols*)
+    (let ((package (find-package (car list))))
       (dolist (symbol (cdr list))
         (fmakunbound symbol)
         (unintern symbol package))))
@@ -82,7 +82,9 @@
   (let ((count 0))
     (macrolet ((clear-it (place)
                  `(when ,place
-                    (setf ,place nil)
+                    ,(if (typep place '(cons (eql sb-int:info)))
+                         `(sb-int:clear-info ,@(cdr place))
+                         `(setf ,place nil))
                     (incf count))))
       ;; 1. Functions, macros, special operators
       (sb-vm::map-allocated-objects
@@ -110,41 +112,51 @@
     (when (plusp count)
       (format t "~&Removed ~D doc string~:P" count)))
 
-  ;; Share identical FUN-INFOs
-  sb-int::
-  (let ((ht (make-hash-table :test 'equalp))
-        (old-count 0))
-    (sb-int:call-with-each-globaldb-name
-     (lambda (name)
-       (binding* ((info (info :function :info name) :exit-if-null)
-                  (shared-info (gethash info ht info)))
-         (incf old-count)
-         (if (eq info shared-info)
-             (setf (gethash info ht) info)
-           (setf (info :function :info name) shared-info)))))
-    (format t "~&FUN-INFO: Collapsed ~D -> ~D~%"
-            old-count (hash-table-count ht)))
+  ;; Remove source forms of compiled-to-memory lambda expressions.
+  ;; The disassembler is the major culprit for retention of these.
+  (sb-vm::map-allocated-objects
+   (lambda (obj type size)
+     (declare (ignore type size))
+     (when (typep obj 'sb-c::debug-source)
+       (unless (sb-c::debug-source-namestring obj)
+         (setf (sb-c::debug-source-form obj) nil))))
+   :all)
 
-  ;; Share identical FUN-TYPEs.
-  (let ((ht (make-hash-table :test 'equal))
-        (raw-accessor
-         (compile nil '(lambda (f) (sb-vm::%%simple-fun-type f)))))
-    (sb-vm::map-allocated-objects
-     (lambda (obj type size)
-       (declare (ignore type size))
-       (when (sb-kernel:code-component-p obj)
-         (dotimes (i (sb-kernel:code-n-entries obj))
-           (let* ((f (sb-kernel:%code-entry-point obj i))
-                  (type (funcall raw-accessor f)))
-             (setf (sb-kernel:%simple-fun-type f)
-                   (or (gethash type ht) (setf (gethash type ht) type)))))))
-     :all))
-
-  (sb-disassem::!compile-inst-printers)
+  ;; Fix unknown types in globaldb
+  (let ((l nil))
+    (do-all-symbols (s)
+      (flet ((fixup (kind)
+               (multiple-value-bind (type present)
+                   (sb-int:info kind :type s)
+                 (when (and present
+                            (sb-kernel:contains-unknown-type-p type))
+                   (setf (sb-int:info kind :type s)
+                         (sb-kernel:specifier-type (sb-kernel:type-specifier type)))
+                   (push s l)))))
+        (fixup :function)
+        (fixup :variable)))
+    (unless (sb-impl::!c-runtime-noinform-p)
+      (let ((*print-pretty* nil)
+            (*print-length* nil))
+        (format t "~&; Fixed types: ~S~%" (sort l #'string<)))))
 
   ;; Unintern no-longer-needed stuff before the possible PURIFY in
   ;; SAVE-LISP-AND-DIE.
-  #-sb-fluid (!unintern-init-only-stuff)
+  #-(or sb-fluid sb-devel) (!unintern-init-only-stuff)
+
+  ;; Mark interned immobile symbols so that COMPILE-FILE knows
+  ;; which symbols will always be physically in immobile space.
+  ;; Due to the possibility of interning a symbol that was allocated in dynamic
+  ;; space, it's not the case that all interned symbols are immobile.
+  ;; And we can't promise anything across reload, which makes it impossible
+  ;; for x86-64 codegen to know which symbols are immediate constants.
+  ;; Except that symbols which existed at SBCL build time must be.
+  #+(and immobile-space (not immobile-symbols))
+  (do-all-symbols (symbol)
+    (when (sb-kernel:immobile-space-obj-p symbol)
+      (sb-kernel:set-header-data
+           symbol (logior (sb-kernel:get-header-data symbol)
+                          (ash 1 sb-vm::+initial-core-symbol-bit+)))))
 
   ;; A symbol whose INFO slot underwent any kind of manipulation
   ;; such that it now has neither properties nor globaldb info,
@@ -182,13 +194,6 @@
                             :key #'dsd-name)))
   (funcall #'(setf slot-value) 'character dsd 'sb-kernel::default))
 
-;;; Lock internal packages
-#+sb-package-locks
-(dolist (p (list-all-packages))
-  (unless (member p (mapcar #'find-package '("KEYWORD" "CL-USER")))
-    (sb-ext:lock-package p)))
-
-"done with warm.lisp, about to SAVE-LISP-AND-DIE"
 ;;; Even if /SHOW output was wanted during build, it's probably
 ;;; not wanted by default after build is complete. (And if it's
 ;;; wanted, it can easily be turned back on.)
@@ -198,12 +203,6 @@
 ;;; The call to CTYPE-OF-CACHE-CLEAR is probably redundant.
 ;;; SAVE-LISP-AND-DIE calls DEINIT which calls DROP-ALL-HASH-CACHES.
 (sb-kernel::ctype-of-cache-clear)
-(setq sb-c::*flame-on-necessarily-undefined-thing* t)
-
-;;; Clean up stray symbols from the CL-USER package.
-(with-package-iterator (iter "CL-USER" :internal :external)
-  (loop (multiple-value-bind (winp symbol) (iter)
-          (if winp (unintern symbol "CL-USER") (return)))))
 
 ;;; In case there is xref data for internals, repack it here to
 ;;; achieve a more compact encoding.
@@ -215,10 +214,74 @@
 ;;; from the same SBCL build, REPACK-XREF is of no use after the
 ;;; target image has been built.
 #+sb-xref-for-internals (sb-c::repack-xref :verbose 1)
-(with-unlocked-packages (#:sb-c)
-  (fmakunbound 'sb-c::repack-xref))
+(fmakunbound 'sb-c::repack-xref)
+
+(progn
+  (load "src/code/shaketree")
+  (sb-impl::shake-packages
+   ;; Retain all symbols satisfying this predicate
+   #+sb-devel
+   (lambda (symbol accessibility)
+     (declare (ignore accessibility))
+     ;; Retain all symbols satisfying this predicate
+     (or (sb-kernel:symbol-info symbol)
+         (and (boundp symbol) (not (keywordp symbol)))))
+   #-sb-devel
+   (lambda (symbol accessibility)
+     (case (symbol-package symbol)
+      (#.(find-package "SB-VM")
+       (or (eq accessibility :external)
+           ;; overapproximate what we need for contribs and tests
+           (member symbol '(sb-vm::map-referencing-objects
+                            sb-vm::map-stack-references
+                            sb-vm::thread-profile-data-slot
+                            sb-vm::thread-alloc-region-slot
+                            ;; Naughty outside-world code uses this.
+                            sb-vm::thread-control-stack-start-slot
+                            sb-vm::primitive-object-size))
+           (search "-OFFSET" (string symbol))
+           (search "-TN" (string symbol))))
+      ((#.(find-package "SB-C")
+        #.(find-package "SB-ASSEM")
+        #.(find-package "SB-DISASSEM")
+        #.(find-package "SB-IMPL")
+        #.(find-package "SB-KERNEL"))
+       ;; Assume all and only external symbols must be retained
+       (eq accessibility :external))
+      (#.(find-package "SB-FASL")
+       ;; Retain +BACKEND-FASL-FILE-IMPLEMENTATION+ and +FASL-FILE-VERSION+
+       ;; (and anything else otherwise reachable)
+       (and (eq accessibility :external)
+            (constantp symbol)))
+      (#.(find-package "SB-BIGNUM")
+       ;; There are 2 important external symbols for sb-gmp.
+       ;; Other externals can disappear.
+       (member symbol '(sb-bignum:%allocate-bignum
+                        sb-bignum:make-small-bignum)))
+      (t
+       ;; By default, retain any symbol with any attachments
+       (or (sb-kernel:symbol-info symbol)
+           (and (boundp symbol) (not (keywordp symbol)))))))
+   :verbose t :print nil)
+  (unintern 'sb-impl::shake-packages 'sb-impl))
+
+;;; Use historical (stupid) behavior for storing pathname namestrings
+;;; in fasls.
+(setq sb-c::*name-context-file-path-selector* 'truename)
+
+;;; Lock internal packages
+(dolist (p (list-all-packages))
+  (unless (member p (mapcar #'find-package '("KEYWORD" "CL-USER")))
+    (sb-ext:lock-package p)))
+
+;;; Clean up stray symbols from the CL-USER package.
+(with-package-iterator (iter "CL-USER" :internal :external)
+  (loop (multiple-value-bind (winp symbol) (iter)
+          (if winp (unintern symbol "CL-USER") (return)))))
 
 #+immobile-code (setq sb-c::*compile-to-memory-space* :dynamic)
 #+sb-fasteval (setq sb-ext:*evaluator-mode* :interpret)
 ;; See comments in 'readtable.lisp'
 (setf (readtable-base-char-preference *readtable*) :symbols)
+
+"done with warm.lisp, about to SAVE-LISP-AND-DIE"
