@@ -39,6 +39,8 @@
          (*elsewhere* nil)
          (*assembly-optimize* nil)
          (*fixup-notes* nil)
+         (sb!vm::*adjustable-vectors* nil)
+         #!+immobile-code (*code-is-immobile* t)
          #!+inline-constants (*unboxed-constants* nil))
     (unwind-protect
         (let ((*features* (cons :sb-assembling *features*)))
@@ -130,7 +132,9 @@
          ,@code
          ,@(generate-return-sequence
             (or (cadr (assoc :return-style options)) :raw))
-         (emit-alignment sb!vm:n-lowtag-bits))
+         (emit-alignment sb!vm:n-lowtag-bits
+                         ;; EMIT-LONG-NOP does not exist for (not x86-64)
+                         #!+x86-64 :long-nop))
        (when *compile-print*
          (format *error-output* "~S assembled~%" ',name)))))
 
