@@ -329,7 +329,8 @@
                    #.sb!vm:complex-vector-widetag))))
     (make-case)))
 
-(defglobal %%simple-array-n-bits-shifts%% (make-array (1+ sb!vm:widetag-mask)))
+(define-load-time-global %%simple-array-n-bits-shifts%%
+    (make-array (1+ sb!vm:widetag-mask)))
 #.(loop for info across sb!vm:*specialized-array-element-type-properties*
         collect `(setf (aref %%simple-array-n-bits-shifts%% ,(sb!vm:saetp-typecode info))
                        ,(sb!vm:saetp-n-bits-shift info)) into forms
@@ -625,7 +626,8 @@ of specialized arrays is supported."
 ;;; vectors or not simple.
 (macrolet ((def (name table-name)
              `(progn
-                (defglobal ,table-name (make-array ,(1+ sb!vm:widetag-mask)))
+                (define-load-time-global ,table-name
+                    (make-array ,(1+ sb!vm:widetag-mask)))
                 (declaim (type (simple-array function (,(1+ sb!vm:widetag-mask)))
                                ,table-name))
                 (defmacro ,name (array-var)
@@ -750,7 +752,7 @@ of specialized arrays is supported."
                                        ;; a cheaper one.
                                        (declare (optimize (speed 1)
                                                           (safety 1)))
-                                     (the* (,type :context aref) new-value)))
+                                     (the* (,type :context :aref) new-value)))
                   ;; For specialized arrays, the return from
                   ;; data-vector-set would have to be reboxed to be a
                   ;; (Lisp) return value; instead, we use the
@@ -964,7 +966,7 @@ of specialized arrays is supported."
   "Return the type of the elements of the array"
   (let ((widetag (%other-pointer-widetag array))
         (table (load-time-value
-                (let ((table (make-array 256 :initial-element nil)))
+                (let ((table (make-array 256 :initial-element :invalid)))
                   (dotimes (i (length sb!vm:*specialized-array-element-type-properties*) table)
                     (let* ((saetp (aref sb!vm:*specialized-array-element-type-properties* i))
                            (typecode (sb!vm:saetp-typecode saetp))
@@ -973,7 +975,11 @@ of specialized arrays is supported."
                       (aver (typep specifier '(or list symbol)))
                       (setf (aref table typecode) specifier)
                       (when complex-typecode
-                        (setf (aref table complex-typecode) specifier)))))
+                        (setf (aref table complex-typecode) specifier))))
+                  (setf (aref table sb!vm:simple-array-widetag) nil
+                        (aref table sb!vm:complex-vector-widetag) nil
+                        (aref table sb!vm:complex-array-widetag) nil)
+                  table)
                 t)))
     (let ((result (aref table widetag)))
       (if result
@@ -1393,7 +1399,7 @@ of specialized arrays is supported."
                      ,(if (or (eq (sb!vm:saetp-specifier saetp) 'character)
                               #!+sb-unicode
                               (eq (sb!vm:saetp-specifier saetp) 'base-char))
-                          *default-init-char-form*
+                          '(code-char 0)
                           (sb!vm:saetp-initial-element-default saetp))))
                  (remove-if-not
                   #'sb!vm:saetp-specifier
@@ -1750,7 +1756,7 @@ function to be removed without further warning."
            (error 'type-error
                   :datum (first args)
                   :expected-type '(simple-array * (*)))))
-       (!defglobal ,table-name ,(make-array (1+ sb!vm:widetag-mask)))
+       (!define-load-time-global ,table-name ,(make-array (1+ sb!vm:widetag-mask)))
 
        ;; This SUBSTITUTE call happens ** after ** all the SETFs below it.
        ;; DEFGLOBAL's initial value is dumped by genesis as a vector filled
@@ -1796,3 +1802,7 @@ function to be removed without further warning."
                                   :element-type (array-element-type array))
            (array-dimensions array)))
   array)
+
+;;; Why is this needed for ARM and not x86 ???
+(defun allocate-vector (type length words)
+  (allocate-vector type length words))
