@@ -36,7 +36,7 @@
 (defconstant compiled-debug-var-more-count-p           #b00000010)
 (defconstant compiled-debug-var-environment-live       #b00000100)
 (defconstant compiled-debug-var-save-loc-p             #b00001000)
-(defconstant compiled-debug-var-id-p                   #b00010000)
+(defconstant compiled-debug-var-same-name-p            #b00010000)
 (defconstant compiled-debug-var-minimal-p              #b00100000)
 (defconstant compiled-debug-var-deleted-p              #b01000000)
 (defconstant compiled-debug-var-indirect-p             #b10000000)
@@ -99,6 +99,7 @@
   (name (missing-arg) :type (or simple-string cons symbol) :read-only t)
   ;; a description of variable locations for this function, in alphabetical
   ;; order by name; or NIL if no information is available
+  ;; If only one variable is encoded then it's stored as is without a vector.
   ;;
   ;; The variable entries are alphabetically ordered. This ordering is
   ;; used in lifetime info to refer to variables: the first entry is
@@ -113,7 +114,8 @@
   ;;   * the variable ID, when it has one
   ;;   * SC-offset of primary location, if it has one
   ;;   * SC-offset of save location, if it has one
-  (vars nil :type (or simple-vector null))
+  ;; Can either be a single value or a vector for multiple values.
+  (vars nil)
   ;; a vector of the packed binary representation of the
   ;; COMPILED-DEBUG-BLOCKs in this function, in the order that the
   ;; blocks were emitted. The first block is the start of the
@@ -161,7 +163,9 @@
   ;; VARIABLES are all required arguments, and are in the order they
   ;; appear in the VARIABLES vector. In other words, :MINIMAL stands
   ;; in for a vector where every element holds its index.
-  (arguments nil :type (or (simple-array * (*)) (member :minimal nil)))
+  ;;
+  ;; Can either be a single value or a vector for multiple values.
+  (arguments nil)
   ;; There are three alternatives for this slot:
   ;;
   ;; a VECTOR
@@ -177,9 +181,11 @@
   (returns :fixed :type (or (simple-array * (*)) (member :standard :fixed)))
   ;; SC-OFFSETs describing where the return PC and return FP are kept.
   #!-fp-and-pc-standard-save
-  (return-pc (missing-arg) :type sc-offset)
+  (return-pc (missing-arg) :type sc+offset)
   #!-fp-and-pc-standard-save
-  (old-fp (missing-arg) :type sc-offset)
+  (return-pc-pass (missing-arg) :type sc+offset)
+  #!-fp-and-pc-standard-save
+  (old-fp (missing-arg) :type sc+offset)
   ;; An integer which contains between 2 and 4 varint-encoded fields:
   ;; START-PC -
   ;; The earliest PC in this function at which the environment is properly
@@ -190,21 +196,26 @@
   (encoded-locs (missing-arg) :type unsigned-byte :read-only t))
 
 (defun cdf-encode-locs (start-pc elsewhere-pc closure-save
-                        #!+unwind-to-frame-and-call-vop bsp-save)
-  (dx-let ((storage (make-array (* 5 4) :element-type '(unsigned-byte 8))))
-    (let ((bytes (make-array (* 5 4) :fill-pointer 0 :displaced-to storage
-                             :element-type '(unsigned-byte 8))))
-      ;; ELSEWHERE is encoded first to simplify the C backtrace logic,
-      ;; which does not need access to any of the subsequent fields.
-      (write-var-integer elsewhere-pc bytes)
-      (write-var-integer start-pc bytes)
-      #!+unwind-to-frame-and-call-vop
-      (write-var-integer (if bsp-save (1+ bsp-save) 0) bytes)
-      ;; More often the BSP-SAVE is non-null than CLOSURE-SAVE is non-null,
-      ;; so the encoding is potentially smaller with CLOSURE-SAVE being last.
-      (when closure-save
-        (write-var-integer (1+ closure-save) bytes))
-      (integer-from-octets bytes))))
+                        #!+unwind-to-frame-and-call-vop bsp-save
+                        #!-fp-and-pc-standard-save lra-saved-pc
+                        #!-fp-and-pc-standard-save cfp-saved-pc)
+  (dx-let ((bytes (make-array (* 5 4) :fill-pointer 0
+                                      :element-type '(unsigned-byte 8))))
+    ;; ELSEWHERE is encoded first to simplify the C backtrace logic,
+    ;; which does not need access to any of the subsequent fields.
+    (write-var-integer elsewhere-pc bytes)
+    (write-var-integer start-pc bytes)
+    #!+unwind-to-frame-and-call-vop
+    (write-var-integer (if bsp-save (1+ bsp-save) 0) bytes)
+    #!-fp-and-pc-standard-save
+    (progn
+      (write-var-integer lra-saved-pc bytes)
+      (write-var-integer cfp-saved-pc bytes))
+    ;; More often the BSP-SAVE is non-null than CLOSURE-SAVE is non-null,
+    ;; so the encoding is potentially smaller with CLOSURE-SAVE being last.
+    (when closure-save
+      (write-var-integer (1+ closure-save) bytes))
+    (integer-from-octets bytes)))
 
 (defun cdf-decode-locs (cdf)
   (let ((encoding (compiled-debug-fun-encoded-locs cdf))
@@ -221,18 +232,32 @@
             #!+unwind-to-frame-and-call-vop
             ;; 0 -> NULL, 1 -> 0, ...
             (bsp-save (let ((i (decode-varint))) (unless (zerop i) (1- i))))
+            #!-fp-and-pc-standard-save
+            (lra-saved-pc (decode-varint))
+            #!-fp-and-pc-standard-save
+            (cfp-saved-pc (decode-varint))
             (closure-save (let ((i (decode-varint))) (unless (zerop i) (1- i)))))
         (values start-pc elsewhere-pc closure-save
+                #!-fp-and-pc-standard-save lra-saved-pc
+                #!-fp-and-pc-standard-save cfp-saved-pc
                 #!+unwind-to-frame-and-call-vop bsp-save)))))
 
-(macrolet ((def (index name)
-             `(defun ,name (cdf)
-                (nth-value ,index (cdf-decode-locs cdf)))))
-  (def 0 compiled-debug-fun-start-pc)
-  (def 1 compiled-debug-fun-elsewhere-pc)
-  ;; Most compiled-debug-funs don't need these
-  (def 2 compiled-debug-fun-closure-save)
-  #!+unwind-to-frame-and-call-vop (def 3 compiled-debug-fun-bsp-save))
+(macrolet ((def (&rest names)
+             `(progn
+                ,@(loop
+                     for name in names
+                     for index from 0
+                     collect
+                       `(defun ,name (cdf)
+                          (nth-value ,index (cdf-decode-locs cdf)))))))
+  (def
+    compiled-debug-fun-start-pc
+    compiled-debug-fun-elsewhere-pc
+    ;; Most compiled-debug-funs don't need these
+    compiled-debug-fun-closure-save
+    #!-fp-and-pc-standard-save compiled-debug-fun-lra-saved-pc
+    #!-fp-and-pc-standard-save compiled-debug-fun-cfp-saved-pc
+    #!+unwind-to-frame-and-call-vop compiled-debug-fun-bsp-save))
 
 (def!struct (compiled-debug-fun-optional (:include compiled-debug-fun)
                                          #-sb-xc-host (:pure t)
@@ -300,11 +325,11 @@
 ;;;     If variables are dumped (level 1), then the variables are all
 ;;;     arguments (in order) with the minimal-arg bit set.
 ;;;    [If returns is specified, then the number of return values]
-;;;    [...sequence of var-length ints holding sc-offsets of the return
+;;;    [...sequence of var-length ints holding SC+OFFSETs of the return
 ;;;     value locations, if fixed return values are specified.]
-;;;    return-pc location sc-offset (as var-length int)
-;;;    old-fp location sc-offset (as var-length int)
-;;;    [nfp location sc-offset (as var-length int), if nfp flag]
+;;;    return-pc location SC+OFFSET (as var-length int)
+;;;    old-fp location SC+OFFSET (as var-length int)
+;;;    [nfp location SC+OFFSET (as var-length int), if nfp flag]
 ;;;    code-start-pc (as a var-length int)
 ;;;     This field implicitly encodes start of this function's code in the
 ;;;     function map, as a delta from the previous function's code start.
@@ -347,7 +372,7 @@
 
   ;; For functions processed by EVAL (including EVAL-WHEN and LOAD on
   ;; a source file), the source form.
-  (form nil :type list :read-only t)
+  (form nil :type list :read-only nil)
   ;; This is the function whose source is the form.
   (function nil :read-only t)
 
@@ -368,6 +393,7 @@
 
 (def!struct (compiled-debug-info
              (:include debug-info)
+             (:copier nil)
              #-sb-xc-host (:pure t))
   ;; a SIMPLE-VECTOR of alternating DEBUG-FUN objects and fixnum
   ;; PCs, used to map PCs to functions, so that we can figure out what
@@ -383,20 +409,11 @@
   ;; WHN 20000120
   (fun-map (missing-arg) :type simple-vector :read-only t)
   ;; Location contexts
-  (contexts nil :type (or simple-vector null) :read-only t)
+  ;; Either a simple-vector or a context if there's only one context.
+  (contexts nil :type t :read-only t)
   (tlf-number nil :type (or index null))
   (char-offset nil :type (or index null)))
 
-(defvar *!initial-debug-sources*)
-
-(defun !debug-info-cold-init ()
-  (let ((now (get-universal-time)))
-    (dolist (debug-source *!initial-debug-sources*)
-      (let* ((namestring (debug-source-namestring debug-source))
-             (timestamp (file-write-date namestring)))
-        (setf (debug-source-created debug-source) timestamp
-              (debug-source-compiled debug-source) now)))))
-
 ;;;; file reading
 ;;;;
 ;;;; When reading from a file, we have to keep track of some source
@@ -422,8 +439,8 @@
   (external-format nil  :read-only t)
   ;; the defaulted, but not necessarily absolute file name (i.e. prior
   ;; to TRUENAME call.) Null if not a file. This is used to set
-  ;; *COMPILE-FILE-PATHNAME*, and if absolute, is dumped in the
-  ;; debug-info.
+  ;; *COMPILE-FILE-PATHNAME*, and if absolute (a harmful constraint to be sure),
+  ;; is dumped in the debug-info.
   (untruename nil :type (or pathname null) :read-only t)
   ;; the file's write date (if relevant)
   (write-date nil :type (or unsigned-byte null)  :read-only t)
