@@ -703,8 +703,8 @@ line break."
 
 ;;;; pprint-dispatch tables
 
-(defglobal *standard-pprint-dispatch-table* nil)
-(defglobal *initial-pprint-dispatch-table* nil)
+(define-load-time-global *standard-pprint-dispatch-table* nil)
+(define-load-time-global *initial-pprint-dispatch-table* nil)
 
 (defstruct (pprint-dispatch-entry
             (:constructor make-pprint-dispatch-entry (type priority fun test-fn))
@@ -749,7 +749,6 @@ line break."
 ;; This used to involve rewriting into a sexpr if CONS was involved,
 ;; since it was not an official specifier. But now it is.
 (defun compute-test-fn (ctype type-spec function)
-  (declare (special sb!c::*backend-type-predicates*))
   ;; Avoid compiling code for an existing structure predicate
   (or (and (eq (info :type :kind type-spec) :instance)
            (let ((layout (info :type :compiler-layout type-spec)))
@@ -1231,7 +1230,7 @@ line break."
 ;;;        puts a newline in between INTO and COUNT.
 ;;;        It would be awesome to have code in common with the macro
 ;;;        the properly represents each clauses.
-(defglobal *loop-separating-clauses*
+(defconstant-eqx +loop-separating-clauses+
   '(:and
     :with :for
     :initially :finally
@@ -1245,7 +1244,8 @@ line break."
     :minimize :minimizing
     :if :when :unless :end
     :for :while :until :repeat :always :never :thereis
-    ))
+    )
+  #'equal)
 
 (defun pprint-extended-loop (stream list)
   (pprint-logical-block (stream list :prefix "(" :suffix ")")
@@ -1258,7 +1258,7 @@ line break."
     (write-char #\space stream)
     (loop for thing = (pprint-pop)
           when (and (symbolp thing)
-                    (member thing  *loop-separating-clauses* :test #'string=))
+                    (member thing +loop-separating-clauses+ :test #'string=))
           do (pprint-newline :mandatory stream)
           do (output-object thing stream)
           do (pprint-exit-if-list-exhausted)
