@@ -57,11 +57,11 @@
 variable: an unreadable object representing the error is printed instead.")
 
 ;; duplicate defglobal because this file is compiled before "reader"
-(defglobal *standard-readtable* nil)
+(define-load-time-global *standard-readtable* nil)
 
 (defun %with-standard-io-syntax (function)
   (declare (type function function))
-  (let ((*package* (find-package "COMMON-LISP-USER"))
+  (let ((*package* (load-time-value (find-package "COMMON-LISP-USER") t))
         (*print-array* t)
         (*print-base* 10)
         (*print-case* :upcase)
@@ -82,7 +82,8 @@ variable: an unreadable object representing the error is printed instead.")
         (*read-eval* t)
         (*read-suppress* nil)
         (*readtable* *standard-readtable*)
-        (*suppress-print-errors* nil))
+        (*suppress-print-errors* nil)
+        (*print-vector-length* nil))
     (funcall function)))
 
 ;;;; routines to print objects
@@ -308,29 +309,26 @@ variable: an unreadable object representing the error is printed instead.")
                    (output-ugly-object stream object))))
            (handle-it (stream)
              (if *suppress-print-errors*
-                 (handler-bind ((condition
-                                  (lambda (condition) nil
-                                    (when (typep condition *suppress-print-errors*)
-                                      (cond (*in-print-error*
-                                             (write-string "(error printing " stream)
-                                             (write-string *in-print-error* stream)
-                                             (write-string ")" stream))
-                                            (t
-                                             ;; Give outer handlers a chance.
-                                             (with-simple-restart
-                                                 (continue "Suppress the error.")
-                                               (signal condition))
-                                             (let ((*print-readably* nil)
-                                                   (*print-escape* t))
-                                               (write-string
-                                                "#<error printing a " stream)
-                                               (let ((*in-print-error* "type"))
-                                                 (output-object (type-of object) stream))
-                                               (write-string ": " stream)
-                                               (let ((*in-print-error* "condition"))
-                                                 (output-object condition stream))
-                                               (write-string ">" stream))))
-                                      (return-from handle-it object)))))
+                 (handler-bind
+                     ((condition
+                       (lambda (condition)
+                         (when (typep condition *suppress-print-errors*)
+                           (cond (*in-print-error*
+                                  (write-string "(error printing " stream)
+                                  (write-string *in-print-error* stream)
+                                  (write-string ")" stream))
+                                 (t
+                                  (let ((*print-readably* nil)
+                                        (*print-escape* t))
+                                    (write-string
+                                     "#<error printing a " stream)
+                                    (let ((*in-print-error* "type"))
+                                      (output-object (type-of object) stream))
+                                    (write-string ": " stream)
+                                    (let ((*in-print-error* "condition"))
+                                      (output-object condition stream))
+                                    (write-string ">" stream))))
+                           (return-from handle-it object)))))
                    (print-it stream))
                  (print-it stream)))
            (check-it (stream)
@@ -373,7 +371,8 @@ variable: an unreadable object representing the error is printed instead.")
       ;; Additionally, if the object is an obsolete CONDITION, don't crash.
       ;; (There is no update-instance protocol for conditions)
       (when (or (sb!kernel::undefined-classoid-p classoid)
-                (and (layout-invalid layout) (condition-classoid-p classoid)))
+                (and (layout-invalid layout)
+                     (logtest (layout-%flags layout) +condition-layout-flag+)))
         ;; not only is this unreadable, it's unprintable too.
         (return-from output-ugly-object
           (print-unreadable-object (object stream :identity t)
@@ -479,7 +478,7 @@ variable: an unreadable object representing the error is printed instead.")
 
 ;;; LETTER-ATTRIBUTE is a local of SYMBOL-QUOTEP. It matches letters
 ;;; that don't need to be escaped (according to READTABLE-CASE.)
-(defparameter *attribute-names*
+(defglobal *attribute-names*
   `((number . number-attribute) (lowercase . lowercase-attribute)
     (uppercase . uppercase-attribute) (letter . letter-attribute)
     (sign . sign-attribute) (extension . extension-attribute)
@@ -825,35 +824,49 @@ variable: an unreadable object representing the error is printed instead.")
 
 (defmethod print-object ((vector vector) stream)
   (let ((readably *print-readably*))
-    (cond ((stringp vector)
-           (cond ((and readably (not (typep vector '(vector character))))
-                  (output-unreadable-array-readably vector stream))
-                 ((or *print-escape* readably)
-                  (write-char #\" stream)
-                  (quote-string vector stream)
-                  (write-char #\" stream))
-                 (t
-                  (write-string vector stream))))
-          ((not (or *print-array* readably))
-           (output-terse-array vector stream))
-          ((bit-vector-p vector)
-           (write-string "#*" stream)
-           (dovector (bit vector)
-             ;; (Don't use OUTPUT-OBJECT here, since this code
-             ;; has to work for all possible *PRINT-BASE* values.)
-             (write-char (if (zerop bit) #\0 #\1) stream)))
-          ((or (not readably) (array-readably-printable-p vector))
-           (descend-into (stream)
-             (write-string "#(" stream)
-             (dotimes (i (length vector))
-               (unless (zerop i)
-                 (write-char #\space stream))
-               (punt-print-if-too-long i stream)
-               (output-object (aref vector i) stream))
-             (write-string ")" stream)))
+    (flet ((cut-length ()
+             (when (and (not readably)
+                        *print-vector-length*
+                        (> (length vector) *print-vector-length*))
+               (print-unreadable-object (vector stream :type t :identity t)
+                 (format stream "~A..."
+                         (make-array *print-vector-length*
+                                     :element-type (array-element-type vector)
+                                     :displaced-to vector)))
+               t)))
+      (cond ((stringp vector)
+             (cond ((and readably (not (typep vector '(vector character))))
+                    (output-unreadable-array-readably vector stream))
+                   ((and *print-escape*
+                         (cut-length)))
+                   ((or *print-escape* readably)
+                    (write-char #\" stream)
+                    (quote-string vector stream)
+                    (write-char #\" stream))
+                   (t
+                    (write-string vector stream))))
+            ((not (or *print-array* readably))
+             (output-terse-array vector stream))
+            ((bit-vector-p vector)
+             (cond ((cut-length))
+                   (t
+                    (write-string "#*" stream)
+                    (dovector (bit vector)
+                      ;; (Don't use OUTPUT-OBJECT here, since this code
+                      ;; has to work for all possible *PRINT-BASE* values.)
+                      (write-char (if (zerop bit) #\0 #\1) stream)))))
+            ((or (not readably) (array-readably-printable-p vector))
+             (descend-into (stream)
+               (write-string "#(" stream)
+               (dotimes (i (length vector))
+                 (unless (zerop i)
+                   (write-char #\space stream))
+                 (punt-print-if-too-long i stream)
+                 (output-object (aref vector i) stream))
+               (write-string ")" stream)))
 
-          (t
-           (output-unreadable-array-readably vector stream)))))
+            (t
+             (output-unreadable-array-readably vector stream))))))
 
 ;;; This function outputs a string quoting characters sufficiently
 ;;; so that someone can read it in again. Basically, put a slash in
@@ -997,7 +1010,7 @@ variable: an unreadable object representing the error is printed instead.")
 ;;;
 ;;; It doesn't need a lock, but if you work on SCRUB-POWER-CACHE or
 ;;; POWERS-FOR-BASE, see that you don't break the assumptions!
-(defglobal *power-cache* (make-array 37 :initial-element nil))
+(define-load-time-global *power-cache* (make-array 37 :initial-element nil))
 (declaim (type (simple-vector 37) *power-cache*))
 
 (defconstant +power-cache-integer-length-limit+ 2048)
@@ -1172,9 +1185,6 @@ variable: an unreadable object representing the error is printed instead.")
 
 (defun flonum-to-string (x &optional width fdigits scale fmin)
   (declare (type float x))
-  ;; FIXME: I think only FORMAT-DOLLARS calls FLONUM-TO-STRING with
-  ;; possibly-negative X.
-  (setf x (abs x))
   (multiple-value-bind (e string)
       (if fdigits
           (flonum-to-digits x (min (- (+ fdigits (or scale 0)))
@@ -1606,6 +1616,8 @@ variable: an unreadable object representing the error is printed instead.")
     (let ((dinfo (%code-debug-info component)))
       (cond ((eq dinfo :bogus-lra)
              (write-string "bogus code object" stream))
+            ((functionp dinfo)
+             (format stream "trampoline ~S" dinfo))
             (t
              (format stream "code object [~D]" (code-n-entries component))
              (let ((fun-name (awhen (%code-entry-point component 0)
@@ -1647,16 +1659,18 @@ variable: an unreadable object representing the error is printed instead.")
                 (values "#.(~S #X~16,'0X #X~16,'0X)"
                         '%make-simd-pack-ub64 #'%simd-pack-ub64s)))
            (multiple-value-call
-            #'format stream format maker (funcall extractor pack))))
+               #'format stream format maker (funcall extractor pack))))
+        (*print-readably*
+         (print-not-readable-error pack stream))
         (t
          (print-unreadable-object (pack stream)
            (flet ((all-ones-p (value start end &aux (mask (- (ash 1 end) (ash 1 start))))
-                      (= (logand value mask) mask))
+                    (= (logand value mask) mask))
                   (split-num (value start)
-                      (loop
-                         for i from 0 to 3
-                         and v = (ash value (- start)) then (ash v -8)
-                         collect (logand v #xFF))))
+                    (loop
+                       for i from 0 to 3
+                       and v = (ash value (- start)) then (ash v -8)
+                       collect (logand v #xFF))))
              (multiple-value-bind (low high)
                  (%simd-pack-ub64s pack)
                (etypecase pack
@@ -1696,6 +1710,9 @@ variable: an unreadable object representing the error is printed instead.")
 
 ;;;; catch-all for unknown things
 
+(declaim (inline lowtag-of))
+(defun lowtag-of (x) (logand (get-lisp-obj-address x) sb!vm:lowtag-mask))
+
 (defmethod print-object ((object t) stream)
   (flet ((output-it (stream)
           (when (eq object sb!pcl:+slot-unbound+)
@@ -1713,6 +1730,10 @@ variable: an unreadable object representing the error is printed instead.")
                       (#.sb!vm:value-cell-widetag
                        (write-string "value cell " stream)
                        (output-object (value-cell-ref object) stream))
+                      (#.sb!vm:filler-widetag
+                       (write-string "pad " stream)
+                       (write (1+ (get-header-data object)) :stream stream)
+                       (write-string "w" stream)) ; words
                       (t
                        (write-string "unknown pointer object, widetag=" stream)
                        (output-integer widetag stream 16 t)))))
