@@ -63,6 +63,16 @@
   (:results (res :scs (any-reg descriptor-reg)))
   (:generator 1
     (move bsp-tn res)))
+
+(define-vop (current-nsp)
+  (:results (res :scs (any-reg descriptor-reg)))
+  (:generator 1
+    (move res nsp-tn)))
+
+(define-vop (set-nsp)
+  (:args (nsp :scs (any-reg descriptor-reg)))
+  (:generator 1
+    (move nsp-tn nsp)))
 
 ;;;; unwind block hackery
 
