@@ -58,6 +58,27 @@
                  (list location)))
     location))
 
+(defun note-this-location (vop kind)
+  "NOTE-THIS-LOCATION VOP Kind
+  Note that the current code location is an interesting (to the debugger)
+  location of the specified Kind. VOP is the VOP responsible for this code.
+  This VOP must specify some non-null :SAVE-P value (perhaps :COMPUTE-ONLY) so
+  that the live set is computed."
+  (let ((lab (gen-label)))
+    (emit-label lab)
+    (note-debug-location vop lab kind *location-context*)))
+
+(defun note-next-instruction (vop kind)
+  "NOTE-NEXT-INSTRUCTION VOP Kind
+   Similar to NOTE-THIS-LOCATION, except the use the location of the next
+   instruction for the code location, wherever the scheduler decided to put
+   it."
+  (let ((loc (note-debug-location vop nil kind)))
+    (emit-postit (lambda (segment posn)
+                   (declare (ignore segment))
+                   (setf (location-info-label loc) posn))))
+  (values))
+
 #!-sb-fluid (declaim (inline ir2-block-physenv))
 (defun ir2-block-physenv (2block)
   (declare (type ir2-block 2block))
@@ -90,10 +111,10 @@
 ;;; variables dumped, compute a bit-vector representing the set of
 ;;; live variables. If the TN is environment-live, we only mark it as
 ;;; live when it is in scope at NODE.
+(declaim (ftype (sfunction (local-tn-bit-vector node ir2-block hash-table (or null vop))
+                           simple-bit-vector)
+                compute-live-vars))
 (defun compute-live-vars (live node block var-locs vop)
-  (declare (type ir2-block block) (type local-tn-bit-vector live)
-           (type hash-table var-locs) (type node node)
-           (type (or vop null) vop))
   (let ((res (make-array (logandc2 (+ (hash-table-count var-locs) 7) 7)
                          :element-type 'bit
                          :initial-element 0))
@@ -134,9 +155,8 @@
 (defun decode-restart-location (x)
   (declare (fixnum x))
   (let ((registers-size #.(integer-length (sb-size (sb-or-lose 'sb!vm::registers)))))
-    (values (make-sc-offset
-             (sc-number-or-lose 'sb!vm::descriptor-reg)
-             (ldb (byte registers-size 0) x))
+    (values (make-sc+offset sb!vm:descriptor-reg-sc-number
+                            (ldb (byte registers-size 0) x))
             (ash x (- registers-size)))))
 
 ;;; Dump a compiled debug-location into *BYTE-BUFFER* that describes
@@ -255,7 +275,6 @@
 
     (when elsewhere-locations
       (dolist (loc (nreverse elsewhere-locations))
-        (push loc locations)
         (dump-location-from-info loc var-locs)))
     (let ((compressed
            (lz-compress (coerce byte-buffer
@@ -271,11 +290,13 @@
   (let ((file-info (get-toplevelish-file-info info)))
     (make-debug-source
      :compiled (source-info-start-time info)
-
      :namestring (or *source-namestring*
                      (make-file-info-namestring
-                      (if (pathnamep (file-info-name file-info))
-                          (file-info-name file-info))
+                      (let ((pathname
+                             (case *name-context-file-path-selector*
+                               (pathname (file-info-untruename file-info))
+                               (truename (file-info-name file-info)))))
+                        (if (pathnamep pathname) pathname))
                       file-info))
      :created (file-info-write-date file-info)
      :form (when function
@@ -332,13 +353,21 @@
                                                                 (1- max-negative))
                                                            (plusp max-negative))
                                     seq))))))
+
+(defun compact-vector (sequence)
+  (cond ((and (= (length sequence) 1)
+              (not (typep (elt sequence 0) '(and vector
+                                             (not string)))))
+         (elt sequence 0))
+        (t
+         (coerce-to-smallest-eltype sequence))))
 
 ;;;; variables
 
-;;; Return a SC-OFFSET describing TN's location.
-(defun tn-sc-offset (tn)
+;;; Return a SC+OFFSET describing TN's location.
+(defun tn-sc+offset (tn)
   (declare (type tn tn))
-  (make-sc-offset (sc-number (tn-sc tn))
+  (make-sc+offset (sc-number (tn-sc tn))
                   (tn-offset tn)))
 
 (defun lambda-ancestor-p (maybe-ancestor maybe-descendant)
@@ -360,8 +389,8 @@
 ;;; environment live and is an argument. If a :DEBUG-ENVIRONMENT TN,
 ;;; then we also exclude set variables, since the variable is not
 ;;; guaranteed to be live everywhere in that case.
-(defun dump-1-var (fun var tn id minimal buffer)
-  (declare (type lambda-var var) (type (or tn null) tn) (type index id)
+(defun dump-1-var (fun var tn minimal buffer &optional same-name-p)
+  (declare (type lambda-var var) (type (or tn null) tn)
            (type clambda fun))
   (let* ((name (leaf-debug-name var))
          (save-tn (and tn (tn-save-tn tn)))
@@ -397,22 +426,24 @@
         (:more-count
          (setq flags (logior flags compiled-debug-var-more-count-p)
                more t))))
-    (unless (or more (zerop id) minimal)
-      (setq flags (logior flags compiled-debug-var-id-p)))
-
-    #!+64-bit
+    (when (and same-name-p
+               (not (or more minimal)))
+      (setf flags (logior flags compiled-debug-var-same-name-p)))
+    #!+64-bit ; FIXME: fails if SB-VM:N-FIXNUM-TAG-BITS is 3
+              ; which early-vm.lisp claims to work
     (cond (indirect
-           (setf (ldb (byte 27 8) flags) (tn-sc-offset tn))
+           (setf (ldb (byte 27 8) flags) (tn-sc+offset tn))
            (when save-tn
-             (setf (ldb (byte 27 35) flags) (tn-sc-offset save-tn))))
+             (setf (ldb (byte 27 35) flags) (tn-sc+offset save-tn))))
           (t
            (if (and tn (tn-offset tn))
-               (setf (ldb (byte 27 8) flags) (tn-sc-offset tn))
+               (setf (ldb (byte 27 8) flags) (tn-sc+offset tn))
                (aver minimal))
            (when save-tn
-             (setf (ldb (byte 27 35) flags) (tn-sc-offset save-tn)))))
+             (setf (ldb (byte 27 35) flags) (tn-sc+offset save-tn)))))
     (vector-push-extend flags buffer)
     (unless (or minimal
+                same-name-p
                 more) ;; &more vars need no name
       ;; Dumping uninterned symbols as debug var names is kinda silly.
       ;; Reconstruction of the name on fasl load produces a new gensym anyway.
@@ -420,9 +451,7 @@
       ;; and PARSE-COMPILED-DEBUG-VARS can create the interned symbol.
       ;; This reduces core size by omitting zillions of symbols whose names
       ;; are spelled the same.
-      (vector-push-extend (if (symbol-package name) name (string name)) buffer)
-      (unless (zerop id)
-        (vector-push-extend id buffer)))
+      (vector-push-extend (if (symbol-package name) name (string name)) buffer))
 
     (cond (indirect
            ;; Indirect variables live in the parent frame, and are
@@ -430,18 +459,18 @@
            ;; The first one/two sc-offsets are for the frame pointer,
            ;; the third is for the stack offset.
            #!-64-bit
-           (vector-push-extend (tn-sc-offset tn) buffer)
+           (vector-push-extend (tn-sc+offset tn) buffer)
            #!-64-bit
            (when save-tn
-             (vector-push-extend (tn-sc-offset save-tn) buffer))
-           (vector-push-extend (tn-sc-offset (leaf-info var)) buffer))
+             (vector-push-extend (tn-sc+offset save-tn) buffer))
+           (vector-push-extend (tn-sc+offset (leaf-info var)) buffer))
           #!-64-bit
           (t
            (if (and tn (tn-offset tn))
-               (vector-push-extend (tn-sc-offset tn) buffer)
+               (vector-push-extend (tn-sc+offset tn) buffer)
                (aver minimal))
            (when save-tn
-             (vector-push-extend (tn-sc-offset save-tn) buffer)))))
+             (vector-push-extend (tn-sc+offset save-tn) buffer)))))
   (values))
 
 ;;; Return a vector suitable for use as the DEBUG-FUN-VARS
@@ -474,27 +503,20 @@
                         :key (lambda (x)
                                (symbol-name (leaf-debug-name (car x))))))
           (prev-name nil)
-          (id 0)
           (i 0)
           (buffer (make-array 0 :fill-pointer 0 :adjustable t))
           ;; XEPs don't have any useful variables
           (minimal (eq (functional-kind fun) :external)))
-      (declare (type (or simple-string null) prev-name)
-               (type index id i))
+      (declare (type index i))
       (dolist (x sorted)
         (let* ((var (car x))
-               (name (symbol-name (leaf-debug-name var))))
-          (cond ((and prev-name (string= prev-name name))
-                 (incf id))
-                (t
-                 (setq id 0  prev-name name)))
-          (dump-1-var fun var (cdr x) id minimal buffer)
+               (name (leaf-debug-name var)))
+          (dump-1-var fun var (cdr x) minimal buffer
+                      (and prev-name (eq prev-name name)))
+          (setf prev-name name)
           (setf (gethash var var-locs) i)
           (incf i)))
-      ;; The vector can be marked as readonly so that compilation into memory
-      ;; is permitted to coalesce when saving the image.
-      ;; Otherwise only COMPILE-FILE would coalesce these.
-      (logically-readonlyize (coerce buffer 'simple-vector)))))
+      (compact-vector buffer))))
 
 ;;; Return a vector suitable for use as the DEBUG-FUN-VARS of
 ;;; FUN, representing the arguments to FUN in minimal variable format.
@@ -502,9 +524,8 @@
   (declare (type clambda fun))
   (let ((buffer (make-array 0 :fill-pointer 0 :adjustable t)))
     (dolist (var (lambda-vars fun))
-      (dump-1-var fun var (leaf-info var) 0 t buffer))
-    ;; See remark in COMPUTE-VARS about LOGICALLY-READONLYIZE here.
-    (logically-readonlyize (coerce buffer 'simple-vector))))
+      (dump-1-var fun var (leaf-info var) t buffer))
+    (compact-vector buffer)))
 
 ;;; Return VAR's relative position in the function's variables (determined
 ;;; from the VAR-LOCS hashtable).  If VAR is deleted, then return DEBUG-INFO-VAR-DELETED.
@@ -566,13 +587,13 @@
           (dolist (var (lambda-vars fun))
             (res (debug-location-for var var-locs)))))
 
-    (coerce-to-smallest-eltype (res))))
+    (compact-vector (res))))
 
 ;;; Return a vector of SC offsets describing FUN's return locations.
 ;;; (Must be known values return...)
 (defun compute-debug-returns (fun)
   (coerce-to-smallest-eltype
-   (mapcar #'tn-sc-offset
+   (mapcar #'tn-sc+offset
            (return-info-locations (tail-set-info (lambda-tail-set fun))))))
 
 ;;;; debug functions
@@ -589,7 +610,7 @@
          (name (if (consp name)
                    (case (car name)
                      ((xep tl-xep)
-                      (assert (eq kind :external))
+                      (aver (eq kind :external))
                       (second name))
                      (&optional-processor
                       (setf kind :optional)
@@ -603,18 +624,24 @@
     (funcall (compiled-debug-fun-ctor kind)
              :name name
              #!-fp-and-pc-standard-save :return-pc
-             #!-fp-and-pc-standard-save (tn-sc-offset (ir2-physenv-return-pc 2env))
+             #!-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc 2env))
+             #!-fp-and-pc-standard-save :return-pc-pass
+             #!-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-return-pc-pass 2env))
              #!-fp-and-pc-standard-save :old-fp
-             #!-fp-and-pc-standard-save (tn-sc-offset (ir2-physenv-old-fp 2env))
+             #!-fp-and-pc-standard-save (tn-sc+offset (ir2-physenv-old-fp 2env))
              :encoded-locs
              (cdf-encode-locs
               (label-position (ir2-physenv-environment-start 2env))
               (label-position (ir2-physenv-elsewhere-start 2env))
               (when (ir2-physenv-closure-save-tn 2env)
-                (tn-sc-offset (ir2-physenv-closure-save-tn 2env)))
+                (tn-sc+offset (ir2-physenv-closure-save-tn 2env)))
               #!+unwind-to-frame-and-call-vop
               (when (ir2-physenv-bsp-save-tn 2env)
-                (tn-sc-offset (ir2-physenv-bsp-save-tn 2env)))))))
+                (tn-sc+offset (ir2-physenv-bsp-save-tn 2env)))
+              #!-fp-and-pc-standard-save
+              (label-position (ir2-physenv-lra-saved-pc 2env))
+              #!-fp-and-pc-standard-save
+              (label-position (ir2-physenv-cfp-saved-pc 2env))))))
 
 ;;; Return a complete C-D-F structure for FUN. This involves
 ;;; determining the DEBUG-INFO level and filling in optional slots as
@@ -701,7 +728,15 @@
     (let* ((sorted (sort dfuns #'< :key #'car))
            (fun-map (compute-debug-fun-map sorted)))
       (make-compiled-debug-info
-       :name (component-name component)
+       ;; COMPONENT-NAME is often not useful, and sometimes completely fubar.
+       ;; Function names, on the other hand, are seldom unhelpful,
+       ;; so if there's only one function, pick that as the component name.
+       ;; Otherwise preserve whatever crummy name was already assigned.
+       :name (let* ((2comp (component-info component))
+                    (entries (sb!c::ir2-component-entries 2comp)))
+               (or (and (not (cdr entries))
+                        (sb!c::entry-info-name (car entries)))
+                   (component-name component)))
        :fun-map fun-map
        :tlf-number component-tlf-num
        :char-offset
@@ -709,9 +744,7 @@
             (aref (file-info-positions
                    (source-info-file-info *source-info*))
                   component-tlf-num))
-       :contexts
-       (and (plusp (length *contexts*))
-            (coerce *contexts* 'simple-vector))))))
+       :contexts (compact-vector *contexts*)))))
 
 ;;; Write BITS out to BYTE-BUFFER in backend byte order. The length of
 ;;; BITS must be evenly divisible by eight.
