@@ -16,17 +16,14 @@
 (in-package "SB!C")
 
 ;;; ANSI limits on compilation
-(def!constant sb!xc:call-arguments-limit sb!xc:most-positive-fixnum
-  #!+sb-doc
+(defconstant sb!xc:call-arguments-limit sb!xc:most-positive-fixnum
   "The exclusive upper bound on the number of arguments which may be passed
   to a function, including &REST args.")
-(def!constant sb!xc:lambda-parameters-limit sb!xc:most-positive-fixnum
-  #!+sb-doc
+(defconstant sb!xc:lambda-parameters-limit sb!xc:most-positive-fixnum
   "The exclusive upper bound on the number of parameters which may be specified
   in a given lambda list. This is actually the limit on required and &OPTIONAL
   parameters. With &KEY and &AUX you can get more.")
-(def!constant sb!xc:multiple-values-limit sb!xc:most-positive-fixnum
-  #!+sb-doc
+(defconstant sb!xc:multiple-values-limit sb!xc:most-positive-fixnum
   "The exclusive upper bound on the number of multiple VALUES that you can
   return.")
 
@@ -46,7 +43,10 @@
 
 ;;; the type of LAYOUT-DEPTHOID slot values
 (def!type layout-depthoid () '(or index (integer -1 -1)))
-(def!type layout-bitmap () '(and integer (not (eql 0))))
+(def!type layout-bitmap ()
+  ;; FIXME: Probably should exclude negative bignum
+  #!+compact-instance-header 'integer
+  #!-compact-instance-header '(and integer (not (eql 0))))
 
 ;;; An INLINEP value describes how a function is called. The values
 ;;; have these meanings:
@@ -63,10 +63,11 @@
 ;;;             references (even under #'without FUNCALL)."
 (deftype inlinep ()
   '(member :inline :maybe-inline :notinline nil))
-(defparameter *inlinep-translations*
+(defconstant-eqx +inlinep-translations+
   '((inline . :inline)
     (notinline . :notinline)
-    (maybe-inline . :maybe-inline)))
+    (maybe-inline . :maybe-inline))
+  #'equal)
 
 ;;; *FREE-VARS* translates from the names of variables referenced
 ;;; globally to the LEAF structures for them. *FREE-FUNS* is like
@@ -89,6 +90,10 @@
 
 ;;; miscellaneous forward declarations
 (defvar *code-segment*)
+;; FIXME: this is a kludge due to the absence of a 'vop' argument
+;; to ALLOCATION-TRAMP in the x86-64 backend.
+#!+immobile-code
+(defvar *code-is-immobile*)
 #!+sb-dyncount (defvar *collect-dynamic-statistics*)
 (defvar *component-being-compiled*)
 (defvar *compiler-error-context*)
@@ -101,9 +106,10 @@
 (defvar *current-path*)
 (defvar *current-component*)
 (defvar *delayed-ir1-transforms*)
-(defvar *eval-tlf-index*)
+#!+sb-dyncount
 (defvar *dynamic-counts-tn*)
 (defvar *elsewhere*)
+(defvar *elsewhere-label*)
 (defvar *event-info*)
 (defvar *event-note-threshold*)
 (defvar *failure-p*)
@@ -127,9 +133,11 @@
 (defvar *undefined-warnings*)
 (defvar *warnings-p*)
 (defvar *lambda-conversions*)
+(defvar *compile-object* nil)
+(defvar *location-context* nil)
+(defvar *msan-unpoison* nil)
 
 (defvar *stack-allocate-dynamic-extent* t
-  #!+sb-doc
   "If true (the default), the compiler respects DYNAMIC-EXTENT declarations
 and stack allocates otherwise inaccessible parts of the object whenever
 possible. Potentially long (over one page in size) vectors are, however, not
@@ -140,6 +148,7 @@ the stack without triggering overflow protection.")
 ;;; This lock is seized in the compiler, and related areas -- like the
 ;;; classoid/layout/class system.
 (defglobal **world-lock** nil)
+#-sb-xc-host
 (!cold-init-forms
  (setf **world-lock** (sb!thread:make-mutex :name "World Lock")))
 (!defun-from-collected-cold-init-forms !world-lock-cold-init)
@@ -148,9 +157,6 @@ the stack without triggering overflow protection.")
   `(sb!thread:with-recursive-lock (**world-lock**)
      ,@body))
 
-(declaim (type fixnum *compiler-sset-counter*))
-(defvar *compiler-sset-counter* 0)
-
 ;;; unique ID for the next object created (to let us track object
 ;;; identity even across GC, useful for understanding weird compiler
 ;;; bugs where something is supposed to be unique but is instead
@@ -169,8 +175,9 @@ the stack without triggering overflow protection.")
 ;;; Rather than registering listeners, they can detect changes by comparing
 ;;; their stored nonce to the current nonce. Additionally the observers
 ;;; can detect whether function definitions have occurred.
-(declaim (fixnum *type-cache-nonce*))
-(!defglobal *type-cache-nonce* 0)
+#-sb-xc-host
+(progn (declaim (fixnum *type-cache-nonce*))
+       (!define-load-time-global *type-cache-nonce* 0))
 
 (def!struct (undefined-warning
             #-no-ansi-print-object
@@ -225,12 +232,13 @@ the stack without triggering overflow protection.")
     (style-warn 'asterisks-around-lexical-variable-name
                 :format-control
                 "using the lexical binding of the symbol ~
-                 ~/sb-impl::print-symbol-with-prefix/, not the~@
+                 ~/sb-ext:print-symbol-with-prefix/, not the~@
                  dynamic binding"
                 :format-arguments (list symbol)))
   (values))
 
-(def!struct (debug-name-marker (:print-function print-debug-name-marker)))
+(def!struct (debug-name-marker (:print-function print-debug-name-marker)
+                               (:copier nil)))
 
 (defvar *debug-name-level* 4)
 (defvar *debug-name-length* 12)
@@ -302,10 +310,19 @@ the stack without triggering overflow protection.")
 ;;; compiler debugging, rather than policy control.)
 (defvar *assembly-optimize* t)
 
+;;; Bound during eval-when :compile-time evaluation.
+(defvar *compile-time-eval* nil)
+(declaim (always-bound *compile-time-eval*))
+
+#!-immobile-code
+(defmacro code-immobile-p (thing)
+  (declare (ignore thing))
+  nil)
+
 (in-package "SB!ALIEN")
 
 ;;; Information describing a heap-allocated alien.
-(def!struct (heap-alien-info)
+(def!struct (heap-alien-info (:copier nil))
   ;; The type of this alien.
   (type (missing-arg) :type alien-type)
   ;; Its name.
@@ -313,3 +330,8 @@ the stack without triggering overflow protection.")
   ;; Data or code?
   (datap (missing-arg) :type boolean))
 (!set-load-form-method heap-alien-info (:xc :target))
+
+;; from 'llvm/projects/compiler-rt/lib/msan/msan.h':
+;;  "#define MEM_TO_SHADOW(mem) (((uptr)(mem)) ^ 0x500000000000ULL)"
+#!+linux ; shadow space differs by OS
+(defconstant sb!vm::msan-mem-to-shadow-xor-const #x500000000000)
