@@ -85,7 +85,7 @@
     other-immediate-1-lowtag
     other-pointer-lowtag))
 
-(def!constant nil-value
+(defconstant nil-value
     (+ static-space-start n-word-bytes other-pointer-lowtag))
 
 (defconstant-eqx fixnum-lowtags
@@ -208,7 +208,7 @@
   unused01-widetag                          ;  5E       5E
   #!+sb-simd-pack
   simd-pack-widetag                         ;       65       65
-  unused02-widetag                          ;  62   69  62   69
+  filler-widetag                            ;  62   69  62   69
   unused03-widetag                          ;  66   6D  66   6D
   unused04-widetag                          ;  6A   71  6A   71
   unused05-widetag                          ;  6E   75  6E   75
@@ -292,7 +292,7 @@
 ;; often the print-name of a symbol, or was a literal in source code
 ;; and loaded from a fasl, or used in a few others situations
 ;; which warrant sharing.
-(def!constant +vector-shareable+ #x100)
+(defconstant +vector-shareable+ #x100)
 
 ;; A vector tagged as +VECTOR-SHAREABLE-NONSTD+ is logically readonly,
 ;; and *not* technically permitted by the standard to be shared.
@@ -303,6 +303,26 @@
 ;; nonetheless, opportunities for sharing abound.
 (defconstant +vector-shareable-nonstd+ #x200)
 
+;;; This is so that COMPILE-FILE knows that things like :ALLOW-OTHER-KEYS
+;;; can be immediate constants.
+#!+(and immobile-space (not immobile-symbols))
+(defconstant +initial-core-symbol-bit+ 8) ; bit index, not bit value
+
+#!+immobile-space
+(progn
+  ;; See 'doc/internal-notes/compact-instance' for rationale
+  (defconstant layout-align #!+64-bit 128 #!-64-bit 256) ; in bytes
+
+  ;; FUNCTION-LAYOUT is a fixnum whose bits are ORed in "as-is" with the
+  ;; low half of a closure header to form the full header word.
+  #!+(and (not (host-feature sb-xc-host)) (not sb-thread))
+  (defglobal function-layout 0)         ; set by genesis
+
+  ;; The cross-compiler stores FUNCTION-LAYOUT in a more obvious way.
+  #+sb-xc-host
+  (defconstant function-layout ; kludge - verified by genesis
+    (logior (+ fixedobj-space-start layout-align) instance-pointer-lowtag)))
+
 #|
 ;; Run this in the SB-VM or SB!VM package once for each target feature combo.
 (defun rewrite-widetag-comments ()
