@@ -72,10 +72,8 @@
 (define-vop (type-check-error/c)
   (:policy :fast-safe)
   (:translate sb!c::%type-check-error/c)
-  (:args (object :scs (descriptor-reg any-reg unsigned-reg signed-reg)
-                 :load-if (not (sc-is object
-                                      descriptor-reg any-reg
-                                      unsigned-reg signed-reg constant))))
+  (:args (object :scs (descriptor-reg any-reg unsigned-reg signed-reg
+                       character-reg constant)))
   (:arg-types * (:constant symbol) (:constant t))
   (:info errcode *location-context*)
   (:vop-var vop)
@@ -87,16 +85,31 @@
     ;; instruction pipe with undecodable junk (the sc-numbers).
     (error-call vop errcode object)))
 
+#!+immobile-space
+(defun type-err-type-tn-loadp (thing)
+  (cond ((sc-is thing immediate)
+         (let ((obj (tn-value thing)))
+           (typecase obj
+             (layout nil)
+             ;; non-static symbols can be referenced as error-break args
+             ;; because they appear in the code constants.
+             ;; static symbols can't be referenced as error-break args
+             ;; because there is no way to refer to an immediate value.
+             (symbol (static-symbol-p obj))
+             (t t))))
+        (t t)))
+
 (macrolet ((def (name error translate context &rest args)
              `(define-vop (,name)
                 ,@(when translate
                     `((:policy :fast-safe)
                       (:translate ,translate)))
                 (:args ,@(mapcar (lambda (arg)
-                                   `(,arg :scs (descriptor-reg any-reg
-                                                unsigned-reg signed-reg)
-                                          :load-if (not (sc-is ,arg descriptor-reg any-reg
-                                                               unsigned-reg signed-reg constant))))
+                                   `(,arg :scs (descriptor-reg any-reg character-reg
+                                                unsigned-reg signed-reg constant)
+                                          #!+immobile-space
+                                          ,@(if (eq name 'type-check-error)
+                                                `(:load-if (type-err-type-tn-loadp ,arg)))))
                                  args))
                 ,@(and context
                        `((:info *location-context*)
@@ -118,12 +131,40 @@
     sb!c::%odd-key-args-error nil)
   (def unknown-key-arg-error unknown-key-arg-error
     sb!c::%unknown-key-arg-error t key)
-  (def nil-fun-returned-error nil-fun-returned-error nil nil fun))
+  (def nil-fun-returned-error nil-fun-returned-error nil nil fun)
+  (def failed-aver sb!kernel::failed-aver-error
+    sb!impl::%failed-aver
+    nil form))
 
+
+(defun emit-internal-error (kind code values &key trap-emitter
+                                                  (compact-error-trap t))
+  (let ((trap-number (if (and (eq kind error-trap)
+                              compact-error-trap)
+                         (+ kind code)
+                         kind)))
+    (if trap-emitter
+        (funcall trap-emitter trap-number)
+        (inst byte trap-number)))
+  (unless (and (eq kind error-trap)
+               compact-error-trap)
+    (inst byte code))
+  (encode-internal-error-args
+   (mapcar (lambda (tn)
+             (cond ((and (tn-p tn) (sc-is tn immediate))
+                    (aver (typep (tn-value tn) '(or symbol layout)))
+                    (make-sc+offset constant-sc-number (tn-offset tn)))
+                   (t
+                    tn)))
+           values)))
+
 (defun encode-internal-error-args (values)
   (with-adjustable-vector (vector)
-    (dolist (tn values)
+    (dolist (where values)
       (write-var-integer
-       (make-sc-offset (sc-number (tn-sc tn)) (or (tn-offset tn) 0))
+       ;; WHERE can be either a TN or a packed SC number + offset
+       (if (tn-p where)
+           (make-sc+offset (sc-number (tn-sc where)) (or (tn-offset where) 0))
+           where)
        vector))
     (loop for octet across vector do (inst byte octet))))
