@@ -107,7 +107,7 @@
                             (remove-keywords options '(:rest-p :length))))
           (let ((offset-sym (symbolicate name "-" slot-name
                                          (if rest-p "-OFFSET" "-SLOT"))))
-            (constants `(def!constant ,offset-sym ,offset))
+            (constants `(defconstant ,offset-sym ,offset))
             (when special
               (specials `(defvar ,special))))
           #-c-headers-only
@@ -184,6 +184,25 @@
                    (defconstant ,constant-name ,sc-number))))))
       `(progn ,@(mapcan #'process-class classes)))))
 
+;;;; some general constant definitions
+
+;;; The maximum number of storage classes and offsets within a given
+;;; storage class. Applies to all backends.
+(defconstant sc-number-limit 62)
+(defconstant sc-number-bits (integer-length (1- sc-number-limit)))
+(deftype sb!c::sc-number () `(integer 0 (,sc-number-limit)))
+
+(defconstant sc-offset-limit (ash 1 21))
+(defconstant sc-offset-bits (integer-length (1- sc-offset-limit)))
+(deftype sc-offset () `(integer 0 (,sc-offset-limit)))
+
+(defconstant finite-sc-offset-limit
+  #!-(or sparc alpha hppa) 32
+  #!+(or sparc alpha hppa) 64)
+(defconstant finite-sc-offset-bits
+  (integer-length (1- finite-sc-offset-limit)))
+(deftype finite-sc-offset () `(integer 0 (,finite-sc-offset-limit)))
+
 ;;;; stuff for defining reffers and setters
 
 (in-package "SB!C")
@@ -204,15 +223,6 @@
 ;;; allows the xc host CL to implement structure slot setters as SETF
 ;;; expanders instead of SETF functions. -- WHN 2002-02-09
 
-;;;; some general constant definitions
-
-;;; FIXME: SC-NUMBER-LIMIT should probably be exported from SB!C
-;;; or SB!VM so that we don't need to do this extra IN-PACKAGE.
-(in-package "SB!C")
-
-;;; the maximum number of SCs in any implementation
-(defconstant sc-number-limit 62)
-
 ;;; Modular functions
 
 ;;; For a documentation, see CUT-TO-WIDTH.
