@@ -27,55 +27,38 @@
 ;;;
 ;;; We enter the basic structure at meta-compile time, and then fill
 ;;; in the missing slots at load time.
-(defmacro define-storage-base (name kind &key size (size-increment size)
-                                           (size-alignment 1))
+(defmacro !define-storage-bases (&rest definitions &aux (index -1) forms)
+  (dolist (def definitions)
+    (destructuring-bind (name kind &key size (size-increment size)
+                                             (size-alignment 1))
+        (cdr def)
 
-  (declare (type symbol name))
-  (declare (type (member :finite :unbounded :non-packed) kind))
+      (declare (type symbol name))
+      (declare (type (member :finite :unbounded :non-packed) kind))
 
-  ;; SIZE is either mandatory or forbidden.
-  (ecase kind
-    (:non-packed
-     (when size
-       (error "A size specification is meaningless in a ~S SB." kind)))
-    ((:finite :unbounded)
-     (unless size (error "Size is not specified in a ~S SB." kind))
-     (aver (typep size 'unsigned-byte))
-     (aver (= 1 (logcount size-alignment)))
-     (aver (not (logtest size (1- size-alignment))))
-     (aver (not (logtest size-increment (1- size-alignment))))))
+      ;; SIZE is either mandatory or forbidden.
+      (ecase kind
+        (:non-packed
+         (when size
+           (error "A size specification is meaningless in a ~S SB." kind)))
+        ((:finite :unbounded)
+         (unless size (error "Size is not specified in a ~S SB." kind))
+         (aver (<= size sb!vm:finite-sc-offset-limit))
+         (aver (= 1 (logcount size-alignment)))
+         (aver (not (logtest size (1- size-alignment))))
+         (aver (not (logtest size-increment (1- size-alignment))))))
 
-  (let ((sb (if (eq kind :non-packed)
-                 (make-sb :name name :kind kind)
-                 (make-finite-sb :name name :kind kind :size size
-                                 :size-increment size-increment
-                                 :size-alignment size-alignment))))
-    `(progn
-       (/show0 "in DEFINE-STORAGE-BASE")
-       ;; DEFINE-STORAGE-CLASS need the storage bases while building
-       ;; the cross-compiler, but to eval this during cross-compilation
-       ;; would kill the cross-compiler.
-       (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-         (let ((sb (,(if (eq kind :non-packed) 'copy-sb 'copy-finite-sb)
-                    ',sb)))
-           (setf *backend-sb-list*
-                 (cons sb (remove ',name *backend-sb-list* :key #'sb-name)))))
-       ,@(unless (eq kind :non-packed)
-           `((let ((res (sb-or-lose ',name)))
-               (/show0 "not :NON-PACKED, i.e. hairy case")
-               (setf (finite-sb-always-live res)
-                     (make-array ',size :initial-element #*))
-               (/show0 "doing second SETF")
-               (setf (finite-sb-conflicts res)
-                     (make-array ',size :initial-element '#()))
-               (/show0 "doing third SETF")
-               (setf (finite-sb-live-tns res)
-                     (make-array ',size :initial-element nil))
-               (/show0 "doing fourth SETF")
-               (setf (finite-sb-always-live-count res)
-                     (make-array ',size :initial-element 0)))))
-       (/show0 "finished with DEFINE-STORAGE-BASE expansion")
-       ',name)))
+      (push (if (eq kind :non-packed)
+                `(make-storage-base :name ',name :kind ,kind)
+                `(make-finite-sb-template
+                                 :index ,(incf index) :name ',name
+                                 :kind ,kind :size ,size
+                                 :size-increment ,size-increment
+                                 :size-alignment ,size-alignment))
+            forms)))
+  ;; Do not clobber the global var while running the cross-compiler.
+  `(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
+     (setf *backend-sbs* (vector ,@(nreverse forms)))))
 
 ;;; Define a storage class NAME that uses the named Storage-Base.
 ;;; NUMBER is a small, non-negative integer that is used as an alias.
@@ -112,9 +95,10 @@
 ;;; :CONSTANT-SCS (SC*)
 ;;;   A list of the names of all the constant SCs that can be loaded into this
 ;;;   SC by a move function.
-(defmacro define-storage-class (name number sb-name &key (element-size '1)
-                                     (alignment '1) locations reserve-locations
-                                     save-p alternate-scs constant-scs)
+(defmacro !define-storage-class (name number sb-name &key (element-size '1)
+                                      (alignment '1) locations reserve-locations
+                                      save-p alternate-scs constant-scs
+                                      operand-size)
   (declare (type symbol name))
   (declare (type sc-number number))
   (declare (type symbol sb-name))
@@ -153,12 +137,14 @@
              t nil)))
     `(progn
        (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-         (let ((res (make-sc :name ',name :number ',number
+         (let ((res (make-storage-class
+                             :name ',name :number ',number
                              :sb (sb-or-lose ',sb-name)
                              :element-size ,element-size
+                             :operand-size ,operand-size
                              :alignment ,alignment
-                             :locations ',locations
-                             :reserve-locations ',reserve-locations
+                             :locations (make-sc-locations ',locations)
+                             :reserve-locations (make-sc-locations ',reserve-locations)
                              :save-p ',save-p
                              :number-stack-p ,nstack-p
                              :alternate-scs (mapcar #'sc-or-lose
@@ -212,7 +198,8 @@
              (setf (svref (sc-load-costs to-sc) num) ',cost)))))
 
      (defun ,name ,lambda-list
-       (sb!assem:assemble (*code-segment* ,(first lambda-list))
+       (declare (ignorable ,(car lambda-list)))
+       (sb!assem:assemble ()
          ,@body))))
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
@@ -259,7 +246,7 @@
 ;;; operand or temporary at meta-compile time. Besides the obvious
 ;;; stuff, we also store the names of per-operand temporaries here.
 (def!struct (operand-parse
-             (:make-load-form-fun just-dump-it-normally)
+             (:copier nil)
              #-sb-xc-host (:pure t))
   ;; name of the operand (which we bind to the TN)
   (name nil :type symbol)
@@ -290,12 +277,11 @@
   (sc nil :type (or symbol null))
   ;; If non-null, we are a temp wired to this offset in SC.
   (offset nil :type (or unsigned-byte null)))
+(!set-load-form-method operand-parse (:host :xc :target))
 
 ;;; A VOP-PARSE object holds everything we need to know about a VOP at
 ;;; meta-compile time.
-(def!struct (vop-parse
-             (:make-load-form-fun just-dump-it-normally)
-             #-sb-xc-host (:pure t))
+(def!struct (vop-parse #-sb-xc-host (:pure t))
   ;; the name of this VOP
   (name nil :type symbol)
   ;; If true, then the name of the VOP we inherit from.
@@ -340,10 +326,6 @@
   (info-args () :type list)
   ;; an efficiency note associated with this VOP
   (note nil :type (or string null))
-  ;; a list of the names of the Effects and Affected attributes for
-  ;; this VOP
-  (effects '#1=(any) :type list)
-  (affected '#1# :type list)
   ;; a list of the names of functions this VOP is a translation of and
   ;; the policy that allows this translation to be done. :FAST is a
   ;; safe default, since it isn't a safe policy.
@@ -354,6 +336,7 @@
   ;; info about how to emit MOVE-ARG VOPs for the &MORE operand in
   ;; call/return VOPs
   (move-args nil :type (member nil :local-call :full-call :known-return)))
+(!set-load-form-method vop-parse (:host :xc :target))
 (defprinter (vop-parse)
   name
   (inherits :test inherits)
@@ -372,8 +355,6 @@
   (variant-vars :test variant-vars)
   (info-args :test info-args)
   (note :test note)
-  effects
-  affected
   translate
   ltn-policy
   (save-p :test save-p)
@@ -491,30 +472,24 @@
 
 ;;; Return a time spec describing a time during the evaluation of a
 ;;; VOP, used to delimit operand and temporary lifetimes. The
-;;; representation is a cons whose CAR is the number of the evaluation
-;;; phase and the CDR is the sub-phase. The sub-phase is 0 in the
-;;; :LOAD and :SAVE phases.
+;;; representation is a fixnum [phase][16-bit sub-phase].
+;;; The sub-phase is 0 in the :LOAD and :SAVE phases.
 (defun parse-time-spec (spec)
   (let ((dspec (if (atom spec) (list spec 0) spec)))
     (unless (and (= (length dspec) 2)
                  (typep (second dspec) 'unsigned-byte))
       (error "malformed time specifier: ~S" spec))
-
-    (cons (case (first dspec)
-            (:load 0)
-            (:argument 1)
-            (:eval 2)
-            (:result 3)
-            (:save 4)
-            (t
-             (error "unknown phase in time specifier: ~S" spec)))
-          (second dspec))))
-
-;;; Return true if the time spec X is the same or later time than Y.
-(defun time-spec-order (x y)
-  (or (> (car x) (car y))
-      (and (= (car x) (car y))
-           (>= (cdr x) (cdr y)))))
+    (let ((phase (case (first dspec)
+                   (:load 0)
+                   (:argument 1)
+                   (:eval 2)
+                   (:result 3)
+                   (:save 4)
+                   (t
+                    (error "unknown phase in time specifier: ~S" spec))) )
+          (sub-phase (second dspec)))
+      (+ (ash phase 16)
+         sub-phase))))
 
 ;;;; generation of emit functions
 
@@ -531,7 +506,7 @@
             (aver sc)
             (setf (aref results index)
                   (if offset
-                      (+ (ash offset (1+ sc-bits))
+                      (+ (ash offset (1+ sb!vm:sc-number-bits))
                          (ash (sc-number-or-lose sc) 1)
                          1)
                       (ash (sc-number-or-lose sc) 1))))
@@ -588,8 +563,8 @@
                                   (lambda (x y)
                                     (let ((x-time (car x))
                                           (y-time (car y)))
-                                      (if (time-spec-order x-time y-time)
-                                          (if (time-spec-order y-time x-time)
+                                      (if (>= x-time y-time)
+                                          (if (>= y-time x-time)
                                               (and (not (cdr x)) (cdr y))
                                               nil)
                                           t)))
@@ -685,10 +660,13 @@
                          (if load-p
                              `(,(cdr (first funs)) ,n-vop ,tn ,load-tn)
                              `(,(cdr (first funs)) ,n-vop ,load-tn ,tn)))))
-          (if (eq (operand-parse-load op) t)
-              `(when ,load-tn ,form)
-              `(when (eq ,load-tn ,(operand-parse-name op))
-                 ,form)))
+          (cond (load-p
+                 form)
+                ((eq (operand-parse-load op) t)
+                 `(when ,load-tn ,form))
+                (t
+                 `(when (eq ,load-tn ,(operand-parse-name op))
+                    ,form))))
         `(when ,load-tn
            (error "load TN allocated, but no move function?~@
                    VM definition is inconsistent, recompile and try again.")))))
@@ -699,9 +677,15 @@
 (defun decide-to-load (parse op)
   (let ((load (operand-parse-load op))
         (load-tn (operand-parse-load-tn op))
-        (temp (operand-parse-temp op)))
+        (temp (operand-parse-temp op))
+        (loads (and (eq (operand-parse-kind op) :argument)
+                    (call-move-fun parse op t))))
     (if (eq load t)
-        `(or ,load-tn (tn-ref-tn ,temp))
+        `(cond (,load-tn
+                ,loads
+                ,load-tn)
+               (t
+                (tn-ref-tn ,temp)))
         (collect ((binds)
                   (ignores))
           (dolist (x (vop-parse-operands parse))
@@ -709,12 +693,14 @@
               (let ((name (operand-parse-name x)))
                 (binds `(,name (tn-ref-tn ,(operand-parse-temp x))))
                 (ignores name))))
-          `(if (and ,load-tn
-                    (let ,(binds)
-                      (declare (ignorable ,@(ignores)))
-                      ,load))
-               ,load-tn
-               (tn-ref-tn ,temp))))))
+          `(cond ((and ,load-tn
+                       (let ,(binds)
+                         (declare (ignorable ,@(ignores)))
+                         ,load))
+                  ,loads
+                  ,load-tn)
+                 (t
+                  (tn-ref-tn ,temp)))))))
 
 ;;; Make a lambda that parses the VOP TN-REFS, does automatic operand
 ;;; loading, and runs the appropriate code generator.
@@ -735,9 +721,8 @@
                     (binds `(,(operand-parse-load-tn op)
                              (tn-ref-load-tn ,temp)))
                     (binds `(,name ,(decide-to-load parse op)))
-                    (if (eq (operand-parse-kind op) :argument)
-                        (loads (call-move-fun parse op t))
-                        (saves (call-move-fun parse op nil))))
+                    (when (eq (operand-parse-kind op) :result)
+                      (saves (call-move-fun parse op nil))))
                    (t
                     (binds `(,name (tn-ref-tn ,temp)))))))
           (:temporary
@@ -746,6 +731,7 @@
           ((:more-argument :more-result))))
 
       `(named-lambda (vop ,(vop-parse-name parse)) (,n-vop)
+         (declare (ignorable ,n-vop))
          (let* (,@(access-operands (vop-parse-args parse)
                                    (vop-parse-more-args parse)
                                    `(vop-args ,n-vop))
@@ -767,8 +753,8 @@
                   ,@(binds))
            (declare (ignore ,@(vop-parse-ignores parse)))
            ,@(loads)
-           (sb!assem:assemble (*code-segment* ,n-vop)
-                              ,@(vop-parse-body parse))
+           (assemble ()
+             ,@(vop-parse-body parse))
            ,@(saves))))))
 
 (defvar *parse-vop-operand-count*)
@@ -919,10 +905,10 @@
             (t
              (error "unknown temporary option: ~S" opt))))
 
-        (unless (and (time-spec-order (operand-parse-dies res)
-                                      (operand-parse-born res))
-                     (not (time-spec-order (operand-parse-born res)
-                                           (operand-parse-dies res))))
+        (unless (and (>= (operand-parse-dies res)
+                         (operand-parse-born res))
+                     (< (operand-parse-born res)
+                        (operand-parse-dies res)))
           (error "Temporary lifetime doesn't begin before it ends: ~S" spec))
 
         (unless (operand-parse-sc res)
@@ -978,10 +964,6 @@
             (setf (vop-parse-cost parse)
                   (vop-spec-arg spec 'unsigned-byte 1 nil))
           (setf (vop-parse-body parse) (cddr spec)))
-        (:effects
-         (setf (vop-parse-effects parse) (rest spec)))
-        (:affected
-         (setf (vop-parse-affected parse) (rest spec)))
         (:info
          (setf (vop-parse-info-args parse) (rest spec)))
         (:ignore
@@ -1044,8 +1026,8 @@
 (defun compute-loading-costs (op load-p)
   (declare (type operand-parse op))
   (let ((scs (operand-parse-scs op))
-        (costs (make-array sc-number-limit :initial-element nil))
-        (load-scs (make-array sc-number-limit :initial-element nil)))
+        (costs (make-array sb!vm:sc-number-limit :initial-element nil))
+        (load-scs (make-array sb!vm:sc-number-limit :initial-element nil)))
     (dolist (sc-name (reverse scs))
       (let* ((load-sc (sc-or-lose sc-name))
              (load-scn (sc-number load-sc)))
@@ -1071,7 +1053,7 @@
               (unless (eq op-load t)
                 (pushnew load-scn (svref load-scs op-scn))))))
 
-        (dotimes (i sc-number-limit)
+        (dotimes (i sb!vm:sc-number-limit)
           (unless (svref costs i)
             (let ((op-sc (svref *backend-sc-numbers* i)))
               (when op-sc
@@ -1084,10 +1066,10 @@
     (values costs load-scs)))
 
 (defparameter *no-costs*
-  (make-array sc-number-limit :initial-element 0))
+  (make-array sb!vm:sc-number-limit :initial-element 0))
 
 (defparameter *no-loads*
-  (make-array sc-number-limit :initial-element t))
+  (make-array sb!vm:sc-number-limit :initial-element t))
 
 ;;; Pick off the case of operands with no restrictions.
 (defun compute-loading-costs-if-any (op load-p)
@@ -1313,13 +1295,7 @@
                                 `(primitive-type-or-lose ',type))
                               (rest type))))
            (:constant
-            ``(:constant ,(named-lambda (vop-arg-typep) (x)
-                              ;; Can't handle SATISFIES during XC
-                              ,(if (and (consp (second type))
-                                        (eq (caadr type) 'satisfies))
-                                   `(,(cadadr type) x)
-                                   `(sb!xc:typep x ',(second type))))
-                         ,',(second type)))))))
+            ``(:constant . ,',(second type)))))))
 
 (defun specify-operand-types (types ops more-ops)
   (if (eq types :unspecified)
@@ -1403,8 +1379,6 @@
       :ltn-policy ',(vop-parse-ltn-policy parse)
       :save-p ',(vop-parse-save-p parse)
       :move-args ',(vop-parse-move-args parse)
-      :effects (vop-attributes ,@(vop-parse-effects parse))
-      :affected (vop-attributes ,@(vop-parse-affected parse))
       ,@(make-costs-and-restrictions parse)
       ,@(make-emit-function-and-friends parse)
       ,@(inherit-vop-info :generator-function iparse
@@ -1508,12 +1482,6 @@
 ;;;     During the evaluation of the body, the names of the operands
 ;;;     and temporaries are bound to the actual TNs.
 ;;;
-;;; :EFFECTS Effect*
-;;; :AFFECTED Effect*
-;;;     Specifies the side effects that this VOP has and the side
-;;;     effects that effect its execution. If unspecified, these
-;;;     default to the worst case.
-;;;
 ;;; :INFO Name*
 ;;;     Define some magic arguments that are passed directly to the code
 ;;;     generator. The corresponding trailing arguments to VOP or
@@ -1558,7 +1526,7 @@
 ;;;     This option causes the VOP template to be entered as an IR2
 ;;;     translation for the named functions.
 ;;;
-;;; :POLICY {:SMALL | :FAST | :SAFE | :FAST-SAFE}
+;;; :POLICY {:SMALL | :SMALL-SAFE | :FAST | :SAFE | :FAST-SAFE}
 ;;;     Specifies the policy under which this VOP is the best translation.
 ;;;
 ;;; :GUARD Form
@@ -1829,8 +1797,8 @@
     (collect ((clauses))
       (do ((cases forms (rest cases)))
           ((null cases)
-           (clauses `(t (error "unknown SC to SC-CASE for ~S:~%  ~S" ,n-tn
-                               (sc-name (tn-sc ,n-tn))))))
+           (clauses `(t (locally (declare (optimize (safety 0))) ;; avoid NIL-FUN-RETURNED-ERROR
+                          (unknown-sc-case ,n-tn)))))
         (let ((case (first cases)))
           (when (atom case)
             (error "illegal SC-CASE clause: ~S" case))
@@ -1849,6 +1817,9 @@
               (,n-sc (sc-number (tn-sc ,n-tn))))
          (cond ,@(clauses))))))
 
+(defun unknown-sc-case (tn)
+  (error "unknown SC to SC-CASE for ~S:~%  ~S" tn (sc-name (tn-sc tn))))
+
 ;;; Return true if TNs SC is any of the named SCs, false otherwise.
 (defmacro sc-is (tn &rest scs)
   (once-only ((n-sc `(sc-number (tn-sc ,tn))))
