@@ -45,71 +45,54 @@
 ;;;; (http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.107.9598)
 
 ;;; Interference graph data structure
-(defstruct (ordered-set
-            (:include sset)
-            (:conc-name #:oset-))
-  (members nil :type list))
-
-(defun oset-adjoin (oset element)
-  (when (sset-adjoin element oset)
-    (push element (oset-members oset))
-    t))
-
-(defun oset-delete (oset element)
-  (when (sset-delete element oset)
-    (setf (oset-members oset)
-          (delete element (oset-members oset)))
-    t))
-
-(defun oset-member (oset element)
-  (sset-member element oset))
-
-(defmacro do-oset-elements ((variable oset &optional return) &body body)
-  `(dolist (,variable (oset-members ,oset) ,return)
-     ,@body))
 
 ;; vertex in an interference graph
-(def!struct (vertex
+(defstruct (vertex
              (:include sset-element)
-             (:constructor make-vertex (tn pack-type)))
+             (:copier nil)
+             (:constructor %make-vertex (tn element-size pack-type)))
   ;; incidence set, as an ordered list (for reproducibility)
-  (incidence (make-ordered-set) :type ordered-set)
+  (full-incidence  (make-oset) :type oset :read-only t)
+  (incidence       #()                :type simple-vector)
+  (incidence-count 0                  :type index)
   ;; list of potential locations in the TN's preferred SB for the
   ;; vertex, taking into account reserve locations and preallocated
   ;; TNs.
-  (initial-domain nil :type list)
-  (initial-domain-size 0 :type index)
+  (initial-domain 0 :type sc-locations)
+  (initial-domain-size 0 :type #.`(integer 0 ,sb!vm:finite-sc-offset-limit))
   ;; TN this is a vertex for.
-  (tn nil :type tn)
+  (tn           nil :type tn                                  :read-only t)
+  (element-size nil :type fixnum                              :read-only t)
   ;; type of packing necessary. We should only have to determine
   ;; colors for :normal TNs/vertices
-  (pack-type nil :type (member :normal :wired :restricted))
-  ;; color offset
-  (color nil :type (or fixnum null))
-  ;; current status, removed from the interference graph or not (on
-  ;; stack or not)
-  (invisible nil :type t)
+  (pack-type    nil :type (member :normal :wired :restricted) :read-only t)
   ;; (tn-spill-cost (vertex-tn vertex))
-  (spill-cost 0 :type fixnum))
+  (spill-cost   0   :type fixnum)
+  ;; color offset
+  (color nil :type (or fixnum null)))
+
+(declaim (inline make-vertex))
+(defun make-vertex (tn pack-type)
+  (%make-vertex tn (sc-element-size (tn-sc tn)) pack-type))
 
 (declaim (inline vertex-sc))
 (defun vertex-sc (vertex)
   (tn-sc (vertex-tn vertex)))
 
 ;; interference graph
-(def!struct (interference-graph
+(defstruct (interference-graph
+             (:copier nil)
              (:constructor %make-interference-graph)
              (:conc-name #:ig-))
   ;; sorted set of yet-uncolored (and not necessarily spilled)
   ;; vertices: vertices with lower spill cost come first.
   (vertices nil :type list)
   ;; unsorted set of precolored vertices.
-  (precolored-vertices nil :type list)
-  (tn-vertex (bug "missing arg") :type hash-table)
+  (precolored-vertices nil :type list :read-only t)
   ;; A function that maps TNs to vertices, and then to the vertex's
   ;; assigned offset, if any.  The offset (or NIL) is returned first,
   ;; then the vertex as a second value.
-  (tn-vertex-mapping (bug "missing arg") :type function))
+  (tn-vertex-mapping (missing-arg) :type function :read-only t))
 
 ;;; Interference graph construction
 ;;;
@@ -132,9 +115,9 @@
   (unless (or (neq (sc-sb (vertex-sc a)) (sc-sb (vertex-sc b)))
               (tn-offset (vertex-tn a))
               (tn-offset (vertex-tn b)))
-    (aver (or (oset-adjoin (vertex-incidence a) b)
+    (aver (or (oset-adjoin (vertex-full-incidence a) b)
               perhaps-redundant))
-    (aver (or (oset-adjoin (vertex-incidence b) a)
+    (aver (or (oset-adjoin (vertex-full-incidence b) a)
               perhaps-redundant))))
 
 ;; Partition the global TNs that appear in that IR2 block, between
@@ -246,15 +229,19 @@
 
 ;; Supposing that TN is restricted to its preferred SC, what locations
 ;; are available?
+(declaim (ftype (sfunction (tn) sc-locations) restricted-tn-locations))
 (defun restricted-tn-locations (tn)
   (declare (type tn tn))
   (let* ((sc (tn-sc tn))
-         (reserve (sc-reserve-locations sc)))
-    (loop
-      for loc in (sc-locations sc)
-      unless (or (and reserve (memq loc reserve)) ; common case: no reserve
-                 (conflicts-in-sc tn sc loc))
-        collect loc)))
+         (reserve (sc-reserve-locations sc))
+         (available-locations (logand (sc-locations sc)
+                                      (lognot reserve)))
+         (locations 0))
+    (declare (type sc-locations locations))
+    (do-sc-locations (location available-locations locations
+                               (sc-element-size sc))
+      (unless (conflicts-in-sc tn sc location)
+        (setf (ldb (byte 1 location) locations) 1)))))
 
 ;; walk over vertices, precomputing as much information as possible,
 ;; and partitioning according to their kind.
@@ -269,15 +256,14 @@
           do (let* ((tn (vertex-tn vertex))
                     (offset (tn-offset tn))
                     (locs (if offset
-                              (list offset)
+                              (sc-offset-to-sc-locations offset)
                               (restricted-tn-locations tn))))
                (aver (not (unbounded-tn-p tn)))
                (setf (vertex-number vertex) i
-                     (vertex-incidence vertex) (make-ordered-set)
                      (vertex-initial-domain vertex) locs
-                     (vertex-initial-domain-size vertex) (length locs)
+                     (vertex-initial-domain-size vertex)
+                     (sc-locations-count locs)
                      (vertex-color vertex) offset
-                     (vertex-invisible vertex) nil
                      (vertex-spill-cost vertex) (tn-cost tn)
                      (gethash tn tn-vertex) vertex)
                (cond (offset) ; precolored -> no need to track conflict
@@ -306,10 +292,11 @@
     (collect ((colored)
               (uncolored))
       (dolist (v vertices)
-        (let ((incidence (vertex-incidence v)))
-          (setf (oset-members incidence)
+        (let ((full-incidence (vertex-full-incidence v)))
+          (setf (oset-members full-incidence)
                 ;; this really doesn't matter, but minimises variability
-                (sort (oset-members incidence) #'< :key #'vertex-number)))
+                (sort (oset-members full-incidence) #'< :key #'vertex-number)
+                (vertex-incidence v) (make-array (oset-count full-incidence))))
         (cond ((vertex-color v)
                (aver (tn-offset (vertex-tn v)))
                (colored v))
@@ -320,26 +307,22 @@
       (%make-interference-graph
        :vertices (stable-sort (uncolored) #'< :key #'vertex-spill-cost)
        :precolored-vertices (colored)
-       :tn-vertex tn-vertex
        :tn-vertex-mapping (lambda (tn)
                             (awhen (gethash tn tn-vertex)
                               (values (vertex-color it) it)))))))
 
-;; &key reset: whether coloring/invisibility information should be
-;; removed from all the remaining vertices
-(defun remove-vertex-from-interference-graph (vertex graph &key reset)
+;;; Coloring information is removed from all remaining vertices.
+(defun reset-interference-graph-without-vertex (graph vertex)
   (declare (type vertex vertex) (type interference-graph graph))
-  (let ((vertices (if reset
-                      (loop for v in (ig-vertices graph)
-                            unless (eql v vertex)
-                              do (aver (not (tn-offset (vertex-tn v))))
-                                 (setf (vertex-invisible v) nil
-                                       (vertex-color v) nil)
-                              and collect v)
-                      (remove vertex (ig-vertices graph)))))
+  (let ((vertices (loop for v in (ig-vertices graph)
+                        unless (eql v vertex)
+                          do (aver (not (tn-offset (vertex-tn v))))
+                             (setf (vertex-color v) nil
+                                   (vertex-incidence-count v) 0)
+                          and collect v)))
     (setf (ig-vertices graph) vertices)
-    (do-oset-elements (neighbor (vertex-incidence vertex) graph)
-      (oset-delete (vertex-incidence neighbor) vertex))))
+    (do-oset-elements (neighbor (vertex-full-incidence vertex) graph)
+      (oset-delete (vertex-full-incidence neighbor) vertex))))
 
 ;;; Support code
 
@@ -349,70 +332,104 @@
   (declare (type fixnum color)
            (type vertex vertex)
            (optimize speed (safety 0)))
-  (let ((color+size (+ color (sc-element-size (vertex-sc vertex)))))
+  (let ((incidence (vertex-incidence vertex))
+        (incidence-count (vertex-incidence-count vertex))
+        (color+size (+ color (vertex-element-size vertex))))
     (flet ((intervals-intersect-p (color2 vertex2)
              (declare (fixnum color2))
              (if (< color2 color)
-                 (< color (+ color2 (sc-element-size (vertex-sc vertex2))))
+                 (< color (+ color2 (vertex-element-size vertex2)))
                  (< color2 color+size))))
-      (do-oset-elements (neighbor (vertex-incidence vertex) t)
-        (cond ((vertex-invisible neighbor))
-              ((intervals-intersect-p (vertex-color neighbor) neighbor)
-               (return nil)))))))
+      (dotimes (i incidence-count t)
+        #-sb-xc-host (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+        (let ((neighbor (aref incidence i)))
+          (when (intervals-intersect-p (vertex-color neighbor) neighbor)
+            (return nil)))))))
 
 ;; Assumes that VERTEX pack-type is :WIRED.
 (defun vertex-color-possible-p (vertex color)
-  (declare (type fixnum color) (type vertex vertex))
+  (declare (type fixnum color)
+           (type vertex vertex))
   (and (or (and (neq (vertex-pack-type vertex) :wired)
                 (not (tn-offset (vertex-tn vertex))))
            (= color (the fixnum (vertex-color vertex))))
-       (member color (vertex-initial-domain vertex))
+       (sc-locations-member color (vertex-initial-domain vertex))
        (color-no-conflicts-p color vertex)))
 
 ;; Sorted list of all possible locations for vertex in its preferred
 ;; SC: more heavily loaded (i.e that should be tried first) locations
 ;; first.  vertex-initial-domain is already sorted, only have to
 ;; remove offsets that aren't currently available.
+(declaim (ftype (sfunction (vertex) sc-locations) vertex-domain))
 (defun vertex-domain (vertex)
-  (declare (type vertex vertex))
-  (remove-if-not (lambda (color)
-                   (vertex-color-possible-p vertex color))
-                 (vertex-initial-domain vertex)))
+  (let ((result 0))
+    (declare (type sc-locations result))
+    (do-sc-locations (color (vertex-initial-domain vertex) result
+                            (vertex-element-size vertex))
+      (when (vertex-color-possible-p vertex color)
+        (setf (ldb (byte 1 color) result) 1)))))
 
 ;; Return a list of vertices that we might want VERTEX to share its
 ;; location with.
 (defun vertex-target-vertices (vertex tn-offset)
   (declare (type vertex vertex) (type function tn-offset))
   (let ((sb (sc-sb (vertex-sc vertex)))
-        (neighbors (vertex-incidence vertex))
-        vertices)
+        (neighbors (vertex-full-incidence vertex))
+        (vertices '()))
     (do-target-tns (current (vertex-tn vertex) :limit 20)
       (multiple-value-bind (offset target)
           (funcall tn-offset current)
         (when (and offset
                    (eq sb (sc-sb (tn-sc current)))
                    (not (oset-member neighbors target)))
-          (pushnew target vertices))))
+          (pushnew target vertices :test #'eq))))
     (nreverse vertices)))
 
-;; Choose the "best" color for these vertices: a color is good if as
-;; many of these vertices simultaneously take that color, and those
-;; that can't have a low spill cost.
-(defun vertices-best-color (vertices colors)
-  (let ((best-color      nil)
-        (best-compatible '())
-        (best-cost       nil))
+;;; Choose the "best" color for these vertices: a color is good if as
+;;; many of these vertices simultaneously take that color, and those
+;;; that can't have a low spill cost.
+(declaim (ftype (sfunction (list sc-locations)
+                           (values sb!vm:finite-sc-offset list))
+                vertices-best-color/single-color))
+(defun vertices-best-color/single-color (vertices color)
+  (let ((compatible '()))
+    (dolist (vertex vertices)
+      (when (and (notany (lambda (existing)
+                           (oset-member (vertex-full-incidence existing)
+                                        vertex))
+                         compatible)
+                 (vertex-color-possible-p vertex color))
+        (push vertex compatible)))
+    (values color compatible)))
+
+(declaim (ftype (sfunction (vertex sc-locations)
+                           (values sb!vm:finite-sc-offset list))
+                vertices-best-color/single-vertex))
+(defun vertices-best-color/single-vertex (vertex colors)
+  (do-sc-locations (color colors nil (vertex-element-size vertex))
+    (when (vertex-color-possible-p vertex color)
+      (return-from vertices-best-color/single-vertex
+        (values color (list vertex)))))
+  (values (sc-locations-first colors) '()))
+
+(declaim (ftype (sfunction (cons sc-locations)
+                           (values sb!vm:finite-sc-offset list))
+                vertices-best-color/general))
+(defun vertices-best-color/general (vertices colors)
+  (let* ((best-color      nil)
+         (best-compatible '())
+         (best-cost       nil))
     ;; TODO: sort vertices by spill cost, so that high-spill cost ones
     ;; are more likely to be compatible?  We're trying to find a
     ;; maximal 1-colorable subgraph here, ie. a maximum independent
     ;; set :\ Still, a heuristic like first attempting to pack in
     ;; max-cost vertices may be useful
-    (dolist (color colors)
+    (do-sc-locations (color colors nil (vertex-element-size (first vertices)))
       (let ((compatible '())
             (cost 0))
         (dolist (vertex vertices)
           (when (and (notany (lambda (existing)
-                               (oset-member (vertex-incidence existing)
+                               (oset-member (vertex-full-incidence existing)
                                             vertex))
                              compatible)
                      (vertex-color-possible-p vertex color))
@@ -424,59 +441,70 @@
                 best-compatible compatible
                 best-cost       cost))))
     (values best-color best-compatible)))
+
+(declaim (inline vertices-best-color))
+(defun vertices-best-color (vertices colors)
+  (declare (type sc-locations colors))
+  (cond
+    ((null vertices)
+     (values (sc-locations-first colors) '()))
+    ((null (rest vertices))
+     (vertices-best-color/single-vertex (first vertices) colors))
+    ((= 1 (sc-locations-count colors))
+     (vertices-best-color/single-color vertices (sc-locations-first colors)))
+    (t
+     (vertices-best-color/general vertices colors))))
 
 ;;; Coloring inner loop
 
 ;; Greedily choose the color for this vertex, also moving around any
 ;; :target vertex to the same color if possible.
 (defun find-vertex-color (vertex tn-vertex-mapping)
-  (awhen (vertex-domain vertex)
-    (let* ((targets (vertex-target-vertices vertex tn-vertex-mapping))
-           (sc (vertex-sc vertex))
-           (sb (sc-sb sc)))
-      (multiple-value-bind (color recolor-vertices)
-          (if targets
-              (vertices-best-color targets it)
-              (values (first it) nil))
-        (aver color)
-        (dolist (target recolor-vertices)
-          (aver (vertex-color target))
-          (unless (eql color (vertex-color target))
-            (aver (eq sb (sc-sb (vertex-sc target))))
-            (aver (not (tn-offset (vertex-tn target))))
-            #+nil ; this check is slow
-            (aver (vertex-color-possible-p target color))
-            (setf (vertex-color target) color)))
-        color))))
+  (let ((domain (vertex-domain vertex)))
+    (unless (zerop domain)
+      (let* ((targets (vertex-target-vertices vertex tn-vertex-mapping))
+             (sc (vertex-sc vertex))
+             (sb (sc-sb sc)))
+        (multiple-value-bind (color recolor-vertices)
+            (vertices-best-color targets domain)
+          (dolist (target recolor-vertices)
+            (aver (vertex-color target))
+            (unless (eql color (vertex-color target))
+              (aver (eq sb (sc-sb (vertex-sc target))))
+              (aver (not (tn-offset (vertex-tn target))))
+              #+nil ; this check is slow
+              (aver (vertex-color-possible-p target color))
+              (setf (vertex-color target) color)))
+          color)))))
 
 ;; Partition vertices into those that are likely to be colored and
 ;; those that are likely to be spilled.  Assumes that the interference
 ;; graph's vertices are sorted with the least spill cost first, so
 ;; that the stacks end up with the greatest spill cost vertices first.
 (defun partition-and-order-vertices (interference-graph)
-  (flet ((domain-size (vertex)
-           (vertex-initial-domain-size vertex))
-         (degree (vertex)
-           (count-if-not #'vertex-invisible
-                         (oset-members (vertex-incidence vertex))))
-         (eliminate-vertex (vertex)
-           (setf (vertex-invisible vertex) t)))
-    (let* ((precoloring-stack '())
-           (prespilling-stack '())
-           (vertices (ig-vertices interference-graph)))
-      ;; walk the vertices from least important to most important TN wrt
-      ;; spill cost.  That way the TNs we really don't want to spill are
-      ;; at the head of the colouring lists.
-      (loop for vertex in vertices do
-        (aver (not (vertex-color vertex))) ; we already took those out above
-        (eliminate-vertex vertex)
-        ;; FIXME: some interference will be with vertices that don't
-        ;;  take the same number of slots. Find a smarter heuristic.
-        (cond ((< (degree vertex) (domain-size vertex))
-               (push vertex precoloring-stack))
-              (t
-               (push vertex prespilling-stack))))
-      (values precoloring-stack prespilling-stack))))
+  (let* ((precoloring-stack '())
+         (prespilling-stack '())
+         (vertices (ig-vertices interference-graph)))
+    ;; walk the vertices from least important to most important TN wrt
+    ;; spill cost.  That way the TNs we really don't want to spill are
+    ;; at the head of the colouring lists.
+    (loop for vertex in vertices do
+         (aver (not (vertex-color vertex))) ; we already took those out above
+       ;; FIXME: some interference will be with vertices that don't
+       ;;  take the same number of slots. Find a smarter heuristic.
+         (cond ((< (vertex-incidence-count vertex)
+                   (vertex-initial-domain-size vertex))
+                (push vertex precoloring-stack))
+               (t
+                (push vertex prespilling-stack))))
+    (values precoloring-stack prespilling-stack)))
+
+(defun color-vertex (vertex color)
+  (declare (type vertex vertex))
+  (setf (vertex-color vertex) color)
+  (do-oset-elements (neighbor (vertex-full-incidence vertex) vertex)
+    (let ((new-count (incf (vertex-incidence-count neighbor))))
+      (setf (aref (vertex-incidence neighbor) (1- new-count)) vertex))))
 
 ;; Try and color the interference graph once.
 (defun color-interference-graph (interference-graph)
@@ -484,8 +512,7 @@
     (flet ((color-vertices (vertices)
              (dolist (vertex vertices)
                (awhen (find-vertex-color vertex tn-vertex)
-                 (setf (vertex-color vertex) it
-                       (vertex-invisible vertex) nil)))))
+                 (color-vertex vertex it)))))
       (multiple-value-bind (probably-colored probably-spilled)
           (partition-and-order-vertices interference-graph)
         (color-vertices probably-colored)
@@ -508,7 +535,7 @@
 #+nil
 (defun collect-min-spill-candidates (vertex)
   (let ((colors '()))
-    (do-oset-elements (neighbor (vertex-incidence vertex))
+    (do-oset-elements (neighbor (vertex-full-incidence vertex))
       (when (eql :normal (vertex-pack-type neighbor))
         (let* ((color (vertex-color neighbor))
                (cell (assoc color colors))
@@ -532,27 +559,27 @@
                                 :key (lambda (vertex)
                                        (tn-loop-depth
                                         (vertex-tn vertex)))))
-         (nvertices (length vertices))
-         (graph (make-interference-graph vertices component))
-         to-spill)
+         (graph (make-interference-graph vertices component)))
     (labels ((spill-candidates-p (vertex)
                (unless (vertex-color vertex)
                  (aver (eql :normal (vertex-pack-type vertex)))
                  t))
-             (iter (to-spill)
-               (when to-spill
-                 (setf (vertex-invisible to-spill) t
-                       (vertex-color to-spill) nil)
-                 (push to-spill spill-list)
-                 (setf graph (remove-vertex-from-interference-graph
-                              to-spill graph :reset t)))
+             (try-color ()
                (color-interference-graph graph)
-               (find-if #'spill-candidates-p (ig-vertices graph))))
+               (find-if #'spill-candidates-p (ig-vertices graph)))
+             (spill (vertex)
+               (setf (vertex-color vertex) nil)
+               (push vertex spill-list)
+               (setf graph (reset-interference-graph-without-vertex
+                            graph vertex))))
       (loop repeat iterations
-            while (setf to-spill (iter to-spill))))
+            for uncolored = (try-color)
+            while uncolored
+            do (spill uncolored)))
     (let ((colored (ig-vertices graph)))
-      (aver (= nvertices (+ (length spill-list) (length colored)
-                            (length (ig-precolored-vertices graph)))))
+      (aver (= (length vertices)
+               (+ (length spill-list) (length colored)
+                  (length (ig-precolored-vertices graph)))))
       colored)))
 
 ;;; Nice interface
@@ -581,9 +608,10 @@
     ;; them in the graph for targeting purposes.
     (do ((tn (ir2-component-wired-tns 2comp) (tn-next tn)))
         ((null tn))
-      (pack-wired-tn tn optimize)
-      (unless (unbounded-tn-p tn)
-        (vertices (make-vertex tn :wired))))
+      (unless (eq (tn-kind tn) :arg-pass)
+        (pack-wired-tn tn optimize)
+        (unless (unbounded-tn-p tn)
+          (vertices (make-vertex tn :wired)))))
 
     ;; Preallocate vertices that *must* be in this finite SC.  If
     ;; targeting is improved, giving them a high priority in regular
