@@ -14,52 +14,60 @@
 ;;; a value for an optimization declaration
 (def!type policy-quality () '(integer 0 3))
 
-(defvar *policy*)
 (defvar *macro-policy* nil)
 ;;; global policy restrictions as a POLICY object or nil
-(!defvar *policy-restrictions* nil)
+(defvar *policy-min* nil)
+(defvar *policy-max* nil)
 
-(defun restrict-compiler-policy (&optional quality (min 0))
-  #!+sb-doc
+(defun restrict-compiler-policy (&optional quality (min 0) (max 3))
   "Assign a minimum value to an optimization quality. QUALITY is the name of
-the optimization quality to restrict, and MIN (defaulting to zero) is the
-minimum allowed value.
+the optimization quality to restrict, MIN (defaulting to zero) is the
+minimum allowed value, and MAX (defaults to 3) is the maximum.
 
 Returns the alist describing the current policy restrictions.
 
 If QUALITY is NIL or not given, nothing is done.
 
-Otherwise, if MIN is zero or not given, any existing restrictions of QUALITY
-are removed. If MIN is between one and three inclusive, it becomes the new
-minimum value for the optimization quality: any future proclamations or
-declarations of the quality with a value less then MIN behave as if the value
-was MIN instead.
+Otherwise, if MIN is zero or MAX is 3 or neither are given, any
+existing restrictions of QUALITY are removed.
 
-This is intended to be used interactively, to facilitate recompiling large
-bodies of code with eg. a known minimum safety.
-
-See also :POLICY option in WITH-COMPILATION-UNIT.
-
-EXPERIMENTAL INTERFACE: Subject to change."
-  (declare (type policy-quality min))
+See also :POLICY option in WITH-COMPILATION-UNIT."
+  (declare (type policy-quality min max))
   (when quality
-    (unless (policy-quality-name-p quality)
-      (error "~S is not a policy quality" quality))
-    ;; The dynamic policy object is immutable, otherwise a construct like
-    ;;  (let ((*policy-restrictions* *policy-restrictions*)) ...)
-    ;; could allow alterations inside the LET to leak out.
-    ;; The structure itself does not declare slots its read-only, because
-    ;; OPTIMIZE declaration processing uses it as a scratchpad.
-    (setf *policy-restrictions*
-          (acond (*policy-restrictions* (copy-structure it))
-                 (t (make-policy 0 0))))
-    (alter-policy *policy-restrictions* (policy-quality-name-p quality)
-                  min (plusp min)))
+    (let ((quality-id (policy-quality-name-p quality)))
+      (unless quality-id
+        (error "~S is not a policy quality" quality))
+      (when (> min max)
+        (error "MIN ~s should be not be greater than MAX ~s." min max))
+      ;; The dynamic policy object is immutable, otherwise a construct like
+      ;;  (let ((*policy-min* *policy-max*)) ...)
+      ;; could allow alterations inside the LET to leak out.
+      ;; The structure itself does not declare slots its read-only, because
+      ;; OPTIMIZE declaration processing uses it as a scratchpad.
+      (setf *policy-min*
+            (if *policy-min*
+                (copy-structure *policy-min*)
+                (make-policy 0 0))
+            *policy-max*
+            (if *policy-max*
+                (copy-structure *policy-max*)
+                (let ((policy (make-policy
+                               (1- (expt 2 (* 2 n-policy-primary-qualities)))
+                               0)))
+                  (setf (policy-dependent-qualities policy)
+                        (1- (expt 2
+                                  (* (- max-policy-qualities n-policy-primary-qualities)
+                                     2))))
+                  policy)))
+      (alter-policy *policy-min* quality-id min (plusp min))
+      (alter-policy *policy-max* quality-id max (< max 3))))
   ;; Return dotted pairs, not elements that look declaration-like.
-  (if *policy-restrictions*
-      (mapc (lambda (x) (rplacd x (cadr x)))
-            (policy-to-decl-spec *policy-restrictions*))
-      '()))
+  (flet ((policy-to-result (policy)
+           (and policy
+                (mapc (lambda (x) (rplacd x (cadr x)))
+                      (policy-to-decl-spec policy)))))
+    (values (policy-to-result *policy-min*)
+            (policy-to-result *policy-max*))))
 
 (defstruct (policy-dependent-quality (:copier nil))
   (name nil :type symbol :read-only t)
@@ -68,57 +76,8 @@ EXPERIMENTAL INTERFACE: Subject to change."
   (values-documentation nil :read-only t))
 
 ;;; names of recognized optimization policy qualities
-(declaim (simple-vector **policy-primary-qualities**
-                        **policy-dependent-qualities**))
-(!defglobal **policy-primary-qualities**
-        #(;; ANSI standard qualities
-          compilation-speed
-          debug
-          safety
-          space
-          speed
-          ;; SBCL extensions
-          ;;
-          ;; FIXME: INHIBIT-WARNINGS is a misleading name for this.
-          ;; Perhaps BREVITY would be better. But the ideal name would
-          ;; have connotations of suppressing not warnings but only
-          ;; optimization-related notes, which is already mostly the
-          ;; behavior, and should probably become the exact behavior.
-          ;; Perhaps INHIBIT-NOTES?
-          inhibit-warnings))
+(declaim (simple-vector **policy-dependent-qualities**))
 (defglobal **policy-dependent-qualities** #())
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defconstant n-policy-primary-qualities (length **policy-primary-qualities**))
-  ;; 1 bit per quality is stored to indicate whether it was explicitly given
-  ;; a value in a lexical policy. In addition to the 5 ANSI-standard qualities,
-  ;; SBCL defines one more "primary" quality and 16 dependent qualities.
-  ;; Both kinds take up 1 bit in the mask of specified qualities.
-  (defconstant max-policy-qualities 32))
-
-;; Each primary and dependent quality policy is assigned a small integer index.
-;; The POLICY struct represents a set of policies in an order-insensitive way
-;; that facilitates quicker lookup than scanning an alist.
-(defstruct (policy (:constructor make-policy
-                                 (primary-qualities &optional presence-bits)))
-  ;; Mask with a 1 for each quality that has an explicit value in this policy.
-  ;; Primary qualities fill the mask from left-to-right and dependent qualities
-  ;; from right-to-left.
-  ;; xc has trouble folding this MASK-FIELD, but it works when host-evaluated.
-  (presence-bits #.(mask-field
-                    (byte n-policy-primary-qualities
-                          (- max-policy-qualities n-policy-primary-qualities))
-                    -1)
-                 :type (unsigned-byte #.max-policy-qualities))
-  ;; For efficiency, primary qualities are segregated because there are few
-  ;; enough of them to fit in a fixnum.
-  (primary-qualities 0 :type (unsigned-byte #.(* 2 n-policy-primary-qualities)))
-  ;; 2 bits per dependent quality is a fixnum on 64-bit build, not on 32-bit.
-  ;; It would certainly be possible to constrain this to storing exactly
-  ;; the 16 currently defined dependent qualities,
-  ;; but that would be overly limiting.
-  (dependent-qualities 0
-   :type (unsigned-byte #.(* (- max-policy-qualities n-policy-primary-qualities)
-                             2))))
 
 ;; Return POLICY as a list suitable to the OPTIMIZE declaration.
 ;; If FORCE-ALL then include qualities without an explicit value too.
@@ -129,11 +88,11 @@ EXPERIMENTAL INTERFACE: Subject to change."
         when (or force-all (logbitp (mod index max-policy-qualities) presence))
         collect
        (list (if (minusp index)
-                 (elt **policy-primary-qualities** (lognot index))
+                 (elt +policy-primary-qualities+ (lognot index))
                  (policy-dependent-quality-name
                   (elt **policy-dependent-qualities** index)))
              (if raw
-                 ;; Raw values are insensitive to *POLICY-RESTRICTIONS*.
+                 ;; Raw values are insensitive to *POLICY-MIN/MAX*.
                  (%%policy-quality policy index)
                  ;; Otherwise take the adjusted quality.
                  (%policy-quality policy index)))))
@@ -154,7 +113,7 @@ EXPERIMENTAL INTERFACE: Subject to change."
 ;;; If it is, return the integer identifier for the quality name.
 (defun policy-quality-name-p (x)
   ;; Standard (and non-standard) primary qualities are numbered from -1 down.
-  (or (awhen (position x **policy-primary-qualities** :test #'eq)
+  (or (awhen (position x +policy-primary-qualities+ :test #'eq)
         (lognot it))
       ;; Dependent qualities are numbered from 0 up.
       (position x **policy-dependent-qualities**
@@ -194,7 +153,7 @@ EXPERIMENTAL INTERFACE: Subject to change."
 ;;; because for deterministic comparison the list was always freshly
 ;;; consed so that destructive sorting could be done for canonicalization.
 (declaim (type policy *policy*)
-         (type (or policy null) *policy-restrictions*))
+         (type (or policy null) *policy-min* *policy-max*))
 
 ;; ANSI-specified default of 1 for each quality.
 (defglobal **baseline-policy** nil)
@@ -240,19 +199,21 @@ EXPERIMENTAL INTERFACE: Subject to change."
          (t
           form)))
 
-(macrolet ((extract-field (floor-expression-primary
-                           floor-expression-dependent)
+(macrolet ((extract-field (min-expression-primary min-expression-dependent
+                           max-expression-primary max-expression-dependent)
              `(if (minusp index)
                   (let ((byte-pos (* (lognot index) 2)))
-                    (max (ldb (byte 2 byte-pos)
-                              (policy-primary-qualities policy))
-                         ,floor-expression-primary))
+                    (min ,max-expression-primary
+                         (max ,min-expression-primary
+                              (ldb (byte 2 byte-pos)
+                                   (policy-primary-qualities policy)))))
                   (let ((byte-pos (* index 2)))
-                    (max (if (logbitp index (policy-presence-bits policy))
-                             (ldb (byte 2 byte-pos)
-                                  (policy-dependent-qualities policy))
-                             1)
-                         ,floor-expression-dependent))))
+                    (min ,max-expression-dependent
+                         (max ,min-expression-dependent
+                              (if (logbitp index (policy-presence-bits policy))
+                                  (ldb (byte 2 byte-pos)
+                                       (policy-dependent-qualities policy))
+                                  1))))))
            (define-getter (name &body body)
              `(defun ,name (policy index)
                 (declare (type policy policy)
@@ -263,38 +224,48 @@ EXPERIMENTAL INTERFACE: Subject to change."
                                index))
                 ,@body)))
 
-  ;; Return the value for quality INDEX in POLICY, using *POLICY-RESTRICTIONS*
+  ;; Return the value for quality INDEX in POLICY, using *POLICY-MIN/MAX*
   ;; Primary qualities are assumed to exist, however policy-restricting functions
   ;; can create a POLICY that indicates absence of primary qualities.
   ;; This does not affect RESTRICT-COMPILER-POLICY because a lower bound of 0
   ;; can be assumed for everything. SET-MACRO-POLICY might care though.
   (define-getter %policy-quality
-    (let ((floor *policy-restrictions*))
-      (macrolet ((quality-floor (get-byte)
-                   `(if floor (ldb (byte 2 byte-pos) (,get-byte floor)) 0)))
-        (extract-field (quality-floor policy-primary-qualities)
-                       (quality-floor policy-dependent-qualities)))))
+    (let ((min *policy-min*)
+          (max *policy-max*))
+      (macrolet ((quality-min (get-byte)
+                   `(if min
+                        (ldb (byte 2 byte-pos) (,get-byte min))
+                        0))
+                 (quality-max (get-byte)
+                   `(if max
+                        (ldb (byte 2 byte-pos) (,get-byte max))
+                        3)))
+        (extract-field (quality-min policy-primary-qualities)
+                       (quality-min policy-dependent-qualities)
+                       (quality-max policy-primary-qualities)
+                       (quality-max policy-dependent-qualities)))))
 
   ;; Return the unadjusted value for quality INDEX in POLICY.
   ;; This is used for converting a policy to a list of elements for display
   ;; and for verifying that after processing declarations, the new policy
   ;; matches the given declarations, thus implying no ambiguity.
   (define-getter %%policy-quality
-    (extract-field 0 0))) ; floor is always 0
+    (extract-field 0 0 3 3)))
 
 ;;; Forward declaration of %COERCE-TO-POLICY.
 ;;; Definition is in 'node' so that FUNCTIONAL and NODE types are defined.
 ;;; Arg is declared of type T because the function explicitly checks it.
-(declaim (ftype (function (t) (values policy &optional)) %coerce-to-policy))
+(declaim (ftype (sfunction (t) policy) %coerce-to-policy))
 
 ;;; syntactic sugar for querying optimization policy qualities
 ;;;
 ;;; Evaluate EXPR in terms of the optimization policy associated with
 ;;; THING. EXPR is a form which accesses optimization qualities by
 ;;; referring to them by name, e.g. (> SPEED SPACE).
-(defmacro policy (thing expr)
-  (let* ((n-policy (gensym "N-POLICY-"))
-         (binds (loop for name across **policy-primary-qualities**
+(defmacro policy (thing expr &optional (coercion-fn '%coerce-to-policy))
+  (declare (notinline identity)) ; constant-folding-error otherwise. (FIXME)
+  (let* ((n-policy (make-symbol "P"))
+         (binds (loop for name across (identity +policy-primary-qualities+)
                       for index downfrom -1
                       collect `(,name (%policy-quality ,n-policy ,index))))
          (dependent-binds
@@ -304,10 +275,11 @@ EXPERIMENTAL INTERFACE: Subject to change."
                                  (if (= ,name 1)
                                      ,(policy-dependent-quality-expression info)
                                      ,name))))))
-    `(let* ((,n-policy (%coerce-to-policy ,thing)))
+    `(let ((,n-policy (,coercion-fn ,thing)))
+       ;; FIXME: automatically inserted IGNORABLE decls are
+       ;; often suggestive of poor style, as is this one.
        (declare (ignorable ,n-policy))
-       (symbol-macrolet (,@binds
-                         ,@dependent-binds)
+       (symbol-macrolet (,@binds ,@dependent-binds)
          ,expr))))
 
 ;;; Dependent qualities
@@ -373,6 +345,22 @@ EXPERIMENTAL INTERFACE: Subject to change."
                  (push (cons quality raw-value) specified-qualities)
                  (alter-policy result index raw-value))))))))
 
+;;; Same as above but slightly more efficient
+(defun %augment-policy (quality value policy)
+  (let ((result (copy-policy policy)))
+    (alter-policy result quality value)))
+
+(defmacro augment-policy (quality value policy)
+  (let ((value-sym (gensym "VALUE"))
+        (policy-sym (gensym "POLICY")))
+    `(let ((,value-sym ,value)
+           (,policy-sym ,policy))
+       (if (policy ,policy-sym (> ,quality ,value-sym))
+           (%augment-policy ,(policy-quality-name-p quality)
+                            ,value-sym
+                            ,policy-sym)
+           ,policy-sym))))
+
 (defvar *macro-policy* nil)
 ;; Set an alternate policy that is used to compile all code within DEFMACRO,
 ;; MACROLET, DEFINE-COMPILER-MARO - whether they occur at toplevel or not -
