@@ -13,7 +13,7 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(reg-tn-encoding) 'sb!vm)
+  (import '(reg-tn-encoding) "SB!VM")
   ;; Imports from SB-VM into this package
   (import '(;; SBs and SCs
             sb!vm::zero sb!vm::immediate-constant
@@ -24,6 +24,8 @@
             sb!vm::zero-tn
             sb!vm::zero-offset sb!vm::null-offset sb!vm::alloc-offset)))
 
+(defconstant +disassem-inst-alignment-bytes+ sb!vm:n-word-bytes)
+
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (setf *assem-scheduler-p* t)
   (setf *assem-max-locations* 100))
@@ -54,7 +56,6 @@
            (tn-offset tn)))))
 
 (defvar *disassem-use-lisp-reg-names* t
-  #!+sb-doc
   "If non-NIL, print registers using the Lisp register names.
 Otherwise, use the Sparc register names")
 
@@ -99,14 +100,14 @@ Otherwise, use the Sparc register names")
            (cond ((null name) nil)
                  (t (make-symbol (concatenate 'string "%" name)))))
        sb!vm::*register-names*)
-  #!+sb-doc "The Lisp names for the Sparc integer registers")
+  "The Lisp names for the Sparc integer registers")
 
 (defparameter sparc-reg-symbols
   #("%G0" "%G1" "%G2" "%G3" "%G4" "%G5" NIL NIL
     "%O0" "%O1" "%O2" "%O3" "%O4" "%O5" "%O6" "%O7"
     "%L0" "%L1" "%L2" "%L3" "%L4" "%L5" "%L6" "%L7"
     "%I0" "%I1" "%I2" "%I3" "%I4" "%I5" NIL "%I7")
-  #!+sb-doc "The standard names for the Sparc integer registers")
+  "The standard names for the Sparc integer registers")
 
 (defun get-reg-name (index)
   (if *disassem-use-lisp-reg-names*
@@ -738,7 +739,6 @@ about function addresses and register values.")
                                 op3 (reg-tn-encoding src1) 1
                                 (if extended 1 0) src2))))
 
-;;; have to do this because def!constant is evalutated in the null lex env.
 (defmacro with-ref-format (printer)
   `(let* ((addend
            '(:choose (:plus-integer immed) ("+" rs2)))
@@ -1598,11 +1598,16 @@ about function addresses and register values.")
 ;;;; Instructions for dumping data and header objects.
 
 (define-instruction word (segment word)
-  (:declare (type (or (unsigned-byte 32) (signed-byte 32)) word))
+  (:declare (type (or (unsigned-byte 32) (signed-byte 32) fixup) word))
   :pinned
   (:delay 0)
   (:emitter
-   (emit-word segment word)))
+   (etypecase word
+     (fixup
+      (note-fixup segment :absolute word)
+      (emit-word segment 0))
+     (integer
+      (emit-word segment word)))))
 
 (define-instruction short (segment short)
   (:declare (type (or (unsigned-byte 16) (signed-byte 16)) short))
@@ -1634,13 +1639,13 @@ about function addresses and register values.")
   :pinned
   (:delay 0)
   (:emitter
-   (emit-header-data segment simple-fun-header-widetag)))
+   (emit-header-data segment simple-fun-widetag)))
 
 (define-instruction lra-header-word (segment)
   :pinned
   (:delay 0)
   (:emitter
-   (emit-header-data segment return-pc-header-widetag)))
+   (emit-header-data segment return-pc-widetag)))
 
 
 ;;;; Instructions for converting between code objects, functions, and lras.
