@@ -32,7 +32,7 @@
     (inst lea bytes
           (make-ea :qword
                    :index rank :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                   :disp (+ (* (1+ array-dimensions-offset) n-word-bytes)
+                   :disp (+ (* array-dimensions-offset n-word-bytes)
                             lowtag-mask)))
     (inst and bytes (lognot lowtag-mask))
     (inst lea header (make-ea :qword :base rank
@@ -151,25 +151,29 @@
   (:policy :fast-safe)
   (:args (array :scs (descriptor-reg))
          (bound :scs (any-reg descriptor-reg))
-         (index :scs (any-reg descriptor-reg) :target result))
-;  (:arg-types * positive-fixnum tagged-num)
-  (:results (result :scs (any-reg descriptor-reg)))
- ; (:result-types positive-fixnum)
+         (index :scs (any-reg descriptor-reg)))
+  ;(:arg-types * positive-fixnum *)
+  (:variant-vars %test-fixnum)
+  (:variant t)
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 5
     (let ((error (generate-error-code vop 'invalid-array-index-error
                                       array bound index))
           (index (if (sc-is index immediate)
-                   (fixnumize (tn-value index))
-                   index)))
+                     (fixnumize (tn-value index))
+                     index)))
+      (when (and %test-fixnum (not (integerp index)))
+        (%test-fixnum index error t))
       (inst cmp bound index)
       ;; We use below-or-equal even though it's an unsigned test,
       ;; because negative indexes appear as large unsigned numbers.
       ;; Therefore, we get the <0 and >=bound test all rolled into one.
-      (inst jmp :be error)
-      (unless (and (tn-p index) (location= result index))
-        (inst mov result index)))))
+      (inst jmp :be error))))
+(define-vop (check-bound/fast check-bound)
+  (:policy :fast)
+  (:variant nil)
+  (:variant-cost 4))
 
 ;;;; accessors/setters
 
@@ -187,7 +191,7 @@
                   ,type vector-data-offset other-pointer-lowtag ,scs
                   ,element-type data-vector-set-with-offset)))
            )
-  (def-full-data-vector-frobs simple-vector * descriptor-reg any-reg)
+  (def-full-data-vector-frobs simple-vector * descriptor-reg any-reg immediate)
   (def-full-data-vector-frobs simple-array-unsigned-byte-64 unsigned-num
     unsigned-reg)
   (def-full-data-vector-frobs simple-array-fixnum tagged-num any-reg)
@@ -855,3 +859,26 @@
                                  other-pointer-lowtag))
           diff :lock)
     (move result diff)))
+
+(defknown layout-inherits-ref-eq (simple-vector index t) boolean (flushable))
+(define-vop (layout-inherits-ref-eq)
+  (:translate layout-inherits-ref-eq)
+  (:policy :fast-safe)
+  (:conditional :e)
+  (:args (vector :scs (descriptor-reg))
+         (index :scs (any-reg descriptor-reg immediate))
+         (thing :scs (descriptor-reg immediate)))
+  (:generator 1
+    (inst cmp
+          (if (sc-is index immediate)
+              (make-ea :dword :base vector
+                       :disp (+ (- other-pointer-lowtag)
+                                (ash (+ vector-data-offset (tn-value index))
+                                     word-shift)))
+              (make-ea :dword :base vector
+                       :disp (+ (- other-pointer-lowtag)
+                                (ash vector-data-offset word-shift))
+                       :index index :scale (ash 1 (- word-shift n-fixnum-tag-bits))))
+          (if (sc-is thing immediate)
+              (make-fixup (tn-value thing) :layout)
+              thing))))
