@@ -17,11 +17,6 @@
 ;; used for things going down the stack but C wants to have args
 ;; indexed in the positive direction.
 
-(defun my-make-wired-tn (prim-type-name sc-name offset)
-  (make-wired-tn (primitive-type-or-lose prim-type-name)
-                 (sc-number-or-lose sc-name)
-                 offset))
-
 (defstruct (arg-state (:copier nil))
   (register-args 0)
   (xmm-args 0)
@@ -35,41 +30,41 @@
                        #!+win32 (arg-state-xmm-args state))))
     (cond ((< reg-args max-int-args)
            (setf (arg-state-register-args state) (1+ reg-args))
-           (my-make-wired-tn prim-type reg-sc
+           (make-wired-tn* prim-type reg-sc
                              (nth reg-args *c-call-register-arg-offsets*)))
           (t
            (let ((frame-size (arg-state-stack-frame-size state)))
              (setf (arg-state-stack-frame-size state) (1+ frame-size))
-             (my-make-wired-tn prim-type stack-sc frame-size))))))
+             (make-wired-tn* prim-type stack-sc frame-size))))))
 
 (define-alien-type-method (integer :arg-tn) (type state)
   (if (alien-integer-type-signed type)
-      (int-arg state 'signed-byte-64 'signed-reg 'signed-stack)
-      (int-arg state 'unsigned-byte-64 'unsigned-reg 'unsigned-stack)))
+      (int-arg state 'signed-byte-64 signed-reg-sc-number signed-stack-sc-number)
+      (int-arg state 'unsigned-byte-64 unsigned-reg-sc-number unsigned-stack-sc-number)))
 
 (define-alien-type-method (system-area-pointer :arg-tn) (type state)
   (declare (ignore type))
-  (int-arg state 'system-area-pointer 'sap-reg 'sap-stack))
+  (int-arg state 'system-area-pointer sap-reg-sc-number sap-stack-sc-number))
 
 (defun float-arg (state prim-type reg-sc stack-sc)
   (let ((xmm-args (max (arg-state-xmm-args state)
                         #!+win32 (arg-state-register-args state))))
     (cond ((< xmm-args max-xmm-args)
            (setf (arg-state-xmm-args state) (1+ xmm-args))
-           (my-make-wired-tn prim-type reg-sc
+           (make-wired-tn* prim-type reg-sc
                              (nth xmm-args *float-regs*)))
           (t
            (let ((frame-size (arg-state-stack-frame-size state)))
              (setf (arg-state-stack-frame-size state) (1+ frame-size))
-             (my-make-wired-tn prim-type stack-sc frame-size))))))
+             (make-wired-tn* prim-type stack-sc frame-size))))))
 
 (define-alien-type-method (double-float :arg-tn) (type state)
   (declare (ignore type))
-  (float-arg state 'double-float 'double-reg 'double-stack))
+  (float-arg state 'double-float double-reg-sc-number double-stack-sc-number))
 
 (define-alien-type-method (single-float :arg-tn) (type state)
   (declare (ignore type))
-  (float-arg state 'single-float 'single-reg 'single-stack))
+  (float-arg state 'single-float single-reg-sc-number single-stack-sc-number))
 
 (defstruct (result-state (:copier nil))
   (num-results 0))
@@ -84,9 +79,9 @@
     (setf (result-state-num-results state) (1+ num-results))
     (multiple-value-bind (ptype reg-sc)
         (if (alien-integer-type-signed type)
-            (values 'signed-byte-64 'signed-reg)
-            (values 'unsigned-byte-64 'unsigned-reg))
-      (my-make-wired-tn ptype reg-sc (result-reg-offset num-results)))))
+            (values 'signed-byte-64 signed-reg-sc-number)
+            (values 'unsigned-byte-64 unsigned-reg-sc-number))
+      (make-wired-tn* ptype reg-sc (result-reg-offset num-results)))))
 
 (define-alien-type-method (integer :naturalize-gen) (type alien)
   (if (<= (alien-type-bits type) 32)
@@ -99,20 +94,20 @@
   (declare (ignore type))
   (let ((num-results (result-state-num-results state)))
     (setf (result-state-num-results state) (1+ num-results))
-    (my-make-wired-tn 'system-area-pointer 'sap-reg
+    (make-wired-tn* 'system-area-pointer sap-reg-sc-number
                       (result-reg-offset num-results))))
 
 (define-alien-type-method (double-float :result-tn) (type state)
   (declare (ignore type))
   (let ((num-results (result-state-num-results state)))
     (setf (result-state-num-results state) (1+ num-results))
-    (my-make-wired-tn 'double-float 'double-reg num-results)))
+    (make-wired-tn* 'double-float double-reg-sc-number num-results)))
 
 (define-alien-type-method (single-float :result-tn) (type state)
   (declare (ignore type))
   (let ((num-results (result-state-num-results state)))
     (setf (result-state-num-results state) (1+ num-results))
-    (my-make-wired-tn 'single-float 'single-reg num-results)))
+    (make-wired-tn* 'single-float single-reg-sc-number num-results)))
 
 (define-alien-type-method (values :result-tn) (type state)
   (let ((values (alien-values-type-values type)))
@@ -127,7 +122,7 @@
     (collect ((arg-tns))
       (dolist (arg-type (alien-fun-type-arg-types type))
         (arg-tns (invoke-alien-type-method :arg-tn arg-type arg-state)))
-      (values (my-make-wired-tn 'positive-fixnum 'any-reg esp-offset)
+      (values (make-wired-tn* 'positive-fixnum any-reg-sc-number esp-offset)
               (* (arg-state-stack-frame-size arg-state) n-word-bytes)
               (arg-tns)
               (invoke-alien-type-method :result-tn
@@ -257,6 +252,9 @@
   (:generator 2
    (inst mov res (make-fixup foreign-symbol :foreign-dataref))))
 
+#!+sb-safepoint
+(defconstant thread-saved-csp-offset -1)
+
 (define-vop (call-out)
   (:args (function :scs (sap-reg)
                    :target rbx)
@@ -266,63 +264,79 @@
   ;; determine which alien was accessed in case it's undefined.
   (:temporary (:sc sap-reg :offset rbx-offset :from (:argument 0)) rbx)
   (:temporary (:sc unsigned-reg :offset rax-offset :to :result) rax)
-  ;; For safepoint builds: Force values of non-volatiles to the stack.
-  ;; These are the callee-saved registers in the native ABI, but
-  ;; safepoint-based GC needs to see all Lisp values on the stack.  Note
-  ;; that R12-R15 are non-volatile registers, but there is no need to
-  ;; spill R12 because it is our thread-base-tn.  RDI and RSI are
-  ;; non-volatile on Windows, but argument passing registers on other
-  ;; platforms.
-  #!+sb-safepoint (:temporary (:sc unsigned-reg :offset r13-offset) r13)
-  #!+sb-safepoint (:temporary (:sc unsigned-reg :offset r14-offset) r14)
-  #!+sb-safepoint (:temporary (:sc unsigned-reg :offset r15-offset) r15)
-  #!+(and sb-safepoint win32) (:temporary
-                               (:sc unsigned-reg :offset rdi-offset) rdi)
-  #!+(and sb-safepoint win32) (:temporary
-                               (:sc unsigned-reg :offset rsi-offset) rsi)
-  (:ignore results
-           #!+(and sb-safepoint win32) rdi
-           #!+(and sb-safepoint win32) rsi
-           #!+win32 args
-           #!+win32 rax
-           #!+sb-safepoint r15
-           #!+sb-safepoint r13)
+  #!+sb-safepoint
+  (:temporary (:sc unsigned-stack :from :eval :to :result) pc-save)
+  (:ignore results)
   (:vop-var vop)
   (:save-p t)
   (:generator 0
-    #!+sb-safepoint
-    (progn
-      ;; Current PC - don't rely on function to keep it in a form that
-      ;; GC understands
-      (let ((label (gen-label)))
-        (inst lea r14 (make-fixup nil :code-object label))
-        (emit-label label)))
-    #!-win32
-    ;; ABI: AL contains amount of arguments passed in XMM registers
-    ;; for vararg calls.
-    (move-immediate rax
-                    (loop for tn-ref = args then (tn-ref-across tn-ref)
-                          while tn-ref
-                          count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
-                                    'float-registers)))
-    #!+win32 (inst sub rsp-tn #x20) ;MS_ABI: shadow zone
-    #!+sb-safepoint
-    (progn                 ;Store SP and PC in thread struct
-      (storew rsp-tn thread-base-tn thread-saved-csp-offset)
-      (storew r14 thread-base-tn thread-pc-around-foreign-call-slot))
     (move rbx function)
-    (inst call rbx)
-    #!+win32 (inst add rsp-tn #x20) ;MS_ABI: remove shadow space
-    #!+sb-safepoint
-    (progn
-      ;; Zeroing out
-      (inst xor r14 r14)
-      ;; Zero PC storage place. NB. CSP-then-PC: same sequence on
-      ;; entry/exit, is actually corrent.
-      (storew r14 thread-base-tn thread-saved-csp-offset)
-      (storew r14 thread-base-tn thread-pc-around-foreign-call-slot))
-    ;; To give the debugger a clue. XX not really internal-error?
-    (note-this-location vop :internal-error)))
+    (emit-c-call vop rax rbx args
+                 sb!alien::*alien-fun-type-varargs-default*
+                 #!+sb-safepoint pc-save)))
+
+;;; Calls to C can generally be made without loading a register
+;;; with the function. We receive the function name as an info argument.
+#!+sb-dynamic-core ;; broken when calling ldso-stubs
+(define-vop (call-out-named)
+  (:args (args :more t))
+  (:results (results :more t))
+  (:info c-symbol varargsp)
+  (:temporary (:sc unsigned-reg :offset rax-offset :to :result) rax)
+  #!+sb-safepoint
+  (:temporary (:sc unsigned-stack :from :eval :to :result) pc-save)
+  (:ignore results)
+  (:vop-var vop)
+  (:save-p t)
+  (:generator 0
+    (emit-c-call vop rax c-symbol args varargsp #!+sb-safepoint pc-save)))
+
+(defun emit-c-call (vop rax fun args varargsp #!+sb-safepoint pc-save)
+  (declare (ignorable varargsp))
+  ;; Current PC - don't rely on function to keep it in a form that
+  ;; GC understands
+  #!+sb-safepoint
+  (let ((label (gen-label)))
+    (inst lea (reg-in-size rax :immobile-code-pc) (make-fixup nil :code-object label))
+    (emit-label label)
+    (move-dword-if-immobile-code pc-save rax))
+  (when sb!c::*msan-unpoison*
+    (inst mov rax (thread-tls-ea (ash thread-msan-param-tls-slot word-shift)))
+    ;; Unpoison parameters
+    (do ((n 0 (+ n n-word-bytes))
+         (arg args (tn-ref-across arg)))
+        ((null arg))
+      ;; KLUDGE: assume all parameters are 8 bytes or less
+      (inst mov (make-ea :qword :base rax :disp n) 0)))
+  #!-win32
+  ;; ABI: AL contains amount of arguments passed in XMM registers
+  ;; for vararg calls.
+  (when varargsp
+    (move-immediate rax
+                  (loop for tn-ref = args then (tn-ref-across tn-ref)
+                        while tn-ref
+                        count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
+                                  'float-registers))))
+  #!+sb-safepoint
+  ;; Store SP in thread struct
+  (storew rsp-tn thread-base-tn thread-saved-csp-offset)
+  #!+win32 (inst sub rsp-tn #x20)       ;MS_ABI: shadow zone
+  ;; From immobile space we use the "CALL rel32" format to the linkage
+  ;; table jump, and from dynamic space we use "CALL [ea]" format
+  ;; where ea is the address of the linkage table entry's operand.
+  ;; So while the former is a jump to a jump, we can optimize out
+  ;; one jump in a statically linked executable.
+
+  (inst call (cond ((tn-p fun) fun)
+                   ((sb!c::code-immobile-p vop) (make-fixup fun :foreign))
+                   (t (make-ea :qword :disp (make-fixup fun :foreign 8)))))
+  ;; For the undefined alien error
+  (note-this-location vop :internal-error)
+  #!+win32 (inst add rsp-tn #x20)       ;MS_ABI: remove shadow space
+  #!+sb-safepoint
+  ;; Zero the saved CSP
+  (inst xor (make-ea-for-object-slot thread-base-tn thread-saved-csp-offset 0)
+        rsp-tn))
 
 (define-vop (alloc-number-stack-space)
   (:info amount)
@@ -397,7 +411,8 @@
                          (subseq *float-regs* 0 #!-win32 8 #!+win32 4))))
       (assemble (segment)
         ;; Make room on the stack for arguments.
-        (inst sub rsp (* n-word-bytes (length argument-types)))
+        (when argument-types
+          (inst sub rsp (* n-word-bytes (length argument-types))))
         ;; Copy arguments from registers to stack
         (dolist (type argument-types)
           (let ((integerp (not (alien-float-type-p type)))
@@ -447,15 +462,7 @@
         #!-sb-thread
         (progn
           ;; arg0 to FUNCALL3 (function)
-          ;;
-          ;; Indirect the access to ENTER-ALIEN-CALLBACK through
-          ;; the symbol-value slot of SB-ALIEN::*ENTER-ALIEN-CALLBACK*
-          ;; to ensure it'll work even if the GC moves ENTER-ALIEN-CALLBACK.
-          ;; Skip any SB-THREAD TLS magic, since we don't expect anyone
-          ;; to rebind the variable. -- JES, 2006-01-01
-          (inst mov rdi (+ nil-value (static-symbol-offset
-                                      'sb!alien::*enter-alien-callback*)))
-          (loadw rdi rdi symbol-value-slot other-pointer-lowtag)
+          (inst mov rdi (make-ea :qword :disp (static-fdefn-fun-addr 'enter-alien-callback)))
           ;; arg0 to ENTER-ALIEN-CALLBACK (trampoline index)
           (inst mov rsi (fixnumize index))
           ;; arg1 to ENTER-ALIEN-CALLBACK (pointer to argument vector)
