@@ -11,8 +11,8 @@
 
 (in-package "SB!VM")
 
-(defconstant arg-count-sc (make-sc-offset any-reg-sc-number rcx-offset))
-(defconstant closure-sc (make-sc-offset any-reg-sc-number rax-offset))
+(defconstant arg-count-sc (make-sc+offset any-reg-sc-number rcx-offset))
+(defconstant closure-sc (make-sc+offset any-reg-sc-number rax-offset))
 
 ;;; Make a passing location TN for a local call return PC.
 ;;;
@@ -24,7 +24,7 @@
                  sap-stack-sc-number return-pc-save-offset))
 
 (defconstant return-pc-passing-offset
-  (make-sc-offset sap-stack-sc-number return-pc-save-offset))
+  (make-sc+offset sap-stack-sc-number return-pc-save-offset))
 
 ;;; This is similar to MAKE-RETURN-PC-PASSING-LOCATION, but makes a
 ;;; location to pass OLD-FP in.
@@ -33,13 +33,12 @@
 ;;; because we want to be able to assume it's always there. Besides,
 ;;; the x86 doesn't have enough registers to really make it profitable
 ;;; to pass it in a register.
-(defun make-old-fp-passing-location (standard)
-  (declare (ignore standard))
+(defun make-old-fp-passing-location ()
   (make-wired-tn *fixnum-primitive-type* control-stack-sc-number
                  ocfp-save-offset))
 
 (defconstant old-fp-passing-offset
-  (make-sc-offset control-stack-sc-number ocfp-save-offset))
+  (make-sc+offset control-stack-sc-number ocfp-save-offset))
 
 ;;; Make the TNs used to hold OLD-FP and RETURN-PC within the current
 ;;; function. We treat these specially so that the debugger can find
@@ -186,13 +185,16 @@
     ;; Save the return-pc.
     (popw rbp-tn (frame-word-offset return-pc-save-offset))))
 
+(defun emit-lea (target source disp)
+  (if (eql disp 0)
+      (inst mov target source)
+      (inst lea target (make-ea :qword :base source :disp disp))))
+
 (define-vop (xep-setup-sp)
   (:generator 1
-    (inst lea rsp-tn
-          (make-ea :qword :base rbp-tn
-                          :disp (- (* n-word-bytes
+    (emit-lea rsp-tn rbp-tn     (- (* n-word-bytes
                                       (- (sb-allocated-size 'stack)
-                                         sp->fp-offset)))))))
+                                         sp->fp-offset))))))
 
 ;;; This is emitted directly before either a known-call-local, call-local,
 ;;; or a multiple-call-local. All it does is allocate stack space for the
@@ -207,18 +209,39 @@
                            :disp (- (* sp->fp-offset n-word-bytes))))
     (inst sub rsp-tn (* n-word-bytes (sb-allocated-size 'stack)))))
 
+(defun make-stack-pointer-tn (&optional nargs)
+  ;; Avoid using a temporary register if the new frame pointer will be
+  ;; at the same location as the new stack pointer
+  (if (and nargs
+           (= (* sp->fp-offset n-word-bytes)
+              (* (max (if (> nargs register-arg-count)
+                          nargs
+                          0)
+                      (sb!c::sb-size (sb-or-lose 'stack)))
+                 n-word-bytes)))
+      (make-wired-tn *fixnum-primitive-type* any-reg-sc-number rsp-offset)
+      (make-normal-tn *fixnum-primitive-type*)))
+
 ;;; Allocate a partial frame for passing stack arguments in a full
 ;;; call. NARGS is the number of arguments passed. We allocate at
-;;; least 3 slots, because the XEP noise is going to want to use them
+;;; least 2 slots, because the XEP noise is going to want to use them
 ;;; before it can extend the stack.
 (define-vop (allocate-full-call-frame)
   (:info nargs)
   (:results (res :scs (any-reg)))
   (:generator 2
-    (inst lea res (make-ea :qword :base rsp-tn
-                           :disp (- (* sp->fp-offset n-word-bytes))))
-    (inst sub rsp-tn (* (max nargs (sb!c::sb-size (sb-or-lose 'stack)))
-                        n-word-bytes))))
+    (let ((fp-offset (* sp->fp-offset n-word-bytes))
+          (stack-size (* (max (if (> nargs register-arg-count)
+                                  nargs
+                                  0)
+                              (sb!c::sb-size (sb-or-lose 'stack)))
+                         n-word-bytes)))
+      (cond ((= fp-offset stack-size)
+             (inst sub rsp-tn stack-size)
+             (move res rsp-tn))
+            (t
+             (inst lea res (make-ea :qword :base rsp-tn :disp (- fp-offset)))
+             (inst sub rsp-tn stack-size))))))
 
 ;;; Emit code needed at the return-point from an unknown-values call
 ;;; for a fixed number of values. Values is the head of the TN-REF
@@ -265,159 +288,72 @@
            (inst jmp :c regs-defaulted)
            ;; Default the unsupplied registers.
            (let* ((2nd-tn-ref (tn-ref-across values))
-                  (2nd-tn (tn-ref-tn 2nd-tn-ref)))
-             (inst mov 2nd-tn nil-value)
+                  (2nd-tn (tn-ref-tn 2nd-tn-ref))
+                  (2nd-tn-live (neq (tn-kind 2nd-tn) :unused)))
+             (when 2nd-tn-live
+               (inst mov 2nd-tn nil-value))
              (when (> nvals 2)
                (loop
-                for tn-ref = (tn-ref-across 2nd-tn-ref)
-                then (tn-ref-across tn-ref)
-                for count from 2 below register-arg-count
-                do (inst mov (tn-ref-tn tn-ref) 2nd-tn))))
+                 for tn-ref = (tn-ref-across 2nd-tn-ref)
+                 then (tn-ref-across tn-ref)
+                 for count from 2 below register-arg-count
+                 unless (eq (tn-kind (tn-ref-tn tn-ref)) :unused)
+                 do
+                 (inst mov (reg-in-size (tn-ref-tn tn-ref) :dword)
+                       (if 2nd-tn-live
+                           (reg-in-size 2nd-tn :dword)
+                           nil-value)))))
            (inst mov rbx-tn rsp-tn)
            (emit-label regs-defaulted)))
        (when (< register-arg-count
                 (sb!kernel:values-type-max-value-count type))
          (inst mov rsp-tn rbx-tn)))
-      ((<= nvals 7)
-       ;; The number of bytes depends on the relative jump instructions.
-       ;; Best case is 31+(n-3)*14, worst case is 35+(n-3)*18. For
-       ;; NVALS=6 that is 73/89 bytes, and for NVALS=7 that is 87/107
-       ;; bytes which is likely better than using the blt below.
-       (let ((regs-defaulted (gen-label))
-             (defaulting-done (gen-label))
-             (default-stack-slots (gen-label)))
-         (note-this-location vop :unknown-return)
-         (inst mov rax-tn nil-value)
-         ;; Branch off to the MV case.
-         (inst jmp :c regs-defaulted)
-         ;; Do the single value case.
-         ;; Default the register args
-         (do ((i 1 (1+ i))
-              (val (tn-ref-across values) (tn-ref-across val)))
-             ((= i (min nvals register-arg-count)))
-           (inst mov (tn-ref-tn val) rax-tn))
-         ;; Fake other registers so it looks like we returned with all the
-         ;; registers filled in.
-         (move rbx-tn rsp-tn)
-         (inst jmp default-stack-slots)
-         (emit-label regs-defaulted)
-         (collect ((defaults))
-           (do ((i register-arg-count (1+ i))
-                (val (do ((i 0 (1+ i))
-                          (val values (tn-ref-across val)))
-                         ((= i register-arg-count) val))
-                     (tn-ref-across val)))
-               ((null val))
-             (let ((default-lab (gen-label))
-                   (tn (tn-ref-tn val))
-                   (first-stack-arg-p (= i register-arg-count)))
-               (defaults (cons default-lab
-                               (cons tn first-stack-arg-p)))
-               (inst cmp rcx-tn (fixnumize i))
-               (inst jmp :be default-lab)
-               (when first-stack-arg-p
-                 ;; There are stack args so the frame of the callee is
-                 ;; still there, save RDX in its first slot temporalily.
-                 (storew rdx-tn rbx-tn (frame-word-offset sp->fp-offset)))
-               (loadw rdx-tn rbx-tn (frame-word-offset (+ sp->fp-offset i)))
-               (inst mov tn rdx-tn)))
-           (emit-label defaulting-done)
-           (loadw rdx-tn rbx-tn (frame-word-offset sp->fp-offset))
-           (move rsp-tn rbx-tn)
-           (let ((defaults (defaults)))
-             (when defaults
-               (assemble (*elsewhere*)
-                 (emit-label default-stack-slots)
-                 (dolist (default defaults)
-                   (emit-label (car default))
-                   (when (cddr default)
-                     ;; We are setting the first stack argument to NIL.
-                     ;; The callee's stack frame is dead, save RDX by
-                     ;; pushing it to the stack, it will end up at same
-                     ;; place as in the (STOREW RDX-TN RBX-TN -1) case
-                     ;; above.
-                     (inst push rdx-tn))
-                   (inst mov (second default) rax-tn))
-                 (inst jmp defaulting-done)))))))
       (t
-       (let ((regs-defaulted (gen-label))
-             (restore-edi (gen-label))
-             (no-stack-args (gen-label))
-             (default-stack-vals (gen-label))
-             (count-okay (gen-label)))
-         (inst std)
-         (note-this-location vop :unknown-return)
-         ;; Branch off to the MV case.
-         (inst jmp :c regs-defaulted)
-         ;; Default the register args, and set up the stack as if we
-         ;; entered the MV return point.
-         (inst mov rbx-tn rsp-tn)
-         (inst mov rdi-tn nil-value)
-         (inst mov rsi-tn rdi-tn)
-         ;; Compute a pointer to where to put the [defaulted] stack values.
-         (emit-label no-stack-args)
-         (inst push rdx-tn)
-         (inst push rdi-tn)
-         (inst lea rdi-tn
-               (make-ea :qword :base rbp-tn
-                        :disp (frame-byte-offset register-arg-count)))
-         ;; Load RAX with NIL so we can quickly store it, and set up
-         ;; stuff for the loop.
-         (inst mov rax-tn nil-value)
-         (inst mov rcx-tn (- nvals register-arg-count))
-         ;; Jump into the default loop.
-         (inst jmp default-stack-vals)
-         ;; The regs are defaulted. We need to copy any stack arguments,
-         ;; and then default the remaining stack arguments.
-         (emit-label regs-defaulted)
-         ;; Compute the number of stack arguments, and if it's zero or
-         ;; less, don't copy any stack arguments.
-         (inst sub rcx-tn (fixnumize register-arg-count))
-         (inst jmp :le no-stack-args)
-         ;; Save EDI.
-         (storew rdi-tn rbx-tn (frame-word-offset (+ sp->fp-offset 1)))
-         ;; Throw away any unwanted args.
-         (inst cmp rcx-tn (fixnumize (- nvals register-arg-count)))
-         (inst jmp :be count-okay)
-         (inst mov rcx-tn (fixnumize (- nvals register-arg-count)))
-         (emit-label count-okay)
-         ;; Save the number of stack values.
-         (inst mov rax-tn rcx-tn)
-         ;; Compute a pointer to where the stack args go.
-         (inst lea rdi-tn
-               (make-ea :qword :base rbp-tn
-                        :disp (frame-byte-offset register-arg-count)))
-         ;; Save ESI, and compute a pointer to where the args come from.
-         (storew rsi-tn rbx-tn (frame-word-offset (+ sp->fp-offset 2)))
-         (inst lea rsi-tn
-               (make-ea :qword :base rbx-tn
-                        :disp (frame-byte-offset
-                               (+ sp->fp-offset register-arg-count))))
-         ;; Do the copy.
-         (inst shr rcx-tn n-fixnum-tag-bits)   ; make word count
-         (inst rep)
-         (inst movs :qword)
-         ;; Restore RSI.
-         (loadw rsi-tn rbx-tn (frame-word-offset (+ sp->fp-offset 2)))
-         ;; Now we have to default the remaining args. Find out how many.
-         (inst sub rax-tn (fixnumize (- nvals register-arg-count)))
-         (inst neg rax-tn)
-         ;; If none, then just blow out of here.
-         (inst jmp :le restore-edi)
-         (inst mov rcx-tn rax-tn)
-         (inst shr rcx-tn n-fixnum-tag-bits)   ; word count
-         ;; Load RAX with NIL for fast storing.
-         (inst mov rax-tn nil-value)
-         ;; Do the store.
-         (emit-label default-stack-vals)
-         (inst rep)
-         (inst stos rax-tn)
-         ;; Restore EDI, and reset the stack.
-         (emit-label restore-edi)
-         (loadw rdi-tn rbx-tn (frame-word-offset (+ sp->fp-offset 1)))
-         (inst mov rsp-tn rbx-tn)
-         (inst cld)))))
-  (values))
+       (collect ((defaults))
+         (let ((default-stack-slots (gen-label)))
+          (assemble ()
+            (note-this-location vop :unknown-return)
+            ;; Branch off to the MV case.
+            (inst jmp :c regs-defaulted)
+            ;; Do the single value case.
+            ;; Default the register args
+            (do ((i 1 (1+ i))
+                 (val (tn-ref-across values) (tn-ref-across val)))
+                ((= i register-arg-count) (setf values val))
+              (unless (eq (tn-kind (tn-ref-tn val)) :unused)
+                (inst mov (tn-ref-tn val) nil-value)))
+            (move rbx-tn rsp-tn)
+            (if (loop for ref = values then (tn-ref-across ref)
+                      while ref
+                      thereis (neq (tn-kind (tn-ref-tn ref)) :unused))
+                (inst jmp default-stack-slots)
+                (inst jmp defaulting-done))
+            REGS-DEFAULTED
+            (do ((i register-arg-count (1+ i))
+                 (val values (tn-ref-across val)))
+                ((null val))
+              (let ((tn (tn-ref-tn val)))
+                (unless (eq (tn-kind tn) :unused)
+                  (let ((default-lab (gen-label)))
+                    (defaults (cons default-lab tn))
+                    (inst cmp ecx-tn (fixnumize i))
+                    (inst jmp :be default-lab)
+                    (sc-case tn
+                      (control-stack
+                       (loadw r11-tn rbx-tn (frame-word-offset (+ sp->fp-offset i)))
+                       (inst mov tn r11-tn))
+                      (t
+                       (loadw tn rbx-tn (frame-word-offset (+ sp->fp-offset i)))))))))
+            DEFAULTING-DONE
+            (move rsp-tn rbx-tn)
+            (let ((defaults (defaults)))
+              (when defaults
+                (assemble (*elsewhere*)
+                  (emit-label default-stack-slots)
+                  (dolist (default defaults)
+                    (emit-label (car default))
+                    (inst mov (cdr default) nil-value))
+                  (inst jmp defaulting-done)))))))))))
 
 ;;;; unknown values receiving
 
@@ -443,7 +379,8 @@
   (let ((type (sb!c::basic-combination-derived-type node))
         (variable-values (gen-label))
         (stack-values (gen-label))
-        (done (gen-label)))
+        (done (gen-label))
+        (unused-count-p (eq (tn-kind count) :unused)))
     (when (sb!kernel:values-type-may-be-single-value-p type)
       (inst jmp :c variable-values)
       (cond ((location= start (first *register-arg-tns*))
@@ -451,7 +388,8 @@
              (inst lea start (make-ea :qword :base rsp-tn :disp n-word-bytes)))
             (t (inst mov start rsp-tn)
                (inst push (first *register-arg-tns*))))
-      (inst mov count (fixnumize 1))
+      (unless (eq (tn-kind count) :unused)
+        (inst mov count (fixnumize 1)))
       (inst jmp done)
       (emit-label variable-values))
     ;; The stack frame is burnt and RETurned from if there are no
@@ -468,7 +406,8 @@
         ;; doesn't do subtraction.
         (inst shl nargs (- word-shift n-fixnum-tag-bits))
         (inst sub rsp-tn nargs)
-        (inst shr nargs (- word-shift n-fixnum-tag-bits)))
+        (unless unused-count-p
+          (inst shr nargs (- word-shift n-fixnum-tag-bits))))
       (emit-label stack-values))
     ;; dtc: this writes the registers onto the stack even if they are
     ;; not needed, only the number specified in rcx are used and have
@@ -479,7 +418,8 @@
       for j below (sb!kernel:values-type-max-value-count type)
       do (storew arg args i))
     (move start args)
-    (move count nargs)
+    (unless unused-count-p
+      (move count nargs))
 
     (emit-label done))
   (values))
@@ -706,11 +646,14 @@
                ;; passing locs in (FIRST (vop-codegen-info vop)).
                ,@(when (eq named :direct) '(fun))
                ,@(when (eq return :fixed) '(nvals))
-               step-instrumenting)
+               step-instrumenting
+               ,@(unless named
+                   '(callable)))
 
                (:ignore
                ,@(unless (or variable (eq return :tail)) '(arg-locs))
-               ,@(unless variable '(args)))
+               ,@(unless variable '(args))
+               ,@(and (eq return :fixed) '(rbx)))
 
                ;; We pass either the fdefn object (for named call) or
                ;; the actual function object (for unnamed call) in
@@ -723,7 +666,15 @@
                                   :from (:argument 0) :to :eval) rax)))
 
                ;; We pass the number of arguments in RCX.
-               (:temporary (:sc unsigned-reg :offset rcx-offset :to :eval) rcx)
+               (:temporary (:sc unsigned-reg :offset rcx-offset
+                            :to ,(if (eq return :fixed)
+                                     :save
+                                     :eval)) rcx)
+
+               ,@(when (eq return :fixed)
+                   ;; Save it for DEFAULT-UNKNOWN-VALUES to work
+                   `((:temporary (:sc unsigned-reg :offset rbx-offset
+                                  :from :result) rbx)))
 
                ;; With variable call, we have to load the
                ;; register-args out of the (new) stack frame before
@@ -845,41 +796,54 @@
                           (storew rbp-tn new-fp
                                   (frame-word-offset ocfp-save-offset))
 
-                          (move rbp-tn new-fp) ; NB - now on new stack frame.
-                          )))
+                          (move rbp-tn new-fp))))  ; NB - now on new stack frame.
 
-               (when step-instrumenting
+               (when (and step-instrumenting
+                          ,@(and (eq named :direct)
+                                 `((not (and #!+immobile-code
+                                             ;; handle-single-step-around-trap can't handle it
+                                             (static-fdefn-offset fun))))))
                  (emit-single-step-test)
                  (inst jmp :eq DONE)
                  (inst break single-step-around-trap))
                DONE
-
                (note-this-location vop :call-site)
-
-               ,(if (eq named :direct)
-                    `(let ((target
-                            (if (and (sb!c::code-immobile-p node)
-                                     (not step-instrumenting))
-                                (make-fixup fun :named-call)
-                                (progn
-                                  ;; RAX-TN was not declared as a temp var,
-                                  ;; however it's sole purpose at this point is
-                                  ;; for function call, so even if it was used
-                                  ;; to compute a stack argument, it's free now.
-                                  ;; If the call hits the undefined fun trap,
-                                  ;; RAX will get loaded regardless.
-                                  (inst mov rax-tn (make-fixup fun :named-call))
-                                  rax-tn))))
-                       (inst ,(if (eq return :tail) 'jmp 'call) target))
-                    `(inst ,(if (eq return :tail) 'jmp 'call)
-                           (make-ea :qword :base rax
-                              :disp ,(if named
-                                         '(- (* fdefn-raw-addr-slot
-                                                n-word-bytes)
-                                             other-pointer-lowtag)
-                                       '(- (* closure-fun-slot n-word-bytes)
-                                           fun-pointer-lowtag)))))
-
+               ,(cond ((eq named :direct)
+                       #!+immobile-code
+                       `(let* ((fixup (make-fixup
+                                       fun
+                                       (if (static-fdefn-offset fun)
+                                           :static-call
+                                           :named-call)))
+                               (target
+                                 (if (and (sb!c::code-immobile-p node)
+                                          (not step-instrumenting))
+                                     fixup
+                                     (progn
+                                       ;; RAX-TN was not declared as a temp var,
+                                       ;; however it's sole purpose at this point is
+                                       ;; for function call, so even if it was used
+                                       ;; to compute a stack argument, it's free now.
+                                       ;; If the call hits the undefined fun trap,
+                                       ;; RAX will get loaded regardless.
+                                       (inst mov rax-tn fixup)
+                                       rax-tn))))
+                          (inst ,(if (eq return :tail) 'jmp 'call) target))
+                       #!-immobile-code
+                       `(inst ,(if (eq return :tail) 'jmp 'call)
+                              (make-ea :qword :disp
+                                       (+ nil-value (static-fun-offset fun)))))
+                      #!-immobile-code
+                      (named
+                       `(inst ,(if (eq return :tail) 'jmp 'call)
+                              (make-ea :qword :base rax
+                                              :disp (- (* fdefn-raw-addr-slot
+                                                          n-word-bytes)
+                                                       other-pointer-lowtag))))
+                      ((eq return :tail)
+                       `(tail-call-unnamed rax callable vop))
+                      (t
+                       `(call-unnamed rax callable vop)))
                ,@(ecase return
                    (:fixed
                     '((default-unknown-values vop values nvals node)))
@@ -891,14 +855,73 @@
 
   (define-full-call call nil :fixed nil)
   (define-full-call call-named t :fixed nil)
+  #!-immobile-code
+  (define-full-call static-call-named :direct :fixed nil)
   (define-full-call multiple-call nil :unknown nil)
   (define-full-call multiple-call-named t :unknown nil)
+  #!-immobile-code
+  (define-full-call static-multiple-call-named :direct :unknown nil)
   (define-full-call tail-call nil :tail nil)
   (define-full-call tail-call-named t :tail nil)
+  #!-immobile-code
+  (define-full-call static-tail-call-named :direct :tail nil)
 
   (define-full-call call-variable nil :fixed t)
   (define-full-call multiple-call-variable nil :unknown t))
 
+;;; Invoke the function-designator FUN. If DESIGNATOR-P is true, FUN might be
+;;; a symbol; if DESIGNATOR-P is false, FUN is definitely a function.
+;;;
+;;; Note: DESIGNATOR-P used to be named CALLABLE. They're both imperfect.
+;;; While it's true that the noun CALLABLE is synonymous with function-designator,
+;;; the former had a strange connotation of (NOT CALLABLE) counterintuitively
+;;; implying that the FUN *is* directly callable, being strictly a function,
+;;; with "callable" parsed in its adjectival sense.
+;;; Also problematic would be NOT-FUNCTIONP since FUN might satisfy FUNCTIONP.
+;;; A pedantically correct change would be to invert the boolean as supplied,
+;;; and name it DEFINITELY-FUNCTIONP, or name it just FUNCTIONP with the
+;;; notion that T means T and NIL means possibly.
+;;; However, we might really want three values: yes, no, and maybe.
+;;; Because as things are, the definitely-NOT-function case goes through
+;;; the "maybe" path even though it _must_ coerce to a function:
+;;;
+;;; * (disassemble '(lambda (f) (funcall (the symbol f) 42)))
+;;;  3F:       8D58F5           LEA EBX, [RAX-11]
+;;;  42:       F6C30F           TEST BL, 15
+;;;  45:       7503             JNE L1
+;;;  47:       FF60FD           JMP QWORD PTR [RAX-3]
+;;;  4A: L1:   FF242570000020   JMP QWORD PTR [#x20000070]       ; TAIL-CALL-SYMBOL
+
+(defun tail-call-unnamed (fun designator-p vop)
+  (let ((relative-call (sb!c::code-immobile-p vop))
+        (fun-ea (make-ea :qword :base fun
+                         :disp (- (* closure-fun-slot n-word-bytes)
+                                  fun-pointer-lowtag))))
+    (if designator-p
+        (assemble ()
+          (%lea-for-lowtag-test ebx-tn fun fun-pointer-lowtag)
+          (inst test bl-tn lowtag-mask)
+          (inst jmp :nz (if relative-call
+                            (make-fixup 'tail-call-symbol :assembly-routine)
+                            not-fun))
+          (inst jmp fun-ea)
+          not-fun
+          (unless relative-call
+            (invoke-asm-routine 'jmp 'tail-call-symbol vop)))
+        (inst jmp fun-ea))))
+
+(defun call-unnamed (fun designator-p vop)
+  (assemble ()
+    (when designator-p
+      (%lea-for-lowtag-test ebx-tn fun fun-pointer-lowtag)
+      (inst test bl-tn lowtag-mask)
+      (inst jmp :z call)
+      (invoke-asm-routine 'call 'call-symbol vop))
+    call
+    (inst call (make-ea :qword :base fun
+                        :disp (- (* closure-fun-slot n-word-bytes)
+                                 fun-pointer-lowtag)))))
+
 ;;; This is defined separately, since it needs special code that BLT's
 ;;; the arguments down. All the real work is done in the assembly
 ;;; routine. We just set things up so that it can find what it needs.
@@ -907,9 +930,9 @@
          (function :scs (descriptor-reg control-stack) :target rax)
          (old-fp)
          (return-pc))
+  (:info callable)
   (:temporary (:sc unsigned-reg :offset rsi-offset :from (:argument 0)) rsi)
   (:temporary (:sc unsigned-reg :offset rax-offset :from (:argument 1)) rax)
-  (:temporary (:sc unsigned-reg) call-target)
   (:vop-var vop)
   (:generator 75
     (check-ocfp-and-return-pc old-fp return-pc)
@@ -917,7 +940,10 @@
     (move rsi args)
     (move rax function)
     ;; And jump to the assembly routine.
-    (invoke-asm-routine 'jmp 'tail-call-variable vop call-target)))
+    (invoke-asm-routine 'jmp (if callable
+                                 'tail-call-callable-variable
+                                 'tail-call-variable)
+                        vop)))
 
 ;;;; unknown values return
 
@@ -1033,7 +1059,6 @@
          (nvals :scs (any-reg) :target rcx))
   (:temporary (:sc unsigned-reg :offset rsi-offset :from (:argument 2)) rsi)
   (:temporary (:sc unsigned-reg :offset rcx-offset :from (:argument 3)) rcx)
-  (:temporary (:sc unsigned-reg) return-asm)
   (:temporary (:sc descriptor-reg :offset (first *register-arg-offsets*)
                    :from (:eval 0)) a0)
   (:node-var node)
@@ -1058,7 +1083,7 @@
         (emit-label not-single)))
     (move rsi vals)
     (move rcx nvals)
-    (invoke-asm-routine 'jmp 'return-multiple vop return-asm)))
+    (invoke-asm-routine 'jmp 'return-multiple vop)))
 
 ;;;; XEP hackery
 
@@ -1202,11 +1227,10 @@
     (inst jmp DONE)
 
     JUST-ALLOC-FRAME
-    (inst lea rsp-tn
-          (make-ea :qword :base rbp-tn
-                   :disp (* n-word-bytes
+    (emit-lea rsp-tn rbp-tn
+                         (* n-word-bytes
                             (- sp->fp-offset
-                               (sb-allocated-size 'stack)))))
+                               (sb-allocated-size 'stack))))
 
     DONE))
 
@@ -1277,6 +1301,9 @@
       (inst jrcxz done)
       (inst lea dst (make-ea :qword :index rcx :scale (ash 2 (- word-shift n-fixnum-tag-bits))))
       (maybe-pseudo-atomic stack-allocate-p
+       ;; FIXME: if COUNT >= 8192, allocates to single-object page(s).
+       ;; All we have to do is unset the '.singleton' bit,
+       ;; a permissible state change as long as pseudo-atomic.
        (allocation dst dst node stack-allocate-p list-pointer-lowtag)
        ;; Set up the result.
        (move result dst)
@@ -1407,12 +1434,9 @@
   ;; register on -SB-THREAD. While this isn't critical for x86-64,
   ;; it's more serious for x86.
   #!+sb-thread
-  (inst cmp (make-ea :qword
-                     :base thread-base-tn
-                     :disp (* thread-stepping-slot n-word-bytes))
-        0)
+  (inst cmp (thread-tls-ea (* thread-stepping-slot n-word-bytes)) 0)
   #!-sb-thread
-  (inst cmp (make-ea :qword
+  (inst cmp (make-ea :byte
                      :disp (+ nil-value (static-symbol-offset
                                          'sb!impl::*stepping*)
                               (* symbol-value-slot n-word-bytes)
