@@ -157,7 +157,7 @@
                                  (fast-method-call
                                   (let* ((fmf (fast-method-call-function emf))
                                          (fun (method-function-from-fast-method-call emf))
-                                         (mf (%make-method-function fmf nil)))
+                                         (mf (%make-method-function fmf)))
                                     (set-funcallable-instance-function mf fun)
                                     (make-instance 'standard-method
                                                    :specializers nil ; XXX
@@ -410,6 +410,12 @@
 (defun gf-requires-emf-keyword-checks (generic-function)
   (member '&key (gf-lambda-list generic-function)))
 
+(defconstant-eqx +standard-method-combination-qualifiers+
+    '(:around :before :after) #'equal)
+
+(defun standard-method-combination-qualifier-p (qualifier)
+  (member qualifier +standard-method-combination-qualifiers+))
+
 (defun standard-compute-effective-method
     (generic-function combin applicable-methods)
   (collect ((before) (primary) (after) (around))
@@ -461,6 +467,12 @@
                                   (make-method ,main-effective-method)))
                  main-effective-method))))))
 
+(defun short-method-combination-qualifiers (type-name)
+  (list type-name :around))
+
+(defun short-method-combination-qualifier-p (type-name qualifier)
+  (or (eq qualifier type-name) (eq qualifier :around)))
+
 (defun short-compute-effective-method
     (generic-function combin applicable-methods)
   (let ((type-name (method-combination-type-name combin))
@@ -563,18 +575,16 @@
         (loop
            (when (>= i more-count)
              (when (and (invalid) (not allow-other-keys))
-               (error 'simple-program-error
-                      :format-control "~@<invalid keyword argument~P: ~
-                                   ~{~S~^, ~} (valid keys are ~{~S~^, ~}).~@:>"
-                      :format-arguments (list (length (invalid)) (invalid) valid-keys)))
+               (%program-error "~@<invalid keyword argument~P: ~
+                                ~{~S~^, ~} (valid keys are ~{~S~^, ~}).~@:>"
+                               (length (invalid)) (invalid) valid-keys))
              (return))
            (let ((key (current-value)))
              (incf i)
              (cond
                ((not (symbolp key))
-                (error 'simple-program-error
-                       :format-control "~@<keyword argument not a symbol: ~S.~@:>"
-                       :format-arguments (list key)))
+                (%program-error "~@<keyword argument not a symbol: ~S.~@:>"
+                                key))
                ((= i more-count)
                 (sb-c::%odd-key-args-error))
                ((eq key :allow-other-keys)
