@@ -27,9 +27,8 @@
 (in-package "SB!PCL")
 
 (declaim (type (member nil early braid complete) **boot-state**))
-(defglobal **boot-state** nil)
+(define-load-time-global **boot-state** nil)
 
-(/show0 "starting early-low.lisp")
 
 ;;; The PCL package is internal and is used by code in potential
 ;;; bottlenecks. And since it's internal, no one should be
@@ -79,7 +78,8 @@
   (and (symbolp type)
        (condition-classoid-p (find-classoid type nil))))
 
-(declaim (special *the-class-t*
+#-sb-xc-host
+(declaim (global  *the-class-t*
                   *the-class-vector* *the-class-symbol*
                   *the-class-string* *the-class-sequence*
                   *the-class-rational* *the-class-ratio*
@@ -124,23 +124,28 @@
 ;;;; PCL instances
 
 (sb!kernel::!defstruct-with-alternate-metaclass standard-instance
-  :slot-names (slots hash-code)
+  ;; KLUDGE: arm64 needs to have CAS-HEADER-DATA-HIGH implemented
+  :slot-names (slots #!-(and compact-instance-header x86-64) hash-code)
   :boa-constructor %make-standard-instance
   :superclass-name t
-  :metaclass-name standard-classoid
-  :metaclass-constructor make-standard-classoid
+  :metaclass-name static-classoid
+  :metaclass-constructor make-static-classoid
   :dd-type structure
   :runtime-type-checks-p nil)
 
+;;; TODO: for x8-64 with #!+immobile-code, we would like 2 additional unboxed
+;;; words to hold the trampline instructions to avoid consing a piece of code
+;;; to jump to this function. It should be "as if" a simple-fun, in as much as
+;;; there's an address you can jump to without loading a register.
 (sb!kernel::!defstruct-with-alternate-metaclass standard-funcallable-instance
   ;; KLUDGE: Note that neither of these slots is ever accessed by its
   ;; accessor name as of sbcl-0.pre7.63. Presumably everything works
   ;; by puns based on absolute locations. Fun fun fun.. -- WHN 2001-10-30
-  :slot-names (clos-slots hash-code)
+  :slot-names (clos-slots #!-compact-instance-header hash-code)
   :boa-constructor %make-standard-funcallable-instance
   :superclass-name function
-  :metaclass-name standard-classoid
-  :metaclass-constructor make-standard-classoid
+  :metaclass-name static-classoid
+  :metaclass-constructor make-static-classoid
   :dd-type funcallable-structure
   ;; Only internal implementation code will access these, and these
   ;; accesses (slot readers in particular) could easily be a
@@ -151,7 +156,11 @@
   ;; (!) as of sbcl-0.pre7.63, so for now it's academic.)
   :runtime-type-checks-p nil)
 
+#!+(and compact-instance-header (not x86-64))
+(defconstant std-instance-hash-slot-index 1)
+#!-compact-instance-header
+(progn
 (defconstant std-instance-hash-slot-index 2)
-(defconstant fsc-instance-hash-slot-index 2)
-
-(/show0 "finished with early-low.lisp")
+;; The first data slot (either index 0 or 1) in the primitive funcallable
+;; instance is the vector of CLOS slots. Following that is the hash.
+(defconstant fsc-instance-hash-slot-index (1+ sb!vm:instance-data-start)))
