@@ -13,13 +13,13 @@
 
 (in-package sb-vm)
 
-(load "test-util.lisp")
 (use-package :test-util)
 
 ;; this is architecture-agnostic
 (defun test-assemble (inst expect)
   (let ((segment (sb-assem:make-segment :type :regular)))
-    (apply (sb-assem::op-encoder-name (car inst)) segment nil (cdr inst))
+    (sb-assem:assemble (segment)
+      (apply (sb-assem::op-encoder-name (car inst)) (cdr inst)))
     (let* ((buf (sb-assem::segment-buffer segment))
            (string
             (with-output-to-string (stream)
@@ -34,7 +34,7 @@
                                           (1- (length string)))))) ; chop final newline
       (assert (string= line expect)))))
 
-(with-test (:name :assemble-movti-instruction :skipped-on '(not :x86-64))
+(with-test (:name :assemble-movti-instruction :skipped-on (not :x86-64))
   (flet ((test-movnti (dst src expect)
            (test-assemble `(movnti ,dst ,src) expect)))
     (test-movnti (make-ea :dword :base rdi-tn :disp 57) eax-tn
@@ -42,7 +42,7 @@
     (test-movnti (make-ea :qword :base rax-tn) r12-tn
                  "4C0FC320         MOVNTI [RAX], R12")))
 
-(with-test (:name :assemble-crc32 :skipped-on '(not :x86-64))
+(with-test (:name :assemble-crc32 :skipped-on (not :x86-64))
   ;; Destination size = :DWORD
   (test-assemble `(crc32 ,eax-tn ,(make-ea :byte  :base rbp-tn))
                  "F20F38F04500     CRC32 EAX, BYTE PTR [RBP]")
@@ -68,7 +68,27 @@
   (test-assemble `(crc32 ,r9-tn ,(make-ea :qword :base r14-tn :index r15-tn))
                  "F24F0F38F10C3E   CRC32 R9, QWORD PTR [R14+R15]"))
 
-(with-test (:name :disassemble-arith-insts :skipped-on '(not (or :x86 :x86-64)))
+(with-test (:name :disassemble-movabs-instruction :skipped-on (not :x86-64))
+  (let* ((bytes (coerce '(#x48 #xA1 8 7 6 5 4 3 2 1
+                          #xA1 8 7 6 5 4 3 2 1
+                          #x66 #xA1 8 7 6 5 4 3 2 1
+                          #xA0 8 7 6 5 4 3 2 1)
+                        '(array (unsigned-byte 8) 1)))
+         (lines
+          (split-string
+           (with-output-to-string (s)
+             (sb-sys:with-pinned-objects (bytes)
+               (sb-disassem:disassemble-memory
+                (sb-sys:sap-int (sb-sys:vector-sap bytes))
+                (length bytes)
+                :stream s)))
+           #\newline)))
+    (pop lines)
+    (dolist (dest-reg '("RAX" "EAX" "AX" "AL"))
+      (assert (search (format nil "MOVABS ~A, [#x102030405060708]" dest-reg)
+                      (pop lines))))))
+
+(with-test (:name :disassemble-arith-insts :skipped-on (not (or :x86 :x86-64)))
   (flet ((try (inst expect)
            (let ((p (search "$fp" expect)))
              (when p
@@ -121,3 +141,31 @@
     #+x86-64
     (try `(add ,rcx-tn ,(memref :qword)) "48034D00         ADD RCX, [$fp]")
     ))
+
+(with-test (:name :disassemble-fs-prefix :skipped-on (not (or :x86-64)))
+  (let ((bytes (coerce '(#x64 #xF0 #x44 #x08 #x04 #x25 #x00 #x04 #x10 #x20)
+                       '(array (unsigned-byte 8) 1)))
+        (s (make-string-output-stream)))
+    (sb-sys:with-pinned-objects (bytes)
+      (sb-disassem::disassemble-memory (sb-sys:sap-int (sb-sys:vector-sap bytes))
+                                       (length bytes)
+                                       :stream s))
+    (assert (search "LOCK OR FS:[#x20100400], R8B"
+                    (get-output-stream-string s)))))
+
+(with-test (:name :disassemble-static-fdefn :skipped-on (not :x86-64))
+  (assert (< (get-lisp-obj-address (sb-kernel::find-fdefn 'sb-impl::sub-gc))
+             sb-vm:static-space-end))
+  ;; Cause SUB-GC to become un-statically-linked
+  (progn (trace sb-impl::sub-gc) (untrace))
+  (let ((lines
+         (split-string (with-output-to-string (s)
+                         (disassemble 'sb-impl::gc :stream s))
+                       #\Newline))
+        (found))
+    ;; Check that find-called-object looked in static space for FDEFNs
+    (dolist (line lines)
+      (when (and (search "CALL" line)
+                 (search " SUB-GC" line))
+        (setq found t)))
+    (assert found)))
