@@ -90,7 +90,7 @@
                     (endp actual))
                    ((or (eq '? (car want)) (equal (car want) (car actual)))
                     (args-equal (cdr want) (cdr actual)))
-                   ((typep (car want) 'sb-impl::unprintable-object)
+                   ((typep (car want) 'sb-debug::unprintable-object)
                     (equalp (car want) (car actual)))
                    (t
                     nil)))
@@ -185,12 +185,12 @@
          (funcall fun)))
 
   (with-test (:name (:backtrace :undefined-function :bug-346)
-                    :skipped-on :interpreter
-                    ;; Failures on SPARC, and probably HPPA are due to
-                    ;; not having a full and valid stack frame for the
-                    ;; undefined function frame.  See PPC
-                    ;; undefined_tramp for details.
-                    :fails-on '(or :sparc))
+              :skipped-on :interpreter
+              ;; Failures on SPARC, and probably HPPA are due to
+              ;; not having a full and valid stack frame for the
+              ;; undefined function frame.  See PPC
+              ;; undefined_tramp for details.
+              :fails-on :sparc)
     (assert-backtrace
      (lambda () (test #'optimized))
      (list (append *undefined-function-frame* '(42))
@@ -207,7 +207,7 @@
            (list `(flet test :in ,*p*) #'not-optimized)))))
 
 (with-test (:name (:backtrace :interrupted-condition-wait)
-                  :skipped-on '(not :sb-thread))
+                  :skipped-on (not :sb-thread))
   (let ((m (sb-thread:make-mutex))
         (q (sb-thread:make-waitqueue)))
     (assert-backtrace
@@ -247,20 +247,20 @@
   (with-test (:name (:backtrace :divide-by-zero :bug-346)
                     :skipped-on :interpreter)
     (assert-backtrace (lambda () (test #'optimized))
-                      `((/ 42 &rest)
+                      `((sb-kernel::integer-/-integer 42 &rest)
                         ((flet test :in ,*p*) ,#'optimized))))
 
   (with-test (:name (:backtrace :divide-by-zero :bug-356)
                     :skipped-on :interpreter)
     (assert-backtrace (lambda () (test #'not-optimized))
-                      `((/ 42 &rest)
+                      `((sb-kernel::integer-/-integer 42 &rest)
                         ((flet not-optimized :in ,*p*))
                         ((flet test :in ,*p*) ,#'not-optimized)))))
 
 (defun throw-test ()
   (throw 'no-such-tag t))
 (with-test (:name (:backtrace :throw :no-such-tag)
-                  :fails-on '(and :sparc :linux))
+                  :fails-on (and :sparc :linux))
   (assert-backtrace #'throw-test '((throw-test))))
 
 (funcall (checked-compile
@@ -560,7 +560,7 @@
 
 #+sb-fasteval
 (with-test (:name (:backtrace :interpreted-factorial)
-            :skipped-on '(not :interpreter))
+            :skipped-on (not :interpreter))
   (assert-backtrace
    (lambda () (fact 5))
    '((fact 0)
@@ -602,7 +602,43 @@
 (with-test (:name :long-string-abbreviation)
   (let ((backtrace (mega-string-replace-fail '(#\- 1))))
     (assert (search (concatenate 'string
-                                 "\"-"
-                                 (make-string 49 :initial-element #\z)
-                                 "...\"")
+                                 "-"
+                                 (make-string 199 :initial-element #\z)
+                                 "...")
                     backtrace))))
+
+(defclass cannot-print-this () ())
+(defmethod print-object ((object cannot-print-this) stream)
+  (error "No go!"))
+
+(with-test (:name (sb-debug:print-backtrace :no-error print-object))
+  ;; Errors during printing objects used to be suppressed in a way
+  ;; that required outer condition handlers to behave in a specific
+  ;; way.
+  (handler-bind ((error (lambda (condition)
+                          (error "~@<~S signaled ~A.~@:>"
+                                 'sb-debug:print-backtrace condition))))
+    (with-output-to-string (stream)
+      (labels ((foo (n x)
+                 (when (plusp n)
+                   (foo (1- n) x))
+                 (when (zerop n)
+                   (sb-debug:print-backtrace :count 100 :stream stream
+                                             :emergency-best-effort t))))
+        (foo 100 (make-instance 'cannot-print-this))))))
+
+(with-test (:name (sb-debug:print-backtrace :no-error :circles))
+  ;; Errors during printing objects used to be suppressed in a way
+  ;; that required outer condition handlers to behave in a specific
+  ;; way.
+  (handler-bind ((error (lambda (condition)
+                          (error "~@<~S signaled ~A.~@:>"
+                                 'sb-debug:print-backtrace condition))))
+    (with-output-to-string (stream)
+      (labels ((foo (n x)
+                 (when (plusp n)
+                   (foo (1- n) x))
+                 (when (zerop n)
+                   (sb-debug:print-backtrace :count 100 :stream stream))))
+        (foo 100 (let ((list (list t)))
+                   (nconc list list)))))))
