@@ -68,7 +68,7 @@
     (assert gc-happend)))
 
 ;;; SB-EXT:GENERATION-* accessors returned bogus values for generation > 0
-(with-test (:name :bug-529014 :skipped-on '(not :gencgc))
+(with-test (:name :bug-529014 :skipped-on (not :gencgc))
   (loop for i from 0 to sb-vm:+pseudo-static-generation+
      do (assert (= (sb-ext:generation-bytes-consed-between-gcs i)
                    (truncate (sb-ext:bytes-consed-between-gcs)
@@ -96,18 +96,16 @@
 
 (with-test (:name :bug-936304)
   (gc :full t)
-  (time
-   (assert (eq :ok (handler-case
+  (assert (eq :ok (handler-case
                        (progn
                          (loop repeat 50 do (stress-gc))
                          :ok)
                      (storage-condition ()
-                       :oom))))))
+                       :oom)))))
 
 (with-test (:name :bug-981106)
   (gc :full t)
-  (time
-   (assert (eq :ok
+  (assert (eq :ok
                (handler-case
                    (dotimes (runs 100 :ok)
                      (let* ((n (truncate (dynamic-space-size) 1200))
@@ -117,9 +115,9 @@
                                       (write-sequence "hi there!" string))))))
                        (assert (eql len (* n (length "hi there!"))))))
                  (storage-condition ()
-                   :oom))))))
+                   :oom)))))
 
-(with-test (:name :gc-logfile :skipped-on '(not :gencgc))
+(with-test (:name :gc-logfile :skipped-on (not :gencgc))
   (assert (not (gc-logfile)))
   (let ((p #p"gc.log"))
     (assert (not (probe-file p)))
@@ -147,7 +145,7 @@
 (defparameter *pin-test-object-address* nil)
 
 (with-test (:name (sb-sys:with-pinned-objects :actually-pins-objects)
-                  :skipped-on ':cheneygc)
+                  :skipped-on :cheneygc)
   ;; The interpreters (both sb-eval and sb-fasteval) special-case
   ;; WITH-PINNED-OBJECTS as a "special form", because the x86oid
   ;; version of WITH-PINNED-OBJECTS uses black magic that isn't
@@ -169,3 +167,69 @@
     (gc :full t)
     (assert (= (sb-kernel:get-lisp-obj-address *pin-test-object*)
                *pin-test-object-address*))))
+
+(defun ensure-code/data-separation ()
+  (let* ((n-bits (+ sb-vm:next-free-page 10))
+         (code-bits (make-array n-bits :element-type 'bit))
+         (data-bits (make-array n-bits :element-type 'bit))
+         (total-code-size 0))
+    (sb-vm::map-allocated-objects
+     (lambda (obj type size)
+       (declare ((and fixnum (integer 1)) size))
+       ;; M-A-O disables GC, therefore GET-LISP-OBJ-ADDRESS is safe
+       (let ((obj-addr (sb-kernel:get-lisp-obj-address obj))
+             (array (cond ((= type sb-vm:code-header-widetag)
+                           (incf total-code-size size)
+                           code-bits)
+                          (t
+                           data-bits))))
+         ;; This is not the most efficient way to update the bit arrays,
+         ;; but the simplest and clearest for sure. (The loop could avoided
+         ;; if the current page is the same as the previously seen page)
+         (loop for index from (sb-vm::find-page-index obj-addr)
+               to (sb-vm::find-page-index (truly-the word
+                                                     (+ (logandc2 obj-addr sb-vm:lowtag-mask)
+                                                        (1- size))))
+               do (setf (sbit array index) 1))))
+     :dynamic)
+    (assert (not (find 1 (bit-and code-bits data-bits))))
+    (let* ((code-bytes-consumed
+            (* (count 1 code-bits) sb-vm:gencgc-card-bytes))
+           (waste
+            (- total-code-size code-bytes-consumed)))
+      ;; This should be true for all platforms.
+      ;; Some have as little as .5% space wasted.
+      (assert (<= waste (* 3/100 code-bytes-consumed))))))
+
+(compile 'ensure-code/data-separation)
+
+(with-test (:name :code/data-separation
+                  :skipped-on (:not :gencgc))
+  (ensure-code/data-separation))
+
+#+immobile-space
+(with-test (:name :immobile-space-addr-p)
+  ;; Upper bound should be exclusive
+  (assert (not (sb-kernel:immobile-space-addr-p
+                (+ sb-vm:fixedobj-space-start
+                   sb-vm:fixedobj-space-size
+                   sb-vm:varyobj-space-size)))))
+
+;;; After each iteration of FOO there are a few pinned conses.
+;;; On alternate GC cycles, those get promoted to generation 1.
+;;; When the logic for page-spanning-object zeroing incorrectly decreased
+;;; the upper bound on bytes used for partially pinned pages, it caused
+;;; an accumulation of pages in generation 1 each with 2 objects' worth
+;;; of bytes, and the remainder waste. Because the waste was not accounted
+;;; for, it did not trigger GC enough to avoid heap exhaustion.
+(with-test (:name :smallobj-auto-gc-trigger)
+  ;; Ensure that these are compiled functions because the interpreter
+  ;; would make lots of objects of various sizes which is insufficient
+  ;; to provoke the bug.
+  (setf (symbol-function 'foo)
+        (compile nil '(lambda () (list 1 2))))
+  ;; 500 million iterations of this loop seems to be reliable enough
+  ;; to show that GC happens.
+  (setf (symbol-function 'callfoo)
+        (compile nil '(lambda () (loop repeat 500000000 do (foo)))))
+  (funcall 'callfoo))
