@@ -18,9 +18,7 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-(in-package :cl-user)
-
-(test-util:with-test (:name :no-meta-info)
+(with-test (:name :no-meta-info)
  (assert-signal (compile nil '(lambda (x) (sb-int:info :type :nokind x)))
                 style-warning))
 
@@ -504,7 +502,7 @@
     (values n-created fdefn-result random-result)))
 
 (test-util:with-test (:name :get-info-value-initializing
-                      :skipped-on '(not :sb-thread))
+                      :skipped-on (not :sb-thread))
   ;; Precompute random generalized function names for testing, some of which
   ;; are "simple" (per the taxonomy of globaldb) and some hairy.
   (let ((work (coerce (loop repeat 10000
@@ -571,7 +569,7 @@
 ;; This test conses ~5 Megabytes on 64-bit almost entirely due
 ;; to allocation of each immutable info storage vector.
 (test-util:with-test (:name :get-info-value-updating
-                      :skipped-on '(not :sb-thread))
+                      :skipped-on (not :sb-thread))
   (flet ((run (names)
            (declare (simple-vector names))
            (let* ((n (length names))
