@@ -14,24 +14,137 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-(load "assertoid.lisp")
-(use-package "ASSERTOID")
+;;;; Pathname accessors
 
-(load "test-util.lisp")
-(use-package "TEST-UTIL")
+(with-test (:name (pathname :accessors :stream-not-associated-to-file type-error))
+  (let ((*stream* (make-string-output-stream)))
+    (declare (special *stream*))
+    (checked-compile-and-assert ()
+        '(lambda (pathname) (pathname-host pathname))
+      (((make-string-output-stream))     (condition 'type-error))
+      (((make-synonym-stream '*stream*)) (condition 'type-error)))))
+
+(with-test (:name (pathname :accessors file-stream))
+  (let* ((name "pathnames.impure")
+         (type "lisp"))
+    (with-open-file (stream (make-pathname :name name :type type))
+      (assert (equal (pathname-name stream) name))
+      (assert (equal (pathname-type stream) type)))))
+
+(with-test (:name (pathname :accessors synonym-stream))
+  (let* ((name "pathnames.impure")
+         (type "lisp"))
+    (with-open-file (stream (make-pathname :name name :type type))
+      (let* ((*stream1* stream)
+             (stream1 (make-synonym-stream '*stream1*))
+             (*stream2* stream1)
+             (stream2 (make-synonym-stream '*stream2*)))
+        (declare (special *stream1* *stream2*))
+        (assert (equal (pathname-name stream1) name))
+        (assert (equal (pathname-name stream2) name))
+        (assert (equal (pathname-type stream1) type))
+        (assert (equal (pathname-type stream2) type))))))
+
+(labels ((unpattern (thing)
+           (typecase thing
+             (cons
+              (list* (unpattern (first thing))
+                     (map 'list #'unpattern (rest thing))))
+             (sb-impl::pattern
+              (sb-impl::pattern-pieces thing))
+             (t
+              thing)))
+         (pattern (thing)
+           (typecase thing
+             ((cons (member :absolute :relative))
+              (list* (pattern (first thing))
+                     (map 'list #'pattern (cdr thing))))
+             (cons
+              (sb-impl::make-pattern thing))
+             (t
+              thing)))
+         (accessor-cases (initarg accessor cases)
+           (map nil (lambda (test-case)
+                      (destructuring-bind (pathname case expected) test-case
+                        ;; Check accessor directory.
+                        (let ((result (funcall accessor pathname :case case)))
+                          (assert (equal (unpattern result) expected)))
+                        ;; Check roundtrip.
+                        (let* ((pathname (make-pathname initarg (pattern expected) :case case))
+                               (result (funcall accessor pathname :case case)))
+                          (assert (equal (unpattern result) expected)))))
+                cases)))
+
+  (with-test (:name (pathname-device :case))
+    (accessor-cases
+     :device #'pathname-device
+     `((,(make-pathname :device "foo") :local  "foo")
+       (,(make-pathname :device "FOO") :local  "FOO")
+       (,(make-pathname :device "Foo") :local  "Foo")
+
+       (,(make-pathname :device "foo") :common "FOO")
+       (,(make-pathname :device "FOO") :common "foo")
+       (,(make-pathname :device "Foo") :common "Foo"))))
+
+  (with-test (:name (pathname-directory :case))
+    (accessor-cases
+     :directory #'pathname-directory
+     `((,#P"/ab/cd/" :local  (:absolute "ab" "cd"))
+       (,#P"/AB/CD/" :local  (:absolute "AB" "CD"))
+       (,#P"/Ab/Cd/" :local  (:absolute "Ab" "Cd"))
+       (,#P"/AB/cd/" :local  (:absolute "AB" "cd"))
+       (,#P"/a*b/"   :local  (:absolute ("a" :multi-char-wild "b")))
+
+       (,#P"/ab/cd/" :common (:absolute "AB" "CD"))
+       (,#P"/AB/CD/" :common (:absolute "ab" "cd"))
+       (,#P"/Ab/Cd/" :common (:absolute "Ab" "Cd"))
+       (,#P"/AB/cd/" :common (:absolute "ab" "CD"))
+       (,#P"/a*b/"   :common (:absolute ("A" :multi-char-wild "B"))))))
+
+  (with-test (:name (pathname-name :case))
+    (accessor-cases
+     :name #'pathname-name
+     `((,#P"foo" :local  "foo")
+       (,#P"FOO" :local  "FOO")
+       (,#P"Foo" :local  "Foo")
+       (,#P"a*b" :local  ("a" :multi-char-wild "b"))
+
+       (,#P"foo" :common "FOO")
+       (,#P"FOO" :common "foo")
+       (,#P"Foo" :common "Foo")
+       (,#P"a*b" :common ("A" :multi-char-wild "B")))))
+
+  (with-test (:name (pathname-type :case))
+    (accessor-cases
+     :type #'pathname-type
+     `((,#P"n.foo" :local  "foo")
+       (,#P"n.FOO" :local  "FOO")
+       (,#P"n.Foo" :local  "Foo")
+       (,#P"n.a*b" :local  ("a" :multi-char-wild "b"))
+
+       (,#P"n.foo" :common "FOO")
+       (,#P"n.FOO" :common "foo")
+       (,#P"n.Foo" :common "Foo")
+       (,#P"n.a*b" :common ("A" :multi-char-wild "B"))))))
+
+
+;;;; Logical pathnames
 
 (setf (logical-pathname-translations "demo0")
       '(("**;*.*.*" "/tmp/")))
 
 ;;; In case of a parse error we want to get a condition of type TYPE-ERROR,
 ;;; because ANSI says so. (This used to be PARSE-ERROR.)
-(assert
-  (typep (grab-condition (logical-pathname "demo0::bla;file.lisp"))
-         'type-error))
+(with-test (:name (logical-pathname :signals type-error))
+  (mapc (lambda (namestring)
+          (assert-error (logical-pathname namestring) type-error))
+        '("demo0::bla;file.lisp"
+          "FOO.txt"
+          "SYS:%")))
 
 ;;; some things SBCL-0.6.9 used not to parse correctly:
 ;;;
-;;; SBCL used to throw an error saying there's no translation.
+;;; SBCL used to signal an error saying there's no translation.
 (with-test (:name (:logical-pathname 1))
   (assert (equal (namestring (translate-logical-pathname "demo0:file.lisp"))
                  "/tmp/file.lisp")))
@@ -209,7 +322,7 @@
   (setf (logical-pathname-translations "scratch")
         '(("**;*.*.*" "/usr/local/doc/**/*"))))
 
-(with-test (:name (:merge-pathname 1))
+(with-test (:name (merge-pathnames 1))
   (loop for (expected-result . params) in
         `( ;; trivial merge
           (#P"/usr/local/doc/foo" #p"foo" #p"/usr/local/doc/")
@@ -295,7 +408,7 @@
                (frob pathname-type)))))
 
 ;;; host-namestring testing
-(with-test (:name :host-namestring)
+(with-test (:name host-namestring)
   (assert (string=
            (namestring (parse-namestring "/foo" (host-namestring #p"/bar")))
            "/foo"))
@@ -317,46 +430,48 @@
 ;;; had difficulty with non-FILE-STREAM stream arguments to pathname
 ;;; functions (they would cause memory protection errors).  Make sure
 ;;; that those errors are gone:
-(with-test (:name (:string-streams-as-pathnames 1))
-  (assert-error (pathname (make-string-input-stream "FOO"))
-                type-error)
-  (assert-error (merge-pathnames (make-string-output-stream))
-                type-error))
+(with-test (:name (string-stream :not-a pathname))
+  (flet ((test (form)
+           (multiple-value-bind (fun failurep warnings)
+               (checked-compile
+                `(lambda () ,form) :allow-warnings 'sb-int:type-warning)
+             (declare (ignore failurep))
+             (assert (= 1 (length warnings)))
+             (assert-error (funcall fun) type-error))))
+    (test '(pathname (make-string-input-stream "FOO")))
+    (test '(merge-pathnames (make-string-output-stream)))))
 
 ;;; ensure print-read consistency (or print-not-readable-error) on
 ;;; pathnames:
 (with-test (:name :print/read-consistency)
-  (let ((pathnames (list
-                    (make-pathname :name "foo" :type "txt" :version :newest)
-                    (make-pathname :name "foo" :type "txt" :version 1)
-                    (make-pathname :name "foo" :type ".txt")
-                    (make-pathname :name "foo." :type "txt")
-                    (make-pathname :name "\\" :type "txt")
-                    (make-pathname :name "^" :type "txt")
-                    (make-pathname :name "foo*" :type "txt")
-                    (make-pathname :name "foo[" :type "txt")
-                    (parse-namestring "SCRATCH:FOO.TXT.1")
-                    (parse-namestring "SCRATCH:FOO.TXT.NEWEST")
-                    (parse-namestring "SCRATCH:FOO.TXT"))))
-    (dolist (p pathnames)
-      (print p)
-      (handler-case
-          (let* ((*print-readably* t)
-                 (new (read-from-string (format nil "~S" p))))
-            (unless (equal new p)
-              (let ((*print-readably* nil))
-                (error "oops: host:~S device:~S dir:~S version:~S~% ->~%~
+  (dolist (p (list (make-pathname :name "foo" :type "txt" :version :newest)
+                   (make-pathname :name "foo" :type "txt" :version 1)
+                   (make-pathname :name "foo" :type ".txt")
+                   (make-pathname :name "foo." :type "txt")
+                   (make-pathname :name #-win32 "\\" #+win32 "^^" :type "txt")
+                   (make-pathname :name "^" :type "txt")
+                   (make-pathname :name "foo*" :type "txt")
+                   (make-pathname :name "foo[" :type "txt")
+                   (parse-namestring "SCRATCH:FOO.TXT.1")
+                   (parse-namestring "SCRATCH:FOO.TXT.NEWEST")
+                   (parse-namestring "SCRATCH:FOO.TXT")))
+    (handler-case
+        (let* ((*print-readably* t)
+               (new (read-from-string (format nil "~S" p))))
+          (unless (equal new p)
+            (let ((*print-readably* nil))
+              (error "oops: host:~S device:~S dir:~S version:~S~% ->~%~
                              host:~S device:~S dir:~S version:~S"
-                       (pathname-host p) (pathname-device p)
-                       (pathname-directory p) (pathname-version p)
-                       (pathname-host new) (pathname-device new)
-                       (pathname-directory new) (pathname-version new)))))
-        (print-not-readable ()
-          nil)))))
+                     (pathname-host p) (pathname-device p)
+                     (pathname-directory p) (pathname-version p)
+                     (pathname-host new) (pathname-device new)
+                     (pathname-directory new) (pathname-version new)))))
+      (print-not-readable ()
+        nil))))
 
 ;;; BUG 330: "PARSE-NAMESTRING should accept namestrings as the
 ;;; default argument" ...and streams as well
-(with-test (:name :parse-namestring/stream)
+(with-test (:name (parse-namestring stream))
   (assert (equal (parse-namestring "foo" nil "/")
                  (parse-namestring "foo" nil #P"/")))
   (let ((test "parse-namestring-test.tmp"))
@@ -374,13 +489,46 @@
 
 ;;; ENOUGH-NAMESTRING should probably not fail when the namestring in
 ;;; question has a :RELATIVE pathname.
-(with-test (:name :enough-namestring)
+(with-test (:name enough-namestring)
   (assert (equal (enough-namestring #p"foo" #p"./") "foo")))
 
+;;;; NAMESTRING
+
 ;;; bug reported by Artem V. Andreev: :WILD not handled in unparsing
-;;; directory lists.
-(with-test (:name :unparse-wild)
-  (assert (equal (namestring #p"/tmp/*/") "/tmp/*/")))
+;;; directory lists. lp#1738775, reported by Richard M. Kreuter, added
+;;; more cases.
+(with-test (:name (namestring :wild :wild-inferiors :up :lp-1738775))
+  (flet ((test (expected pathname)
+           (assert (equal (namestring pathname) expected))))
+    ;; The second variant using MAKE-PATHNAME makes sure we don't just
+    ;; return the original namestring.
+    (test "/tmp/*/" #P"/tmp/*/")
+    (test "/tmp/*/" (make-pathname :directory '(:absolute "tmp" :wild)))
+
+    ;; lp#1738775 reported breakage in case FIRST in (:absolute FIRST
+    ;; ...) is not of type STRING.
+    (test "/*/" #P"/*/")
+    (test "/*/" (make-pathname :directory '(:absolute :wild)))
+
+    (test "/**/" #P"/**/")
+    (test "/**/" (make-pathname :directory '(:absolute :wild-inferiors)))
+
+    ;; FIXME "Invalid combinations" in 19.2.2.4.3 requires :ABSOLUTE
+    ;; :UP to signal a FILE-ERROR, but we don't.
+    (test "/../" #P"/../")
+    (test "/../" (make-pathname :directory '(:absolute :up)))))
+
+(with-test (:name (namestring :escape-pattern-pieces))
+  (labels ((prepare (namestring)
+             #-win32 (substitute #\\ #\E namestring)
+             #+win32 (substitute #\^ #\E namestring))
+           (test (expected namestring)
+             (let ((pathname (pathname (prepare namestring))))
+               (assert (string= (prepare expected) (namestring pathname))))))
+    #-win32 (test "*E?"    "*E?")
+    #-win32 (test "*E*"    "*E*")
+            (test "*E[ab]" "*E[ab]")
+            (test "*EE"    "*EE")))
 
 ;;; Printing of pathnames; see CLHS 22.1.3.1. This section was started
 ;;; to confirm that pathnames are printed as their namestrings under
@@ -409,11 +557,20 @@
 
 ;;; we failed to unparse logical pathnames with :NAME :WILD :TYPE NIL.
 ;;; (Reported by Pascal Bourguignon.
-(with-test (:name :unparse-logical-wild)
+(with-test (:name (namestring :unparse-logical-wild))
   (let ((pathname (make-pathname :host "SYS" :directory '(:absolute :wild-inferiors)
                                  :name :wild :type nil)))
     (assert (string= (namestring pathname) "SYS:**;*"))
     (assert (string= (write-to-string pathname :readably t) "#P\"SYS:**;*\""))))
+
+(with-test (:name (namestring :signals file-error))
+  (flet ((test (&rest initargs)
+           (let ((pathname (apply #'make-pathname initargs)))
+             (assert-error (namestring pathname) file-error))))
+    (test :name "foo.")
+    (test :name "")
+    (test :type "foo")
+    (test :name "foo" :type ".bar")))
 
 ;;; reported by James Y Knight on sbcl-devel 2006-05-17
 (with-test (:name :merge-back)
@@ -422,9 +579,19 @@
     (assert (equal (merge-pathnames p1 p2)
                    (make-pathname :directory '(:relative :back "foo" "bar"))))))
 
+(with-test (:name (native-namestring :signals file-error :unix))
+  (flet ((test (&rest initargs)
+           (let ((pathname (apply #'make-pathname initargs)))
+             (assert-error (native-namestring pathname) file-error))))
+    (test :directory '(:absolute (:home "no-such-user")))
+    (test :directory '(:absolute :wild))
+    (test :name :wild)
+    (test :name "foo" :type :wild)
+    (test :type "bar")))
+
 ;;; construct native namestrings even if the directory is empty (means
 ;;; that same as if (:relative))
-(with-test (:name :native-namestring)
+(with-test (:name (sb-ext:native-namestring 1))
   (assert (equal (sb-ext:native-namestring (make-pathname :directory '(:relative)
                                                           :name "foo"
                                                           :type "txt"))
@@ -484,7 +651,7 @@
 ;;; non-NIL strings in NATIVE-NAMESTRING implementations.  Revised by
 ;;; RMK 2007-11-28, attempting to preserve the apparent intended
 ;;; denotation of SBCL's then-current pathname implementation.
-(with-test (:name (:native-namestring 2))
+(with-test (:name (sb-ext:native-namestring 2))
   (assert (equal
            (loop with components = (list nil :unspecific "" "a")
                  for name in components
@@ -511,7 +678,7 @@
   #|""         |#     "C:\\tmp\\"   "C:\\tmp\\"   "C:\\tmp\\."  "C:\\tmp\\.a"
   #|"a"        |#     "C:\\tmp\\a"  "C:\\tmp\\a"  "C:\\tmp\\a." "C:\\tmp\\a.a"))))
 
-(with-test (:name :delete-file-logical-pathname)
+(with-test (:name (delete-file logical-pathname))
   (setf (logical-pathname-translations "SB-TEST")
         (list (list "**;*.*.*" (make-pathname :name :wild
                                               :type :wild
@@ -524,18 +691,10 @@
     (assert (delete-file test))
     (assert (not (probe-file test)))))
 
-(with-test (:name :logical-pathname-type-error)
-  (assert (eq :type-error-ok
-              (handler-case (logical-pathname "FOO.txt")
-                (type-error () :type-error-ok))))
-  (assert (eq :type-error-ok
-              (handler-case (logical-pathname "SYS:%")
-                (type-error () :type-error-ok)))))
-
 ;;; Reported by Willem Broekema: Reading #p"\\\\" caused an error due
 ;;; to insufficient sanity in input testing in EXTRACT-DEVICE (in
 ;;; src;code;win32-pathname).
-(with-test (:name :bug-489698 :skipped-on '(not :win32))
+(with-test (:name :bug-489698 :skipped-on (not :win32))
   (assert (equal (make-pathname :directory '(:absolute))
                  (read-from-string "#p\"\\\\\\\\\""))))
 
@@ -589,20 +748,25 @@
 
 ;;; lp#673625
 (with-test (:name :pathname-escape-first-directory-component
-                  :fails-on :win32)
+            :fails-on :win32)
   ;; ~ / :HOME
-  (assert (equal (pathname-directory #p"\\~/foo/") '(:relative "~" "foo")))
-  (assert (equal (native-namestring #p"\\~/foo/") "~/foo/"))
+  (assert (equal (pathname-directory #-win32 #p"\\~/foo/"
+                                     #+win32 #p"^~/foo/")
+                 '(:relative "~" "foo")))
+  (assert (equal (native-namestring #-win32 #p"\\~/foo/"
+                                    #+win32 #p"^~/foo/")
+                 #-win32 "~/foo/"
+                 #+win32 "~\\foo\\"))
   (assert (equal (namestring (make-pathname :directory '(:absolute "~zot")))
                  "\\~zot/"))
   ;; * / :WILD
   (assert (equal (pathname-directory #p"\\*/") '(:relative "*"))))
 
-(with-test (:name :ensure-directories-exist-with-odd-d-p-d)
+(with-test (:name (ensure-directories-exist :with-odd-d-p-d))
   (let ((*default-pathname-defaults* #p"/tmp/foo"))
     (ensure-directories-exist "/")))
 
-(with-test (:name :long-file-name :skipped-on '(not :win32))
+(with-test (:name :long-file-name :skipped-on (not :win32))
   (let* ((x '("hint--if-you-are-having-trouble-deleting-this-test-directory"
               "use-the-7zip-file-manager"))
          (base (truename
@@ -624,14 +788,14 @@
     (assert (not (probe-file shallow)))))
 
 #+unix
-(with-test (:name :simplify-namestring)
+(with-test (:name sb-int:simplify-namestring)
   (assert (string= (sb-int:simplify-namestring "./a/b/../c/")
                    "a/c/")))
 
 (with-test (:name :back-and-truename)
   (probe-file (make-pathname :directory '(:absolute "a" "b" :back))))
 
-(with-test (:name (:parse-namestring :displaced))
+(with-test (:name (parse-namestring :displaced))
   (let* ((string "abc")
          (disp (make-array 0 :element-type (array-element-type string)
                              :displaced-to string
@@ -641,7 +805,7 @@
       (assert (equal path #P""))
       (assert (zerop pos)))))
 
-(with-test (:name (:parse-native-namestring :displaced))
+(with-test (:name (sb-ext:parse-native-namestring :displaced))
   (let* ((string "abc")
          (disp (make-array 0 :element-type (array-element-type string)
                              :displaced-to string
